/* 
    channel for Bt848 frame grabber driver

    Copyright (C) 1996,97 Marcus Metzler (mocm@metzlerbros.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * channel.cc 
 */

#include "channel.hh"
#include <string.h>

gChannel::gChannel(const Channel & chan, const Transponder & trans,
		   const Lnb & ln)
{
	freq = trans.freq / 1000;
	strncpy(name, chan.name, mnamelen);
	name[mnamelen] = '\0';
	num = -1;
	color = 255;
	brightness = 0;
	hue = 0;
	contrast = 255;
	norm = -1;
	input = 0;
	pol = trans.pol + Vpol;
	sr = trans.srate / 1000;
	fec = trans.fec;
	v_pid = chan.vpid;
	a_pid = chan.apids[0];
	lnb = ln.diseqcnr;
	type = 1 + Analog;
	dvb_num = chan.pnr;
	cout << dvb_num << endl;
}

ChannelList::ChannelList()
{
	channels = new gChannel[channelinc];
	numchan = 0;
	maxnumc = channelinc;
}

ChannelList::ChannelList(char *filen)
{
	init(filen);
}

void ChannelList::init(char *filen)
{
	channels = new gChannel[channelinc];
	numchan = 0;
	maxnumc = channelinc;
	ifstream fin(filen);
	char dummy[128];

	if (numchan > 0)
		delete[]channels;

	if (!fin) {
		cerr << "Can`t open " << filen << " for Channel List" <<
		    endl;
		/*
		   cerr << "Using default channel and writing "<< filen << endl;
		   gChannel ch(9460,"Test 1",0);
		   ch.Freq()=12552;
		   ch.Input()=0;
		   ch.Color()=255;
		   ch.Brightness()=0;
		   ch.Hue()=0;
		   ch.Contrast()=255;
		   ch.Norm()=VIDEO_MODE_PAL;
		   ch.Type()=Analog;
		   AddChannel(ch);
		   ch.Freq()=12552;
		   ch.SR()=22000;
		   ch.FEC()=AUTO;
		   ch.LNB()=0;
		   ch.Type()=DVBS;
		   ch.V_pid()=163;
		   ch.A_pid()=104;
		   ch.Pol()=Vpol;
		   ofstream fout(filen);
		   fout << ch << endl;
		   ch.SetName("Test 2");
		   ch.Norm()=-1;
		   AddChannel(ch);
		   fout << ch << endl;
		 */
	} else {
		while (fin.peek() == '#') {
			fin.getline(dummy, 128);
		}
		while (fin.ignore(1000, '*') != NULL) {
			gChannel ch;
			fin >> ch;
			AddChannel(ch);
		}
	}
}

void ChannelList::AddChannel(gChannel & chan)
{
	numchan++;
	if (numchan > maxnumc) {
		maxnumc += channelinc;
		gChannel *ch = new gChannel[maxnumc];
		for (int i = 0; i < numchan - 1; i++) {
			ch[i] = channels[i];
		}
		delete[]channels;
		channels = ch;
	}
	channels[numchan - 1] = chan;
	channels[numchan - 1].Num() = numchan - 1;
}

void ChannelList::DelChannel(char *name)
{
	int i;
	for (i = 0; !strcmp(channels[i].Name(), name); i++);
	DelChannel(i);
}

void ChannelList::DelChannel(char *name, int freq)
{
}

void ChannelList::DelChannel(int nb)
{
	for (int i = nb; i < numchan - 1; i++)
		channels[i] = channels[i + 1];
	numchan--;
}


ostream & operator << (ostream & stream, gChannel & x) {
	stream << "*" << endl;
	stream << "Channel: " << x.name << endl;
	stream << "Frequency: " << x.freq << endl;
	stream << "CBHC: " << x.color << " ";
	stream << x.brightness << " ";
	stream << x.hue << " ";
	stream << x.contrast << endl;
	stream << "NI: " << x.norm << " ";
	stream << x.input << endl;
	if (x.norm == -1) {
		stream << "SAT: " << x.pol -
		    Vpol << " " << x.sr << " " << x.fec << " " << x.
		    v_pid << " " << x.a_pid << " " << x.lnb << " " << x.
		    type - Analog << " " << x.dvb_num << endl;
	}
	return stream;
}

istream & operator >> (istream & stream, gChannel & x) {
	char cdummy[mnamelen + 2];
	stream >> cdummy;
	stream.get(cdummy, mnamelen + 2);	// get full channel name
	x.SetName(cdummy + 1);

	stream >> cdummy >> x.freq >> cdummy >> x.color >> x.brightness;
	stream >> x.hue >> x.contrast >> cdummy >> x.norm >> x.input;
	if (x.norm == -1) {
		stream >> cdummy >> x.pol >> x.sr >> x.fec >> x.v_pid >> x.
		    a_pid >> x.lnb >> x.type >> x.dvb_num;
		x.pol += Vpol;
		x.type += Analog;
	}
	return stream;
}

ostream & operator << (ostream & stream, ChannelList & x) {
	for (int i = 0; i < x.numchan; i++) {
		stream << x.channels[i] << endl;
	}
	return stream;
}
