/* 
    gtvscreen for v4l frame grabber driver

    Copyright (C) 1999 Marcus Metzler (mocm@metzlerbros.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __GTVSCREEN_H__
#define __GTVSCREEN_H__


#ifdef __cplusplus
extern "C" {
#endif				/* __cplusplus */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <X11/Xlib.h>
#include <X11/extensions/xf86dga.h>
#include <X11/IntrinsicP.h>
#include <X11/ObjectP.h>
#include <X11/StringDefs.h>
#include <X11/Xatom.h>
#include <gdk/gdktypes.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkmain.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkeventbox.h>
#ifndef NOCARD
#include "videodev.h"
#include "dvb_v4l.h"
#endif
	/* some defaults */
#define DCOL 254
#define DBRI 0
#define DHUE 0
#define DCONTRAST 216
#define DNORM  PAL
#define DIN COMP1
#define MAXCLIP 200
#define DEFAULT_WIDTH 400
#define DEFAULT_HEIGHT 300
#define FULLX 640
#define FULLY 480

typedef struct freqtype_ {
		uint fre;
		int sr;
		int lnb;
		int fec;
		int type;
		int pol;
		int apid;
		int vpid;
	} freqtype;



	enum {
		GTV_NOTHING,
		GTV_CAP,
		GTV_OVL,
		GTV_DEVICE,
		GTV_NORM,
		GTV_FREQ,
		GTV_INPUT,
		GTV_BRIGHT,
		GTV_COLOR,
		GTV_HUE,
		GTV_CONTRAST,
		GTV_BPP,
		GTV_VISNAME,
		GTV_AUDIO,
		GTV_REALW,
		GTV_REMOTE,
		GTV_NINPUT,
		GTV_NAUDIO,
		GTV_FULLS,
		GTV_FD,
		GTV_EXT_SW,
		GTV_PLAY,
		GTV_RECORD,
		GTV_STOP,
		GTV_PAUSE,
		GTV_FFWD,
		GTV_FBWD,
		GTV_FSKP,
		GTV_BSKP,
		GTV_APID,
		GTV_MAXX,
		GTV_MAXY,
		GTV_TUNE,
		GTV_COVL,
		GTV_DVB,
		GTV_MPGD,
		GTV_MPGE
	};

	enum { F1_2, F2_3, F3_4, F4_5, F5_6, F6_7, F7_8, F8_9, AUTO, Vpol,
		Hpol, Analog, DVBS, DVBT, DVBC, NOTSET, TEND
	};
	extern char *combost[];

#define GTVSCREEN(obj)  GTK_CHECK_CAST (obj, gtvscreen_get_type (), Gtvscreen)
#define GTVSCREEN_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gtvscreen_get_type (), GtvscreenClass)
#define IS_GTVSCREEN(obj)       GTK_CHECK_TYPE (obj, gtvscreen_get_type ())


	typedef struct _Gtvscreen Gtvscreen;
	typedef struct _GtvscreenClass GtvscreenClass;

	struct _Gtvscreen {
		GtkWidget widget;
		/* resources */
		gchar devname[20];
		int CAP_ON;
		int OVL_ON;
		int pause;
		int dummy;	// there is some kind of wrong memory access
		int fd1;
		int fd2;
		int PLY_ON;
		int REC_ON;
		freqtype FREQ;
		int bpp;
		char *visname;
		int realw;
		int dvb;
#ifndef NOCARD
		struct video_window vwin;
		struct video_buffer vbuf;
		struct video_tuner vtuner;
		struct video_clip cliprecs[MAXCLIP];
		unsigned char *TargetMem;
		struct video_mbuf mbuf;
		struct video_play_mode pmode;
#endif
		/* private state */
		struct video_picture vp;
		struct video_channel vc;
		struct video_audio va;
		struct video_capability vcap;
		int fbttv;
		Boolean Change;
		GdkVisual *visual;
		GdkColor colors[256];
		GdkColormap *cmap;
		Boolean CMSET;
		int audio_state;
		unsigned int Format;
		GdkImage *image;
		GdkGC *gc;
		Boolean start;
		Boolean stop;
		Boolean remote;
		GtkAccelGroup *accel_group;
	        gint timeout;
	        gint refresh;
#ifdef HAVE_LIBXXF86VM
		int next_vid;
		int isfull;
#endif
	};

	struct _GtvscreenClass {
		GtkVBoxClass parent_class;

		void (*chinput) (Gtvscreen * gtv);
		void (*chcapture) (Gtvscreen * gtv);
		void (*choverlay) (Gtvscreen * gtv);
		void (*chattrib) (Gtvscreen * gtv);
		void (*chfreq) (Gtvscreen * gtv);
		void (*chaudio) (Gtvscreen * gtv);
		void (*chplay) (Gtvscreen * gtv);
		void (*chstop) (Gtvscreen * gtv);
		void (*chpause) (Gtvscreen * gtv);
		void (*chrecord) (Gtvscreen * gtv);
		void (*chffwd) (Gtvscreen * gtv);
		void (*chfbwd) (Gtvscreen * gtv);
		void (*chfskp) (Gtvscreen * gtv);
		void (*chbskp) (Gtvscreen * gtv);

	};

	guint gtvscreen_get_type(void);
	GtkWidget *gtvscreen_new(void);
	GtkWidget *gtvscreen_new_with_dev(gchar *dev);
	GtkWidget *gtvscreen_new_with_fd(int fd);
	Boolean gtvscreen_set_values(Gtvscreen * gtv, int val_type,
				     gpointer value);
	Boolean gtvscreen_get_values(Gtvscreen * gtv, int val_type,
				     gint * value);
	void gtk_gtvscreen_size(Gtvscreen * gtv, gint width, gint height);


#ifdef __cplusplus
}
#endif				/* __cplusplus */
#endif				/* __GTVSCREEN_H__ */
