/*
 * avpes2any tools for the Siemens Fujitsu DVB PCI card
 *
 * Copyright (C) 2000 Marcus Metzler for convergence integrated media GmbH
 * 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 * 

 * The author can be reached at marcus@convergence.de, 

 * the project's page is at http://linuxtv.org/dvb/
 */


#include "cpptools.hh"

const int SIZE = 1024;
int fda_out, fdv_out;

void awrite(u8 const *buf,long int l)
{
	write(fda_out,buf,l);
}

void vwrite(u8 const *buf,long int l)
{
	write(fdv_out,buf,l);
}


int main(int argc, char **argv)
{
	strstream str;
	ifstream fin;
	int fd;
		
	if(argc <=2){
		if (argc == 2){
			fd = open(argv[1],O_RDONLY);
			fin.attach(fd);
			if (fin)
				cerr << "Reading " << argv[1] << endl;
			else
				exit(1);
		} else {
		 	fin.attach(0);
			fd = STDIN_FILENO;
			if (fin)
				cerr << "Reading from stdin" << endl;
			else 
				exit(1);
		}
		
		if (!strcmp(basename(argv[0]), "streamtype")){
			char *streams[4] = {"AV_PES_STREAM", 
						    "PS_STREAM", "TS_STREAM"
						    , "PES_STREAM"};
			char *tv_form[3] = {"DUNNO", "PAL", "NTSC"};
			
			cout << streams[stream_type(fin)] << endl;
			int f=tv_norm(fin);
			cout << tv_form[int(f&0x0F)] << endl;
			cout << "MPEG" << int((f>>4)&0x0F) << endl;
			
		}
		
		if (!strcmp(basename(argv[0]), "pes2vid"))
			extract_video_from_PES(fin,cout);

		if (!strcmp(basename(argv[0]), "pes2aud"))
			extract_audio_from_PES(fin,cout);

		if (!strcmp(basename(argv[0]), "pes2vid_es"))
			extract_es_video_from_PES(fin,cout);

		if (!strcmp(basename(argv[0]), "pes2aud_es"))
			extract_es_audio_from_PES(fin,cout);

		if (!strcmp(basename(argv[0]), "av_pes2vid_es"))
			extract_video_from_AV_PES(fin,cout);
		

		if (!strcmp(basename(argv[0]), "av_pes2aud"))
			extract_audio_from_AV_PES(fin,cout);
		
		if (!strcmp(basename(argv[0]), "pes2av_pes") ||
		    !strcmp(basename(argv[0]), "ps2av_pes")){
			u8 buf[SIZE];
			p2a p;
			init_p2a(&p);
			int count = 1;
			while (count > 0){
				count = read(fd,buf,SIZE);
				kpes_to_av_pes(buf, count,&p);
			}
		}

		if (!strcmp(basename(argv[0]), "es2av_pes")){
			u8 buf[MAX_AV];
			int count = 1;
			while (count > 0){
				count = read(fd,buf,MAX_AV);
				es_to_av_pes(buf, count);
			}
		}

		if (!strcmp(basename(argv[0]), "av_pes2pes")){
			a2p p;
			u8 buf[SIZE];
			
			init_a2p(&p);
			int count = 1;
			while (count > 0){
				count = read(fd,buf,SIZE);
				av_pes_to_pes(buf,count, &p);
			} 
		}

		if (!strcmp(basename(argv[0]), "av_pes2ps")){
			a2p p;
			u8 buf[SIZE];
			
			init_a2p(&p);
			p.type = PS_STREAM;
			int count = 1;
			while (count > 0){
				count = read(fd,buf,SIZE);
				av_pes_to_pes(buf,count, &p);
			} 
		}

		if (!strcmp(basename(argv[0]), "pes2ts")){
			p2t_t p;
			u8 buf[SIZE];
			
			init_p2t(&p,NULL);
			int count = 1;
			while (count > 0){
				count = read(fd,buf,SIZE);
				pes_to_ts(buf, count, 160, &p);
			} 
		}

		if (!strcmp(basename(argv[0]), "av_pes2ts")){
			a2p p;
			u8 buf[SIZE];
			
			init_a2p(&p);
			p.type = TS_STREAM;
			int count = 1;
			while (count > 0){
				count = read(fd,buf,SIZE);
				av_pes_to_pes(buf,count, &p);
			} 
		}

		if (!strcmp(basename(argv[0]), "ts2av_pes")){
			ts_av_pes2(fd);
		}

		if (!strcmp(basename(argv[0]), "ts_demux")){
			cerr << "Opening stderr for audio" << endl;
			cerr << "Opening stdout for video" << endl;
			
			ts_demux2(fd,STDOUT_FILENO,STDERR_FILENO);
		}


		if (!strcmp(basename(argv[0]), "tspids"))
			TS_PIDS(fin,cout);

		if (!strcmp(basename(argv[0]), "analyze"))
			analyze(fin);
		      
		if (!strcmp(basename(argv[0]), "check_av_pes")){
			
			AV_PES_Packet aps;
			while (!fin.eof()){
				aps.init();
				fin >> aps;
				cout << aps;
				aps.Info() = 1;
				cerr << aps;
			}
		}
	
		
		if (!strcmp(basename(argv[0]), "pesplot")){
			PES_Packet pes;
			long int apts=0;
			long int vpts=0;

			while(!fin.eof()){
				streampos pos=fin.tellg();
				fin >> pes;
				if(pes.has_pts()){
					switch(pes.Stream_ID()){
					case AUDIO_STREAM_S ... AUDIO_STREAM_E:
						apts = ntohl(*((unsigned int *)
							       pes.P()->pts));
						break;
					case VIDEO_STREAM_S ... VIDEO_STREAM_E:
						vpts = ntohl(*((unsigned int *)
							       pes.P()->pts));
						break;
					}
					if (apts && vpts)
						cout << pos << " " 
						     << apts << " " << vpts
						     << endl;
				}
			}
		}
	}

	
	if (argc == 3 || argc == 4){
		int i=0;
		if (argc == 4){
			fd = open(argv[1],O_RDONLY);
			fin.attach(fd);
			if (fin)
				cerr << "Reading " << argv[1] << endl;
			else
				exit(1);
		} else {
			i=1;
		 	fin.attach(0);
			fd = STDIN_FILENO;
			if (fin)
				cerr << "Reading from stdin" << endl;
			else 
				exit(1);
		}
		

		if (!strcmp(basename(argv[0]), "remux")){
			remux(fd,STDOUT_FILENO,atoi(argv[2-i]),
			      atoi(argv[3-i]));
		}
		      
		if (!strcmp(basename(argv[0]), "ts2pes"))
			TS_to_PES(fin,cout,atoi(argv[2-i]), atoi(argv[3-i]));

		if (!strcmp(basename(argv[0]), "pes_demux")){
			fda_out = open(argv[2],O_WRONLY|O_CREAT);
			fdv_out = open(argv[3],O_WRONLY|O_CREAT);
			if(fda_out < 0 || fdv_out < 0){
				cerr << "Error opening output files" << endl;
				exit(1);
			}
			cerr << "Opening " << argv[2] << " for audio" << endl;
			cerr << "Opening " << argv[3] << " for video" << endl;

			dmx_t p;
			init_dmx(&p,awrite,vwrite);
			u8 buf[SIZE];
			
			int count = 1;
			while (count > 0){
				count = read(fd,buf,SIZE);
				dmx_pes(buf,count, &p);
			} 
		}

		if (!strcmp(basename(argv[0]), "ts_demux")){
			fda_out = open(argv[2],O_WRONLY|O_CREAT);
			fdv_out = open(argv[3],O_WRONLY|O_CREAT);
			if(fda_out < 0 || fdv_out < 0){
				cerr << "Error opening output files" << endl;
				exit(1);
			}
			cerr << "Opening " << argv[2] << " for audio" << endl;
			cerr << "Opening " << argv[3] << " for video" << endl;
			
			ts_demux2(fd,fdv_out,fda_out);
		}

		if (!strcmp(basename(argv[0]), "es_demux")){
			int count;
			PES_Packet pes;

			fda_out = open(argv[2],O_WRONLY|O_CREAT);
			fdv_out = open(argv[3],O_WRONLY|O_CREAT);

			if(fda_out < 0 || fdv_out < 0){
				cerr << "Error opening output files" << endl;
				exit(1);
			}
			cerr << "Opening " << argv[2] << " for audio" << endl;
			cerr << "Opening " << argv[3] << " for video" << endl;


			int afound = 0;
			int found = 0;
			int c;
			int ca;
			u8 *buf;
			int alength = 0;

			u8 bufa[MAX_PLENGTH];
			while (found < 4){
				pes.init();
				fin >> pes;
				count = pes.Length();
				buf = pes.Data();
				c=0;
				ca=0;

				switch (pes.Stream_ID()){
				case VIDEO_STREAM_S ... VIDEO_STREAM_E:
					while ( pes.Stream_ID() == 0xE0 && 
						c < count && found < 4){
						switch(found){
						case 0:
						case 1:
							if (buf[c] == 0x00)
								found++;
							else if(found) found--;
							break;
						case 2:
							if (buf[c] == 0x01)
								found++;
							else found = 0;
							if (buf[c] == 0x00) 
								found=2;
							break;
							
						case 3:
							if (buf[c] == 0xb3)
								found++;
							else found = 0;
							break;
						}
						c++;
					}
					break;
					
				case AUDIO_STREAM_S ... AUDIO_STREAM_E:
					memcpy(bufa,buf,count);
					alength = count;
					afound = 1;
					break;
				}
			}
			if (found==4){
				write(fdv_out,buf+c-4,count-c+4);
				/*
				if (afound){
					found = 0;
					c = 0;
 					while (found<2 && c < alength){
						switch(found){
						case 0:
							if (bufa[c] == 0xff)
								found ++;
							break;
						case 1:
							if ((bufa[c] & 0xf0)
							    == 0xf0)
								found ++;
							else found = 0;
							break;
						}
						c++;
					}
					if (found == 2){
						write(fda_out,bufa+c-2,
						      count-c+2);
					}
				}
				*/
			} else exit(0);

			while(!fin.eof()){
				pes.init();
				fin >> pes;
				if(pes.Stream_ID() == 0xE0)
					write(fdv_out,pes.Data(),pes.Length());
				if(pes.Stream_ID() == 0xC0)
					write(fda_out,pes.Data(),pes.Length());

			}
		}


// FOR TESTS ONLY
		if (!strcmp(basename(argv[0]), "av_pes2any")){

			int time = atoi(argv[2]);
			int times = atoi(argv[3]);
			char *frame;
			
			frame = new char [1024*128];
			memset(frame,0,1024*128);
			int r = 0;
			int c = 1;
			while (c > 0 && r < 1023*127){
			   c = read(fd, frame+r,1024);
			   if (c>0) r += c;
			}
			cerr << "Read " << r << " bytes" << endl;
			cerr << "Writing " << times << " times" << endl;
			for (int i = 0 ; i < times; i++)
			  write(STDOUT_FILENO,frame,r);
			sleep(time);
			delete [] frame;

				/*
			int sframe = atoi(argv[2]);
			int eframe = atoi(argv[2]);
			PS_Packet ps;
			PES_Packet pes;
			int fc = 0;
			
			cerr << "cutting from frame" << sframe << " to "
			     << eframe << endl;
			streampos c1 = fin.tellg();
			streampos c2 = fin.tellg();
			streampos c3 = fin.tellg();

			u8 buf[1024*1024];
			while(fin){
				  u32 pts;
				fin >> ps;
				cerr << "SCR base: " << hex << setw(5) 
				     << setfill(int('0')) \
				     << ps.SCR_base() << " " << dec
				     << "ext : " << ps.SCR_ext() << endl;
				int w=0;
				for (int i=0; i < ps.NPES(); i++){
					pes.init();
					fin >> pes;
					pts2pts((u8 *)&pts,pes.PTS());
						  cerr  << "PTS : 0x" << hex << setw(5) 
						  << setfill(int('0')) \
						  << ntohl(pts) << " " << dec;
						  cerr << endl;
					u8 *buf = pes.P()->pes_pckt_data;
					int c = 0;
					int l=pes.P()->length;
					int j;

				streampos p = fin.tellg();
				fin.read(buf,4);
				if (buf[0] == 0x00 && 
				    buf[1] == 0x00 &&
				    buf[2] == 0x01 && 
				    buf[3] == 0xB8) {
						c1 = p;
						Cerr << "hours: " 
						     << int((buf[c]>>2)&
							    0x1F)
						     << "  minutes: "
						     << int(((buf[c]<<4)&
							     0x30)|
							    ((buf[c+1]>>4)&
							     0x0F))
						     << " seconds: "
						     << int(((buf[c+1]<<3)&
								     0x38)|
							    ((buf[c+2]>>5)&
							     0x0F))
						     << endl;
				} else fin.seekg(p);
				p = fin.tellg();
				fin.read(buf,6);
				
				if ( buf[0] == 0x00 && 
				     buf[1] == 0x00 &&
				     buf[2] == 0x01 && 
				     buf[3] == 0x00) {
					fc++;
					cerr << "picture: " 
					     << fc 
					     << " ("
					     << ((buf[5]&0x38) >>3)
					     << ")"
					     << endl;
					
					if ((buf[5] & 0x38) 
					    == 0x08){
						cerr << "I-frame" 
						     << endl;
						c2 = p; 
					} else {
						c3 = p;
					}
				} else fin.seekg(p+1);
				if (c3 > c1 && c2 > c1 && c3 > c2){
					fin.seekg(c1);
					int ll = c3 - c1;
					fin.read(buf,ll);
					buf[ll] = 0x00;
					buf[ll+1] = 0x01;
					buf[ll+2] = 0x00;
					buf[ll+3] = 0x00;
//					cout.write(buf,ll);
					for (int t=0; t< 100;t++){
						cout.write(buf,ll);
					}
					usleep(10000);
					exit(0);
				}
				
				
			}

			  cerr << "MUX rate: " << ps.MUX()*50*8/1000000.0
			  << " Mbit/s" << endl;
			  cerr << "RATE bound: " << ps.Rate()*50*8/1000000.0
			  << " Mbit/s" << endl;
			  cerr << "Audio bound: " << hex << "0x" 
			  << int(ps.P()->audio_bound)
			  << endl;
			  cerr << "Video bound: " << hex << "0x" 
			  << int(ps.P()->video_bound)
			  << dec
			  << endl;
			  
			  cerr << endl;
		}
*/
		}
		
		
	}
		

}
