#include <strstream.h>
#include "ntuxzap.hh"
#include "interface.hh"

#ifdef HAVE_XCURSES
char *XCursesProgramName="alphalist_ex";
#endif

/*
 * This program demonstrates the Cdk alphalist widget.
 */
#define	MAXINFOLINES	10000
#define NLIST 3

DVB dvb;
ChannelList chlist[NLIST];
CDKSCREEN *cdkscreen		= (CDKSCREEN *)NULL;
int cchan = -1;
int it = 0;
int oit = 1;


int get_dvbrc(char *path,DVB &dv)
{
        ifstream dvbin;

        const char *home = getenv("HOME");
        const char *file = ".dvbrc";

        strncpy(path, home, STRSIZE);
        strncat(path, "/", STRSIZE);
        strncat(path, file, STRSIZE);
        dvbin.open(path);
        if (dvbin) {
                dvbin >> dv;
                return 1;
        }

        return 0;
}

static void get_dvbfav(char *path, int nb, int len)
{
        const char *home = getenv("HOME");
        const char *file = ".dvbfav";

	ostrstream str(path,len);
	
	str << home << "/" << file << nb << ends; 
}

static void load_ch(int nb)
{
	char filen[80];

	get_dvbfav(filen,nb,80);
	cout << _("Reading channel list from ") << filen << endl;
	chlist[nb].init(filen);
}

static int cnum (gChannel & ch)
{
	char buf[80];
        ostrstream str(buf, 80);

        uint16_t onid  =  uint16_t((ch.DVB_Num() & 
				    0xFFFF000000000000ULL) >> 48);
	uint16_t satid =  uint16_t((ch.DVB_Num() & 
				    0x0000FFFF00000000ULL) >> 32);
        uint16_t tpid  =  uint16_t((ch.DVB_Num() & 
				    0x00000000FFFF0000ULL) >> 16);
	ushort sid     =  ushort   (ch.DVB_Num() & 0x000000000000FFFFULL);


	int chnr;
	int found=0;
	for (chnr=0; chnr< dvb.num[CHAN]; chnr++) {
		if (dvb.chans[chnr].pnr==sid   && 
		    (dvb.chans[chnr].onid==onid || onid == 0xffff) && 
		    (dvb.chans[chnr].satid==satid || satid == 0xffff)  && 
                    (dvb.chans[chnr].tpid==tpid)) {
		  found = 1;
		  break;
                }
	}
	if (!found) chnr = -1;
	return chnr;
}

int save_ch(EObjectType cdktype, void *object, void *data, chtype key)
{
	int nb = int(data);
	char filen[80];
	char buf[80], *mesg[10], temp[256];

	ostrstream str(buf, 80);

	get_dvbfav(filen,nb,80);
	
	mesg[0] = _("<C>Saving Channel List");
	sprintf (temp, _("<C>Favorite %d"), nb);
	mesg[1] = copyChar (temp);
	mesg[2] = "";
	mesg[3] = _("<C>Press any key to continue.");
	popupLabel (cdkscreen, mesg, 4);

	ofstream fout(filen);
	fout << _("# This is the list of favorite channels") << endl;
	fout << _("# The format is:") << endl;
	fout << _("#   *   = start of a channel declaration") << endl;
	fout << _("#  Channel    : <Name>") << endl;
	fout << _("#               <dvbnum> id of channel in DVB structure")
	    << endl;
	fout << chlist[nb] << endl;
	return 0;
}

int del_ch(EObjectType cdktype, void *object, void *data, chtype key)
{
	int nb = int(data);
	char *mesg[10], temp[256];


	sprintf (temp, _("<C>Deleting Channel %s from List")
		 ,dvb.chans[cchan].name) ;
	mesg[0] = copyChar(temp);
	sprintf (temp, _("<C>Favorite %d"), nb);
	mesg[1] = copyChar (temp);
	mesg[2] = _("(the change will occur after next selection)");
	mesg[3] = _("<C>Press any key to continue.");
	popupLabel (cdkscreen, mesg, 4);
	
	chlist[nb].DelChannel(dvb.chans[cchan].name);
	it = nb+1;
	oit = 0;

}

int add_ch(EObjectType cdktype, void *object, void *data, chtype key)
{
	char *in[NLIST];
	int chnr = cchan;

	if (cchan < 0) return -1;

	for (int i=0; i<NLIST; i++){
		strstream str;
		str << _("Favorite ") << i << ends;
		in[i] = copyChar(str.str());
	}
	strstream str;
	str << _("<C></B/24>Add channel ") << dvb.chans[cchan].name 
	    << _(" to list\n<C>") << ends;
	char *t = copyChar(str.str());
	char *l  = _("</B>List: ");
	
	CDKITEMLIST *ilist;

	ilist = newCDKItemlist(cdkscreen,LEFT,LEFT,
			       t,l,in,NLIST,0,TRUE,TRUE);
	int clistn = activateCDKItemlist (ilist, NULL);
	if(ilist->exitType == vNORMAL){
	        int k = 0, i = 0, j = 0;
		
		while (dvb.tps[k].id != dvb.chans[chnr].tpid)
			k++;
		while (dvb.tps[k].satid != dvb.sats[i].id)
			i++;
		while (dvb.sats[i].lnbid != dvb.lnbs[j].id)
			j++;
		cerr << chnr << endl;

		gChannel ch(dvb.chans[chnr], dvb.tps[k], dvb.lnbs[j]);
		chlist[clistn].AddChannel(ch);
		
	}
	destroyCDKItemlist(ilist);
	refreshCDKScreen(cdkscreen);
	return 0;
}

int getList (CDKALPHALIST *alphaList, int k)
{
	int num;
	char **info = NULL;
	int set = 0;
	int j;
	switch (k){
	case 0:
		num = dvb.num[CHAN];
		info = new (char *) [num];
		for (int i=0; i < num; i++){
			strstream str;
			str << dvb.chans[i].name << "  (" << i << ")"<< ends;
			info[i] = copyChar(str.str());
		}
		set = 1;
		break;
	case 1:
	case 2:
	case 3:
		num = chlist[k-1].NumChan();
		info = new (char *) [num];
		j = 0;
		for (int i = 0; i < num; i++) {
			strstream str;
			int cn = cnum(chlist[k-1](i));
			if (cn >= 0){
				str << chlist[k-1](i).Name() << " (" << cn 
				    << ")" << ends;
				info[j] = copyChar(str.str());
				j++;
			}
		}
		num = j;
		break;
	}
	if (alphaList && info) setCDKAlphalistContents(alphaList, info, num);
	if (k == 0){
		bindCDKObject(vALPHALIST,alphaList, KEY_F1, add_ch, NULL);
	} else {
		bindCDKObject(vALPHALIST,alphaList, KEY_F1, save_ch, 
			      (void *) (k-1));
		bindCDKObject(vALPHALIST,alphaList, KEY_F8, del_ch, 
			      (void *) (k-1));
	}

	delete(info);
}


int main (int argc, char **argv)
{
   /* Declare variables. */
	char filen[80];
	WINDOW *cursesWin		= (WINDOW *)NULL;
	char *word			= (char *)NULL;
	char  *mesg[10], temp[256];
	CDKALPHALIST *alphaList	= (CDKALPHALIST *)NULL;
	char *title			= "<C></B/24>nTuxZap\n<C>";
	char *label			= _("</B>Channel: ");
	int c,channel = 0,dev = 0;

#ifdef ENABLE_NLS
	setlocale(LC_ALL,"");
	bindtextdomain("ntuxzap", LOCALE_DIR);
	textdomain("ntuxzap");
#endif

	for (;;) {
		if (-1 == (c = getopt(argc, argv, "d:c:")))
			break;
		switch (c) {
		case 'c':
			channel = atoi(optarg);
			break;
		case 'd':
			dev = atoi(optarg);
			break;

		case 'h':
		default:
			cerr << _("usage: ") << argv[0] << _("  [ options ] ") 
			     <<	endl << endl << _("options:") << endl
			     << endl << _("    -c <n>    set channel") << endl
			     << endl << _("    -d <n>    set devices (0 =/dev/ost/xxx0)") << endl;
			exit(1);
		}
	}
	
	dvb.init("","",dev);


	if (get_dvbrc(filen,dvb)) {
		cout << _("Found DVB input file ") << filen << endl;
	} else {
		cout << _("Could not find DVB input file ") 
		     << filen << endl;
	}

	for (int i = 0; i < NLIST; i++){
		char buf[80];
		ostrstream str1(buf, 80);
		
		str1 << _("Favorite ") << i << ends;
		load_ch(i);
	}

	
	/* Set up CDK. */ 
	cursesWin = initscr();
	cdkscreen = initCDKScreen (cursesWin);

	/* Start color. */
	initCDKColor();
	char *info[1];
	info[1] = copyChar(_("none"));
	alphaList = newCDKAlphalist (cdkscreen, CENTER, CENTER, 
				     0, 0, title, label,
				     info, 1,
				     '.', A_REVERSE, TRUE, FALSE);
	

	while (1){
		if (oit != it) {
			oit = it;
			getList (alphaList,it);
		}

		refreshCDKScreen(cdkscreen);
		word = activateCDKAlphalist (alphaList, NULL);

	/* Determine what the user did. */
		if (alphaList->exitType == vESCAPE_HIT)
		{
			char *in[NLIST+1];
			in[0] = copyChar(_("DVBList"));
			for (int i=0; i<NLIST; i++){
				strstream str;
				str << _("Favorite ") << i << ends;
				in[i+1] = copyChar(str.str());
			}
			char *t = _("<C></B/24>Choose List\n<C>");
			char *l  = _("</B>List: ");
		 
			CDKITEMLIST *ilist;

			ilist = newCDKItemlist(cdkscreen,LEFT,LEFT,
					       t,l,in,NLIST+1,0,TRUE,TRUE);
			int nit = activateCDKItemlist (ilist, NULL);
			if(ilist->exitType == vNORMAL){
				it = nit;
			}
			destroyCDKItemlist(ilist);
		}
		else if (alphaList->exitType == vNORMAL)
		{
			int i=-1;
			int l = strlen(word);
			l--;
			while (word[l] != '(' && l>0) l--;
			istrstream str(word+l+1);
			str >> i;
			cchan = i;
			if (cchan>= 0) dvb.SetChannel(cchan);
		}
	}
	/* Clean up. */
	destroyCDKAlphalist (alphaList);
	delwin (cursesWin);
	endCDK();
	exit (0);
}

