#ifndef _DVB_H_
#define _DVB_H_

#define DVB_FIRM_PATH "/lib/DVB/"

#include "i2c.h"

#ifdef __KERNEL__
#include "linux/socket.h" 
#include "linux/netdevice.h" 
#else
#include <netinet/in.h>
#endif

#if LINUX_VERSION_CODE > 0x020300
#include <linux/devfs_fs_kernel.h>
#endif

/* DEBI transfer mode defs */

#define DEBINOSWAP 0x4e0000
#define DEBISWAB   0x5e0000

#include "dvb_v4l.h"

#ifdef __KERNEL__

#if LINUX_VERSION_CODE < 0x020300
#define net_device device
#define DECLARE_MUTEX(foo)         struct semaphore foo = MUTEX
#define DECLARE_MUTEX_LOCKED(foo)  struct semaphore foo = MUTEX_LOCKED
#define WAIT_QUEUE                 struct wait_queue*
#define init_waitqueue_head(wq)    *(wq) = NULL;
#define DECLARE_WAITQUEUE(wait, current) struct wait_queue wait = { current, NULL }
#define set_current_state(state_value)			\
	do { current->state = state_value; } while (0)
#define list_for_each(pos, head) \
	for (pos = (head)->next; pos != (head); pos = pos->next)

#else

#define WAIT_QUEUE                 wait_queue_head_t

#endif

#include <ost/dvbdev.h>
#include <ost/video.h>
#include <ost/audio.h>
#include <ost/demux.h>
#include <ost/dmx.h>
#include <ost/sec.h>
#include <ost/frontend.h>
#include <ost/ca.h>

#include "dvb_demux.h"
#include "dmxdev.h"


#define DVB_SET_FRONTEND        _IOW('v',  BASE_VIDIOCPRIVATE+0x10, struct frontend)
#define DVB_GET_FRONTEND        _IOR('v',  BASE_VIDIOCPRIVATE+0x11, struct frontend)
#define DVB_INIT_FRONTEND       _IOR('v',  BASE_VIDIOCPRIVATE+0x12, void)
#define DVB_RESET               _IOR('v',  BASE_VIDIOCPRIVATE+0x13, void)
#define DVB_WRITEREG            _IOR('v',  BASE_VIDIOCPRIVATE+0x14, u8 *)
#define DVB_READREG             _IOR('v',  BASE_VIDIOCPRIVATE+0x15, u8 *)


#define DATA_TELETEXT            0x00 
#define DATA_FSECTION            0x01
#define DATA_IPMPE               0x02
#define DATA_MPEG_RECORD         0x03
#define DATA_DEBUG_MESSAGE       0x04
#define DATA_COMMON_INTERFACE    0x05
#define DATA_MPEG_PLAY           0x06
#define DATA_BMP_LOAD            0x07
#define DATA_IRCOMMAND           0x08
#define DATA_PIPING              0x09
#define DATA_STREAMING           0x0a

#define DATA_PES_RECORD          0x10
#define DATA_PES_PLAY            0x11


#define CI_CMD_ERROR             0x00
#define CI_CMD_ACK               0x01
#define CI_CMD_SYSTEM_READY      0x02
#define CI_CMD_KEYPRESS          0x03
#define CI_CMD_ON_TUNED          0x04
#define CI_CMD_ON_SWITCH_PROGRAM 0x05
#define CI_CMD_SECTION_ARRIVED   0x06
#define CI_CMD_SECTION_TIMEOUT   0x07
#define	CI_CMD_TIME              0x08
#define	CI_CMD_ENTER_MENU        0x09
#define	CI_CMD_FAST_PSI          0x0a
#define	CI_CMD_GET_SLOT_INFO     0x0b

#define	CI_MSG_NONE              0x00
#define	CI_MSG_CI_INFO           0x01
#define	CI_MSG_MENU              0x02
#define	CI_MSG_LIST              0x03
#define	CI_MSG_TEXT              0x04
#define	CI_MSG_REQUEST_INPUT     0x05
#define	CI_MSG_INPUT_COMPLETE    0x06
#define	CI_MSG_LIST_MORE         0x07
#define	CI_MSG_MENU_MORE         0x08
#define	CI_MSG_CLOSE_MMI_IMM     0x09
#define	CI_MSG_SECTION_REQUEST   0x0a
#define	CI_MSG_CLOSE_FILTER      0x0b
#define	CI_PSI_COMPLETE          0x0c
#define	CI_MODULE_READY          0x0d
#define	CI_SWITCH_PRG_REPLY      0x0e
#define	CI_MSG_TEXT_MORE         0x0f

#define	CI_MSG_CA_PMT            0xe0
#define	CI_MSG_ERROR             0xf0


#ifdef V4L2
#define capture_device dvb_struct

/*  Per-open data for handling multiple opens on one device */
struct device_open
{
	int			isopen;
	int			noncapturing;
	struct capture_device	*dev;
};
#define MAX_OPENS	3
#endif


typedef struct dvb_buf {
        int length;
        u8 *buf;
	spinlock_t lock;
	int head;
        int tail;
	WAIT_QUEUE queue;
} dvb_buf_t;


/*
    AV_PES
 */		

#define AV_VIDEO 0x01
#define AV_AUDIO 0x02
#define AV_PCM   0x03
#define AV_RES   0x55
	
#define PTS_FLAG      0x10
#define PRE_BYTE_MASK 0x03

#define MAX_AV  0x17f8
//#define MAX_AV  0x07f8
#define MAX_AVA 0x07f8

#define PROG_STREAM_MAP  0xBC
#define PRIVATE_STREAM1  0xBD
#define PADDING_STREAM   0xBE
#define PRIVATE_STREAM2  0xBF
#define AUDIO_STREAM_S   0xC0
#define AUDIO_STREAM_E   0xDF
#define VIDEO_STREAM_S   0xE0
#define VIDEO_STREAM_E   0xEF
#define ECM_STREAM       0xF0
#define EMM_STREAM       0xF1
#define DSM_CC_STREAM    0xF2
#define ISO13522_STREAM  0xF3
#define PROG_STREAM_DIR  0xFF

#define PTS_DTS_FLAGS    0xC0

//pts_dts flags 
#define PTS_ONLY         0x80
#define PTS_DTS          0xC0
#define TS_SIZE        188
#define TRANS_ERROR    0x80
#define PAY_START      0x40
#define TRANS_PRIO     0x20
#define PID_MASK_HI    0x1F
//flags
#define TRANS_SCRMBL1  0x80
#define TRANS_SCRMBL2  0x40
#define ADAPT_FIELD    0x20
#define PAYLOAD        0x10
#define COUNT_MASK     0x0F

// adaptation flags
#define DISCON_IND     0x80
#define RAND_ACC_IND   0x40
#define ES_PRI_IND     0x20
#define PCR_FLAG       0x10
#define OPCR_FLAG      0x08
#define SPLICE_FLAG    0x04
#define TRANS_PRIV     0x02
#define ADAP_EXT_FLAG  0x01

// adaptation extension flags
#define LTW_FLAG       0x80
#define PIECE_RATE     0x40
#define SEAM_SPLICE    0x20

#define MAX_PLENGTH    0xFFFF

	
typedef struct av_pes_packet_s{
        u8                     stream_id;
	u8                     counter;
	u8                     reserved;
	u8                     flags;
	u8                     llength[2];
	u8                     pts[4];
	short                  length;
	u8                     gap;
	u8                     payload[MAX_AV];
} av_pes_packet_t;

typedef struct p2a_s {
	int                    found;
	u8                     cid;
	av_pes_packet_t        apsv;
	av_pes_packet_t        apsa;
	av_pes_packet_t        apsp;
	unsigned short         plength;
	u8                     plen[2];
	u8                     flag1;
	u8                     flag2;
	u8                     hlength;
	u8                     pts[5];
	int                    mpeg;
	int                    video_format;
	int                    hsize;
	int                    vsize;
	int                    aspect;
	int                    framerate;
	u8                     check;
	int                    found2;
	int                    nonblock;
	int                    what;
        u8                     buf[MAX_AV+8];
	int                    done;
} p2a_t;



typedef struct section_s {
	int                    id;
	int                    length;
	int                    found;
	u8                     payload[4096+3];
} section_t;


#define MY_STATE_PES_START     1
#define MY_STATE_PES_STARTED   2
#define MY_STATE_FULL          4

#define MASKL   DMX_MAX_FILTER_SIZE
#define MAXFILT DVB_DEMUX_FILTER_MAX

typedef struct qpsk_s {
        struct qpskEvent        events[8];
        int                     eventw;
        int                     eventr;
        int                     overflow;
        WAIT_QUEUE              eventq;
	spinlock_t              eventlock;
} qpsk_t;

struct dvb_filter {
        int                    state;
        int                    flags;
        int                    type;
        u8                     ts_state;

        u16                    pid;
        u8                     value[MASKL];
        u8                     mask[MASKL];
};



typedef struct trans_s {
	dvb_demux_filter_t     *filter;//[MAXFILT];
	int                    found;
	u8                     packet[TS_SIZE];
	u8                     transbuf[MAXFILT*TS_SIZE];
	int                    transcount[MAXFILT];
	section_t              sec[MAXFILT];
	u16                    pid1;
	u16                    pid2;
	u16                    cpid;
	int                    which_pesto;
	p2a_t                  pesto[2];
        void                   *pointer;
        long                   (*pes_out)(u8 const *buf, long int count, 
                                          void * data);
} trans_t;


enum {AV_PES_STREAM, PS_STREAM, TS_STREAM, PES_STREAM};

typedef struct ps_packet_s{
	u8                     scr[6];
	u8                     mux_rate[3];
	u8                     stuff_length;
	u8                     data[20];
	u8                     sheader_llength[2];
	int                    sheader_length;
	u8                     rate_bound[3];
	u8                     audio_bound;
	u8                     video_bound;
	u8                     reserved;
	int                    npes;
	int                    mpeg;
} ps_packet_t;

typedef struct a2p_s{
	int                    type;
	int                    found;
	int                    length;
	int                    headr;
	u8                     cid;
	u8                     flags;
	u8                     abuf[MAX_PLENGTH];
	int                    alength;
	u8                     vbuf[MAX_PLENGTH];
	int                    vlength;
	int                    plength;
	u8                     last_av_pts[4];
	u8                     av_pts[4];
	u8                     scr[4];
	u16                    count0;
	u16                    count1;
	u16                    pidv;
	u16                    pida;
	u16                    countv;
	u16                    counta;
        void                   *dataA;
        void                   *dataV;
        void                   (*write_cb)(u8 const *buf, long int count,
                                     void *data);
} a2p_t;


typedef struct p2t_s {
        u8               pes[TS_SIZE];
        u8               counter;
        long int         pos;
        int              frags;
        dvb_demux_feed_t *feed;
} p2t_t;


#define MAX_NUM_DVB 4
#define MCMAX 10

/* place to store all the neceassary device information */
struct dvb_struct {
#ifdef V4L2
        struct v4l2_device      v;
	char                    shortname[16];
	int                     is_registered;
	int                     open_count;
	struct device_open      open_data[MAX_OPENS];
	int                     capturing_opens;
#else
	struct video_device	video;
	struct video_device	vbi;
#endif

	int                     num;
	struct video_audio	audio;


	/* add-on devices */
	
	struct saa7146  *saa;
	struct i2c_adapter      *i2cbus;
	struct i2c_client       *tuner;
	struct i2c_client       *decoder;
	struct i2c_client       *sound;
	
	int dvbtype;
#define DVB_NONE 0
#define DVB_S    1
#define DVB_C    2
#define DVB_T    3

        int                     cardtype;
#define DVB_CARD_TT_SIEMENS   0
#define DVB_CARD_TT_BUDGET    1

	/* buffers */
		
        void *iobuf; // memory for all buffers

        struct dvb_buf          avin;
        struct dvb_buf          avout;
        struct dvb_buf          fsec;
#define AVINLEN (512*1024)
#define AVOUTLEN (256*1024)
#define SECBUFLEN (256*1024)

        struct dvb_buf          vout;
        struct dvb_buf          aout;

#define AOUTLEN (64*1024)
#define VOUTLEN (256*1024)

	/* bitmap loading states */

        void                    *bmpbuf;
        int                     bmpp;
        int                     bmplen;
        int                     bmp_win;
        u16                     bmp_x, bmp_y;
        int                     bmp_trans;
        int                     bmp_state;
#define BMP_NONE     0
#define BMP_LOADING  1
#define BMP_LOADINGS 2
#define BMP_LOADED   3
	spinlock_t              bmplock;
	WAIT_QUEUE              bmpq;

        spinlock_t              debilock;
	struct semaphore        dcomlock;
	WAIT_QUEUE              debiq;
	int                     debitype;
	int                     debilen;

	int                     recording;
	int                     playing;
#define RP_NONE  0
#define RP_VIDEO 1
#define RP_AUDIO 2
#define RP_AV    3

	struct                  frontend front;
        struct                  net_device net;
        char                    netname[9];

        int                     osdwin;
        u16                     osdbpp[8];
        int                     ci_slot[2];
#define CI_MODULE_PRESENT 1
#define CI_MODULE_OK      2
        int                     ci_stat;

        int                     playmode;
        int                     vidmode;
	trans_t                 transi;
	a2p_t                   ps_ts_stream;
	dvb_demux_filter_t      p2a_filter[MAXFILT];

        struct dvb_device       dvb_dev;

        dmxdev_t                dmxdev;

        dvb_demux_t             hw_demux;
        char                    hw_demux_id[8];

        dmx_frontend_t          hw_frontend;
        dmx_frontend_t          mem_frontend;

        qpsk_t                  qpsk;
        int                     secbusy;

        boolean                 video_blank;
        struct videoStatus      videostate;
        struct audioStatus      audiostate;
        u32                     powerstate;
	trans_t                 dmx_trans;

        dvb_demux_filter_t      *handle2filter[32];
        p2t_t                   p2t_filter[MAXFILT];

        struct dvb_filter       filter[MAXFILT];
        struct dvb_filter       *hfilter[MAXFILT];
        
        int tsf;
        int arm_errors;
        int feeding;
};

#endif

#endif
/*
 * Local variables:
 * c-indent-level: 8
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -8
 * c-argdecl-indent: 8
 * c-label-offset: -8
 * c-continued-statement-offset: 8
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
