/******************************************************************************
 * hardware functions
 ******************************************************************************/

static int 
StartHWFilter(dvb_demux_filter_t *dvbdmxfilter)
{
        struct dvb_struct *dvb=
                (struct dvb_struct *) dvbdmxfilter->feed->demux->priv;

        u16 buf[20];
        int ret, i;
        u16 handle;
        u16 mode=0x5020;
        
        if (dvbdmxfilter->type==DMX_TYPE_SEC) {
          printk("section filter %d\n", dvbdmxfilter->feed->pid);
                for (i=0; i<16; i++)
                        buf[i+4]=(dvbdmxfilter->filter.filter_value[i]<<8)|
                                  dvbdmxfilter->filter.filter_mask[i];
                mode=4;
        }

        buf[0] = (COMTYPE_PID_FILTER << 8) + AddPIDFilter; 
        buf[1] = 16;	
        buf[2] = dvbdmxfilter->feed->pid;
        buf[3] = mode;

        ret=CommandRequest(dvb, buf, 20, &handle, 1);          
        if (ret<0)
                return ret;

        dvb->handle2filter[handle]=dvbdmxfilter;
        dvbdmxfilter->hw_handle=handle;

        return ret;
}

static int 
StopHWFilter(dvb_demux_filter_t *dvbdmxfilter)
{
        struct dvb_struct *dvb=
          (struct dvb_struct *) dvbdmxfilter->feed->demux->priv;

        u16 buf[3];
        u16 answ[2];
        int ret;
        u16 handle=dvbdmxfilter->hw_handle;
        
        dvb->handle2filter[handle]=NULL;

        buf[0] = (COMTYPE_PID_FILTER << 8) + DelPIDFilter; 
        buf[1] = 1;	
        buf[2] = handle;
        ret=CommandRequest(dvb, buf, 3, answ, 2);          

        if (answ[1] != handle) {
                printk("dvb: filter shutdown error :%d\n", answ[1]);
                ret=-1;
        }
        return ret;
}

static int 
dvb_start_feed(dvb_demux_feed_t *dvbdmxfeed)
{
        dvb_demux_t *dvbdmx=dvbdmxfeed->demux;
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdmx->priv;
  
        if (!dvbdmx->dmx.frontend)
                return -EINVAL;

        if (dvbdmxfeed->type == DMX_TYPE_TS) {
	        if ((dvbdmxfeed->ts_type & TS_DECODER) 
		    && (dvbdmxfeed->pes_type<=1)) {
		        switch (dvbdmx->dmx.frontend->source) {
			case DMX_MEMORY_FE: 
			        if (dvbdmxfeed->ts_type & TS_PACKET)
				       if (dvbdmx->pids[0]!=0xffff &&
					    dvbdmx->pids[1]!=0xffff) {
					        AV_StartPlay(dvb);
						dvbdmx->playing=1;
					}
				
				break;
			default:
			        SetPIDArray(dvb, dvbdmx->pids);
				if (dvbdmxfeed->ts_type & TS_PACKET) {
				  if (dvbdmx->pids[0]!=0xffff && 
				      dvbdmx->pids[1]!=0xffff) {
				          init_a2p(&dvb->ps_ts_stream, 
                                                   av_pes_cb_napi,
                                                   (void *)dvbdmx->pesfilter[0],
                                                   (void *)dvbdmx->pesfilter[1],
                                                   (dvbdmxfeed->ts_type & 
                                                    TS_PAYLOAD_ONLY) ? 
                                                   PS_STREAM : TS_STREAM);
                                          AV_StartRecord(dvb);
                                          dvbdmx->recording=1;
				  }
				}
				break;
			}
		} else
		        if ((dvbdmxfeed->ts_type & TS_PACKET) &&
                            (dvbdmx->dmx.frontend->source!=DMX_MEMORY_FE))
                                StartHWFilter(dvbdmxfeed->filter); 
        }

        
        if (dvbdmxfeed->type == DMX_TYPE_SEC) {
                int i;

	        for (i=0; i<DVB_DEMUX_FILTER_MAX; i++) {
		        if (dvbdmx->filter[i].state!=DMX_STATE_READY)
			        continue;
			if (dvbdmx->filter[i].type!=DMX_TYPE_SEC)
			        continue;
			if (dvbdmx->filter[i].filter.parent!=&dvbdmxfeed->feed.sec)
			        continue;

			dvbdmxfeed->feed.sec.is_filtering=1;
			dvbdmx->filter[i].state=DMX_STATE_GO;
                        if (dvbdmx->dmx.frontend->source!=DMX_MEMORY_FE)
                                StartHWFilter(&dvbdmx->filter[i]); 
                }
	}
        return 0;
}


static int 
dvb_stop_feed(dvb_demux_feed_t *dvbdmxfeed)
{
        dvb_demux_t *dvbdmx=dvbdmxfeed->demux;
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdmx->priv;

        if (dvbdmxfeed->type == DMX_TYPE_TS) {
        if ((dvbdmxfeed->ts_type & TS_DECODER) 
            && (dvbdmxfeed->pes_type<=1)) {
                switch (dvbdmx->dmx.frontend->source) {
                case DMX_MEMORY_FE: 
                        if (dvbdmx->playing) {
                                AV_Stop(dvb);
                                dvbdmx->playing=0;
                        }
                        break;
                default:
                        if (!dvbdmx->pesfilter[dvbdmxfeed->pes_type])
                                return -EINVAL;
                        dvbdmx->pids[dvbdmxfeed->pes_type]=0xffff;
                        dvbdmx->pesfilter[dvbdmxfeed->pes_type]=0;
                        if (dvbdmx->recording) {
                                AV_Stop(dvb);
                                dvbdmx->recording=0;
                        }
#if 0 // should we switch off decoding when closing the device?
                        SetPIDArray(dvb, dvbdmx->pids);
#endif
                }
        } else
          if ((dvbdmxfeed->ts_type & TS_PACKET) &&
              (dvbdmx->dmx.frontend->source!=DMX_MEMORY_FE))
            return StopHWFilter(dvbdmxfeed->filter); 
	}



        if (dvbdmxfeed->type == DMX_TYPE_SEC) {
                int i;

	        for (i=0; i<DVB_DEMUX_FILTER_MAX; i++)
		        if (dvbdmx->filter[i].state==DMX_STATE_GO && 
			    dvbdmx->filter[i].filter.parent==&dvbdmxfeed->feed.sec) {
			        dvbdmx->filter[i].state=DMX_STATE_READY;
				StopHWFilter(&dvbdmx->filter[i]); 
                }

	}

        return 0;
}


static int inline
vidcom(struct dvb_struct *dvb, u32 com, u32 arg)
{
        return outcom(dvb, 0x80, 0x02, 4, 
                      (com>>16), (com&0xffff), 
                      (arg>>16), (arg&0xffff));
}

static int inline
audcom(struct dvb_struct *dvb, u32 com)
{
        return outcom(dvb, 0x80, 0x03, 4, 
                      (com>>16), (com&0xffff));
}

/******************************************************************************
 ******************************************************************************
 * DVB DEVICE LEVEL FUNCTIONS
 ******************************************************************************
 ******************************************************************************/

/******************************************************************************
 * SEC device file operations
 ******************************************************************************/

#define QPSK_MAX_EVENT 8

static void add_qpsk_event(qpsk_t *qpsk, struct qpskEvent *ev)
{
        int wp=qpsk->eventw;
        struct timeval tv;

        get_fast_time(&tv);
        ev->timestamp=tv.tv_sec;
        
        spin_lock(&qpsk->eventlock);
        wp=(wp+1)%QPSK_MAX_EVENT;
        if (wp==qpsk->eventr) {
                qpsk->overflow=1;
                qpsk->eventr=
                  (qpsk->eventr+1)%QPSK_MAX_EVENT;
        }
        memcpy(&qpsk->events[qpsk->eventw], ev, sizeof(struct qpskEvent));
        qpsk->eventw=wp;
        spin_unlock(&qpsk->eventlock);
        wake_up(&qpsk->eventq);
}



/* The following are similar to the SECCDI but not the same
   since SECCDI would not make much sense (e.g. no driver data pointer) 
*/

int secSetTone(struct dvb_struct *dvb, secToneMode mode)
{
        int val;

        switch (mode) {
        case SEC_TONE_ON:
                val=1;
                break;
        case SEC_TONE_OFF:
                val=0;
                break;
        default:
          return -EINVAL;
        }
        dvb->front.ttk=val;
        Set22K(dvb, val);
        return 0;
}

int secSetVoltage(struct dvb_struct *dvb, secVoltage voltage)
{
        int power=1, volt=0;
                       
        switch (voltage) {
        case SEC_VOLTAGE_LT: //FIXME: Test which one is true
        case SEC_VOLTAGE_OFF:
                power=0;
                break;
        case SEC_VOLTAGE_13:
                volt=0;
                break;
        case SEC_VOLTAGE_18:
                volt=1;
                break;
        case SEC_VOLTAGE_13_5:
        case SEC_VOLTAGE_18_5:
        default:
                return -EINVAL;
        }
        dvb->front.volt=volt;
        // this guarantees that running SEC commands are done atomically
        Set22K(dvb, dvb->front.ttk);
        SetVoltage(dvb, power, volt);
        return 0;
}

int secSendSequence(struct dvb_struct *dvb, struct secCmdSequence *seq)
{
        int i, ret, burst, len;
        u8 msg[16];

        switch (seq->miniCommand) {
        case SEC_MINI_NONE:
                burst=-1;
                break;
        case SEC_MINI_A:
                burst=0;
                break;
        case SEC_MINI_B:
                burst=1;
                break;
        default:
                return -EINVAL;
        }

        for (i=0; i<seq->numCommands; i++) {
                switch (seq->commands[i].type) {
                case SEC_CMDTYPE_DISEQC:
                        len=seq->commands[i].u.diseqc.numParams;
                        if (len>SEC_MAX_DISEQC_PARAMS)
                                return -EINVAL;

                        msg[0]=0xe0;
                        msg[1]=seq->commands[i].u.diseqc.addr;
                        msg[2]=seq->commands[i].u.diseqc.cmd;
                        memcpy(msg+3, &seq->commands[i].u.diseqc.params, len);
                        SendDiSEqCMsg(dvb, len+3, msg, -1);
                        break;
                case SEC_CMDTYPE_PAUSE:
                        //FIXME: fix firmware to allow programmed pause length
                        SendDiSEqCMsg(dvb, 0, msg, -1);
                        break;
                case SEC_CMDTYPE_VSEC:
                default:
                        return -EINVAL;
                }
        }
        
        if (burst!=-1)
                SendDiSEqCMsg(dvb, 0, msg, burst);
        
        ret=secSetVoltage(dvb, seq->voltage);
        if (ret<0)
                return ret;
        return secSetTone(dvb, seq->continuousTone);
}




/******************************************************************************
 * DVB device file operations
 ******************************************************************************/

static int 
dvbdev_open(struct dvb_device *dvbdev, int type, 
            struct inode *inode, struct file *file)
{
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdev->priv;

	switch (type) {
	case DVB_DEVICE_VIDEO:
                dvb->video_blank=true;
                dvb->videostate.streamSource=VIDEO_SOURCE_DEMUX;
	case DVB_DEVICE_AUDIO:
                init_trans(&dvb->transi, dvb->p2a_filter, kpes_to_av_pes, (void *) dvb);
                if (!dvb->playing){
                        dvb->avout.tail = dvb->avout.head;
                        init_p2a(&dvb->transi.pesto[0]);
                        init_p2a(&dvb->transi.pesto[1]);
                        dvb->transi.which_pesto = 0;
                }
                break;
	case DVB_DEVICE_SEC:
                if (file->f_flags&O_NONBLOCK) 
                        return -EWOULDBLOCK;
                break;
	case DVB_DEVICE_FRONTEND:
                break;
	case DVB_DEVICE_DEMUX:
                return DmxDevFilterAlloc(&dvb->dmxdev, file);
	case DVB_DEVICE_DVR:
                return DmxDevDVROpen(&dvb->dmxdev, file);
        default:
                return -EINVAL;
	}
        return 0;
}

static int 
dvbdev_close(struct dvb_device *dvbdev, int type, 
             struct inode *inode, struct file *file)
{
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdev->priv;

	switch (type) {
	case DVB_DEVICE_VIDEO:
	        AV_Stop(dvb);
		break;
	case DVB_DEVICE_AUDIO:
                break;
	case DVB_DEVICE_SEC:
                break;
	case DVB_DEVICE_FRONTEND:
                break;
	case DVB_DEVICE_DEMUX:
                return DmxDevFilterFree(&dvb->dmxdev, file);
	case DVB_DEVICE_DVR:
                return DmxDevDVRClose(&dvb->dmxdev, file);
        default:
                return -EINVAL;
	}
        return 0;
}

static ssize_t 
dvbdev_read(struct dvb_device *dvbdev, int type, 
            struct file *file, char *buf, size_t count, loff_t *ppos)
{
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdev->priv;

	switch (type) {
	case DVB_DEVICE_VIDEO:
		break;
	case DVB_DEVICE_AUDIO:
                break;
	case DVB_DEVICE_DEMUX:
                return DmxDevRead(&dvb->dmxdev, file, buf, count, ppos);
	case DVB_DEVICE_DVR:
                return DmxDevDVRRead(&dvb->dmxdev, file, buf, count, ppos);
	default:
	        return -EOPNOTSUPP;
	}
        return 0;
}

static ssize_t 
dvbdev_write(struct dvb_device *dvbdev, int type,
             struct file *file, 
             const char *buf, size_t count, loff_t *ppos)
{
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdev->priv;

	switch (type) {
	case DVB_DEVICE_VIDEO:
	        return dvb_play(dvb, buf, count, file->f_flags&O_NONBLOCK);
	case DVB_DEVICE_AUDIO:
	        return dvb_play(dvb, buf, count, file->f_flags&O_NONBLOCK);
	case DVB_DEVICE_DVR:
                return DmxDevDVRWrite(&dvb->dmxdev, file, buf, count, ppos);
	default:
	        return -EOPNOTSUPP;
	}
}


static int 
dvbdev_ioctl(struct dvb_device *dvbdev, int type, 
             struct file *file, unsigned int cmd, unsigned long arg)
{
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdev->priv;
        void *parg=(void *)arg;

	switch (type) {
	case DVB_DEVICE_VIDEO:
                if (((file->f_flags&O_ACCMODE)==O_RDONLY) &&
                    (cmd!=VIDEO_GET_STATUS))
                        return -EPERM;

                switch (cmd) {

                case VIDEO_STOP:
		        dvb->videostate.playState=VIDEO_STOPPED;
                        vidcom(dvb, 0x000e, dvb->video_blank);
                        return 0; 

                case VIDEO_PLAY:
		        dvb->videostate.playState=VIDEO_PLAYING;
                        vidcom(dvb, 0x000d, 0);
                        break;

                case VIDEO_FREEZE:
		        dvb->videostate.playState=VIDEO_FREEZED;
                        //outcom(dvb, COMTYPE_ENCODER, SetFreezeMode, 1, (u16) 1);
                        vidcom(dvb, 0x000e, dvb->video_blank);
                        break;

                case VIDEO_CONTINUE:
		        if (dvb->videostate.playState==VIDEO_FREEZED) {
			        dvb->videostate.playState=VIDEO_PLAYING;
				vidcom(dvb, 0x000d, 0);
			} 
                        break;

                case VIDEO_SELECT_SOURCE:
                        dvb->videostate.streamSource=(videoStreamSource_t) arg;
                        break;

                case VIDEO_SET_BLANK:
                        dvb->videostate.videoBlank=(boolean) arg;
                        break;

                case VIDEO_GET_STATUS:
                        if(copy_to_user(parg, &dvb->videostate, 
					sizeof(struct videoStatus)))
                                return -EFAULT;
                        break;

                case VIDEO_GET_EVENT:
                        //FIXME: write firmware support for this
                        return -EOPNOTSUPP;

                case VIDEO_SET_DIPLAY_FORMAT:
                {
                        videoDisplayFormat_t format=(videoDisplayFormat_t) arg;
                        u16 val=0;
                        
                        switch(format) {
                        case VIDEO_PAN_SCAN:
                                val=VID_PAN_SCAN_PREF;
                                break;

                        case VIDEO_LETTER_BOX:
                                val=VID_VC_AND_PS_PREF;
                                break;

                        case VIDEO_CENTER_CUT_OUT:
                                val=VID_CENTRE_CUT_PREF;
                                break;

                        default:
                                return -EINVAL;
                        }

                        dvb->videostate.videoFormat=format;
                        return outcom(dvb, COMTYPE_ENCODER, SetPanScanType, 
                                      1, (u16) val);
                }

                case VIDEO_STILLPICTURE:
		{ 
		        struct videoDisplayStillPicture pic;

                        if(copy_from_user(&pic, parg, 
                                          sizeof(struct videoDisplayStillPicture)))
                                return -EFAULT;

                        dvb_play(dvb, pic.iFrame, pic.size, file->f_flags&O_NONBLOCK);
                        dvb_play(dvb, pic.iFrame, pic.size, file->f_flags&O_NONBLOCK);
                        dvb_play(dvb, pic.iFrame, pic.size, file->f_flags&O_NONBLOCK);
                        dvb_play(dvb, pic.iFrame, pic.size, file->f_flags&O_NONBLOCK);
                        dvb_play(dvb, pic.iFrame, pic.size, file->f_flags&O_NONBLOCK);
                        break;
		}

                case VIDEO_FAST_FORWRD:
                        if (dvb->videostate.streamSource!=VIDEO_SOURCE_MEMORY)
                                return -EPERM;
                        vidcom(dvb, 0x16, arg);
                        break;

                case VIDEO_SLOWMOTION:
                        if (dvb->videostate.streamSource!=VIDEO_SOURCE_MEMORY)
                                return -EPERM;
                        vidcom(dvb, 0x22, arg);
                        break;

                default:
                        return -ENOIOCTLCMD;
		}
                return 0;

	case DVB_DEVICE_AUDIO:
                if (((file->f_flags&O_ACCMODE)==O_RDONLY) &&
                    (cmd!=AUDIO_GET_STATUS))
                        return -EPERM;

                switch (cmd) {

                case AUDIO_STOP:
			audcom(dvb, 1);
                        dvb->audiostate.playState=AUDIO_STOPPED;
                        break;

                case AUDIO_PLAY:
			audcom(dvb, 2);
                        dvb->audiostate.playState=AUDIO_PLAYING;
		        break;

                case AUDIO_PAUSE:
			audcom(dvb, 1);
		        dvb->audiostate.playState=AUDIO_PAUSED;
                        break;

                case AUDIO_CONTINUE:
		        if (dvb->audiostate.playState==AUDIO_PAUSED) {
			        dvb->audiostate.playState=AUDIO_PLAYING;
				audcom(dvb, 2);
			} 
		        break;

                case AUDIO_SELECT_SOURCE:
                        dvb->audiostate.streamSource=(audioStreamSource_t) arg;
		        break;

                case AUDIO_SET_MUTE:
                {
		        u16 val=((boolean) arg) ? 0x80 : 0xc5; 

                        dvb->audiostate.muteState=(boolean) arg;
		        SendDAC(dvb, 3, val);
		        SendDAC(dvb, 4, val);
			audcom(dvb, arg ? 1 : 2);
                        break;
		}
                case AUDIO_SET_AV_SYNC:
                        dvb->audiostate.AVSyncState=(boolean) arg;
                        //FIXME: fix firmware to support this
                        return -EOPNOTSUPP;

                case AUDIO_SET_BYPASS_MODE:
		        return -EINVAL;

                case AUDIO_CHANNEL_SELECT:
		        dvb->audiostate.channelSelect=(audioChannelSelect_t) arg;
                        
                        switch(dvb->audiostate.channelSelect) {
                        case AUDIO_STEREO:
  			        audcom(dvb, 0x80);
                                break;

                        case AUDIO_MONO_LEFT:
  			        audcom(dvb, 0x100);
                                break;

                        case AUDIO_MONO_RIGHT:
  			        audcom(dvb, 0x200);
                                break;

                        default:
                                return -EINVAL;
			}
			return 0;

                case AUDIO_GET_STATUS:
                        if(copy_to_user(parg, &dvb->audiostate, 
					sizeof(struct audioStatus)))
                                return -EFAULT;
                        break;

                default:
                        return -ENOIOCTLCMD;
                }
		break;

	case DVB_DEVICE_FRONTEND:
                switch (cmd) {
                case OST_SELFTEST:
                        break;

                case OST_SET_POWER_STATE:
                  break;

                case OST_GET_POWER_STATE:
                  break;

                case FE_READ_STATUS:
                {
                        feStatus stat;
                        
                        GetFront(dvb);
                        stat=0;
                        if (dvb->front.power)
                                stat|=FE_HAS_POWER;
                        if ((dvb->front.sync&0x1f)==0x1f)
                                stat|=FE_HAS_SIGNAL;
                        if (dvb->front.inv)
                                stat|=QPSK_SPECTRUM_INV;

                        if(copy_to_user(parg, &stat, sizeof(stat)))
                                return -EFAULT;
                        break;
                }

                case FE_READ_BER:
                {
                        uint32_t ber;
                        
                        GetFront(dvb);
                        /*FIXME: ENOSIGNAL??
                          if (!dvb->front.power)
                          return -ENOSIGNAL;
                        */
                        ber=dvb->front.vber*10;
                        if(copy_to_user(parg, &ber, sizeof(ber)))
                                return -EFAULT;
                        break;
                }

                case FE_READ_SIGNAL_STRENGTH:
                {
                        uint32_t signal;
                        
                        GetFront(dvb);
                        /*FIXME: ENOSIGNAL=??
                          if (!dvb->front.power)
                          return -ENOSIGNAL;
                        */
                        signal=dvb->front.agc; //FIXME: scale value
                        if(copy_to_user(parg, &signal, sizeof(signal)))
                                return -EFAULT;
                        break;
                }

                case FE_READ_SNR:
                        return -ENOSYS;

                case FE_READ_UNCORRECTED_BLOCKS: 
                  //FIXME: VES1893 only supports counter up to 0x7f
                  //       how should we handle this??
                        return -ENOSYS; 

                case FE_GET_NEXT_FREQUENCY:
                {
                        uint32_t freq;
                    
                        if(copy_from_user(&freq, parg, sizeof(freq)))
                                return -EFAULT;

                        // FIXME: how does one calculate this?
                        freq+=1000; //FIXME: KHz like in QPSK_TUNE??
                        
                        if(copy_to_user(parg, &freq, sizeof(freq)))
                                return -EFAULT;
                        break;
                }

                case FE_GET_NEXT_SYMBOL_RATE:
                {
                        uint32_t rate;
                        
                        if(copy_from_user(&rate, parg, sizeof(rate)))
                                return -EFAULT;
                        // FIXME: how does one really calculate this?
                        if (rate<5000000)
                                rate+=500000;
                        else if(rate<10000000)
                                rate+=1000000;
                        else if(rate<30000000)
                                rate+=2000000;
                        else 
                                return -EINVAL;
                        if(copy_to_user(parg, &rate, sizeof(rate)))
                                return -EFAULT;
                        break;
                }

                case QPSK_TUNE:
                {
                        struct qpskParameters para;
                        static const uint8_t fectab[8]={8,0,1,2,4,6,0,8};
                        u32 val;
                        
                        if(copy_from_user(&para, parg, sizeof(para)))
                                return -EFAULT;
                        if ((file->f_flags&O_ACCMODE)==O_RDONLY) 
                                return -EPERM;
                        if (para.FEC_inner>7)
                                return -EINVAL;
                        val=para.iFrequency*1000;
                        if (dvb->front.freq!=val) {
                                tuner_command(dvb, TUNER_SET_TVFREQ, &val); 
                                dvb->front.freq=val;
                        }

                        val=para.SymbolRate;
                        if (dvb->front.srate!=val || 
                            dvb->front.fec!=fectab[para.FEC_inner]) {
                                dvb->front.srate=val;
                                dvb->front.fec=fectab[para.FEC_inner];
                                decoder_command(dvb, DVB_SET_FRONTEND, &dvb->front); 
                        } 
                        {
                                struct qpskEvent ev;
                                
                                ev.type=0;
                                ev.u.completionEvent.iFrequency=para.iFrequency;
                                ev.u.completionEvent.SymbolRate=para.SymbolRate;
                                ev.u.completionEvent.FEC_inner=para.FEC_inner;
                                
                                add_qpsk_event(&dvb->qpsk, &ev);

                        }
                        return 0;
                }

                case QPSK_GET_EVENT:
                { 
                        int ret;

                        if (dvb->qpsk.overflow) {
                                dvb->qpsk.overflow=0;
                                return -ENOBUFS; //FIXME: -EBUFFEROVERFLOW;
                        }
                        if (dvb->qpsk.eventw==dvb->qpsk.eventr) {
                                if (file->f_flags&O_NONBLOCK) 
                                        return -EWOULDBLOCK;
                                else {
                                  ret=wait_event_interruptible(dvb->qpsk.eventq,
                                                               dvb->qpsk.eventw!=
                                                               dvb->qpsk.eventr);
                                  if (ret<0)
                                          return ret;
                                }
                        }
                        spin_lock(&dvb->qpsk.eventlock);
                        if(copy_to_user(parg, &dvb->qpsk.events[dvb->qpsk.eventr], 
                                        sizeof(struct qpskEvent))) {
                                spin_unlock(&dvb->qpsk.eventlock);
                                return -EFAULT;
                        }
                        dvb->qpsk.eventr=(dvb->qpsk.eventr+1)%QPSK_MAX_EVENT;
                        spin_unlock(&dvb->qpsk.eventlock);
                        return 0;
                }

                case QPSK_FE_INFO:
                {        
                        struct qpskFrontendInfo feinfo;
                        
                        feinfo.minFrequency=500;  //KHz?
                        feinfo.maxFrequency=2100000;
                        feinfo.minSymbolRate=500000;
                        feinfo.maxSymbolRate=30000000;
                        feinfo.hwType=0;    //??
                        feinfo.hwVersion=0; //??
                        if(copy_to_user(parg, &feinfo, sizeof(feinfo)))
                                return -EFAULT;
                        break;
                }

                case QPSK_WRITE_REGISTER:
                        return -EINVAL;

                case QPSK_READ_REGISTER:
                        return -EINVAL;
                }
		return 0;
        
        // NOTE: right now the DiSEqC support in the firmware is set up
        //       as busy waiting so that atomicity is guaranteed automatically
	case DVB_DEVICE_SEC:
                if (file->f_flags&O_NONBLOCK) 
                        return -EWOULDBLOCK;
                switch (cmd) {
                case SEC_GET_STATUS: 
                {        
                        struct secStatus status;
                        
                        status.busMode=SEC_BUS_IDLE;
                        
                        if (dvb->secbusy) 
                                status.busMode=SEC_BUS_BUSY;
                        if (!dvb->front.power) 
                                status.busMode=SEC_BUS_OFF;
                        
                        status.selVolt=(dvb->front.volt ?
                                        SEC_VOLTAGE_18 :
                                        SEC_VOLTAGE_13);

                        status.contTone=(dvb->front.ttk ?
                                         SEC_TONE_ON :
                                         SEC_TONE_OFF);
                        if(copy_to_user(parg, &status, sizeof(status)))
                                return -EFAULT;
                }
                        break;
                case SEC_RESET_OVERLOAD:
                        if ((file->f_flags&O_ACCMODE)==O_RDONLY) 
                                return -EPERM;
                        // overload control not supported by Siemens DVB card
                        break;

                case SEC_SEND_SEQUENCE:
                {
                        struct secCmdSequence seq;

                        if(copy_from_user(&seq, parg, sizeof(seq)))
                                return -EFAULT;

                        if ((file->f_flags&O_ACCMODE)==O_RDONLY) 
                                return -EPERM;

                        return secSendSequence(dvb, &seq);
                }
                case SEC_SET_TONE:
                {
                        secToneMode mode = (secToneMode) arg;

                        if ((file->f_flags&O_ACCMODE)==O_RDONLY) 
                                return -EPERM;
                        
                        return secSetTone(dvb, mode);
                }
                case SEC_SET_VOLTAGE:
                {
                        secVoltage val = (secVoltage) arg;

                        if ((file->f_flags&O_ACCMODE)==O_RDONLY) 
                                return -EPERM;
                        
                        return secSetVoltage(dvb, val);

                }
                default:
                        return -EINVAL;
                }
		return 0;

        case DVB_DEVICE_DEMUX:
                return DmxDevIoctl(&dvb->dmxdev, file, cmd, arg);

        default:
                return -EOPNOTSUPP;
        }

        return 0;
}

static unsigned int 
dvbdev_poll(struct dvb_device *dvbdev, int type, 
            struct file *file, poll_table * wait)
{
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdev->priv;

	switch (type) {
	case DVB_DEVICE_FRONTEND:
                if (dvb->qpsk.eventw!=dvb->qpsk.eventr)
                        return (POLLIN | POLLRDNORM | POLLPRI);

                poll_wait(file, &dvb->qpsk.eventq, wait);
                
                if (dvb->qpsk.eventw!=dvb->qpsk.eventr)
                        return (POLLIN | POLLRDNORM | POLLPRI);
                break;

        case DVB_DEVICE_DEMUX:
                return DmxDevPoll(&dvb->dmxdev, file, wait);

	default:
	        return -EOPNOTSUPP;
        }

        return 0;
}




/******************************************************************************
 ******************************************************************************
 * DVB DRIVER LEVEL FUNCTIONS
 ******************************************************************************
 ******************************************************************************/


/******************************************************************************
 * driver registration 
 ******************************************************************************/

static int dvb_register(struct dvb_struct *dvb)
{
        int result, i;
        struct dvb_device *dvbd=&dvb->dvb_dev;

        dvb->dmxdev.filternum=32;
        dvb->dmxdev.demux=&dvb->hw_demux.dmx;
        //dvb->dmxdev.set_pids=(int (*)(void *, u16 *)) SetPIDArray;
        //dvb->dmxdev.priv=(void *) dvb;
        dvb->dmxdev.capabilities=0;
        DmxDevInit(&dvb->dmxdev);
        
        for (i=0; i<32; i++)
                dvb->handle2filter[i]=NULL;

        init_waitqueue_head(&dvb->qpsk.eventq);
        spin_lock_init (&dvb->qpsk.eventlock);
        dvb->qpsk.eventw=dvb->qpsk.eventr=0;
        dvb->qpsk.overflow=0;
        dvb->secbusy=0;

	dvb->audiostate.AVSyncState=0;
	dvb->audiostate.muteState=0;
	dvb->audiostate.playState=AUDIO_STOPPED;
	dvb->audiostate.streamSource=AUDIO_SOURCE_DEMUX;
	dvb->audiostate.channelSelect=AUDIO_STEREO;
	dvb->audiostate.bypassMode=0;

	dvb->videostate.videoBlank=0;
	dvb->videostate.playState=VIDEO_STOPPED;
	dvb->videostate.streamSource=VIDEO_SOURCE_DEMUX;
	dvb->videostate.videoFormat=VIDEO_FORMAT_4_3;
	dvb->videostate.displayFormat=VIDEO_CENTER_CUT_OUT;

        // init and register dvb device structure
        dvbd->priv=(void *) dvb;
        dvbd->open=dvbdev_open;
        dvbd->close=dvbdev_close;
        dvbd->read=dvbdev_read;
        dvbd->write=dvbdev_write;
        dvbd->ioctl=dvbdev_ioctl;
        dvbd->poll=dvbdev_poll;
        
        result=dvb_register_device(&dvb->dvb_dev);
        if (result<0)
                return result;


        // init and register demuxes
        
        memcpy(dvb->hw_demux_id, "demux0", 7);
        dvb->hw_demux_id[5]=dvb->num+0x30;
        DvbDmxInit(&dvb->hw_demux, (void *) dvb, dvb->hw_demux_id,
                   "TI", "AV7110");
        dvb->hw_demux.start_feed=dvb_start_feed;
        dvb->hw_demux.stop_feed=dvb_stop_feed;
        dvb->hw_demux.write_to_decoder=0;

        dvb->mem_frontend.id="mem_frontend";
        dvb->mem_frontend.vendor="memory";
        dvb->mem_frontend.model="sw";
        dvb->mem_frontend.source=DMX_MEMORY_FE;

        /*
        memcpy(dvb->sw_demux_id, "demuxs0", 8);
        dvb->sw_demux_id[6]=dvb->num+0x30;
        DvbDmxInit(&dvb->sw_demux, (void *) dvb, dvb->sw_demux_id,
                     "TI", "AV7110");
        
        dvb->sw_demux.dmx.add_frontend(&dvb->sw_demux.dmx, &dvb->mem_frontend);
        */

        return 0;
}


/* Register frontend
   This is an extra routine because the frontend type is not known until the
   frontend module registered itself on the I2C bus (which can happen after the 
   dvb module is installed)
*/

static int dvb_register_frontend(struct dvb_struct *dvb)
{
        int ret;
        dmx_frontend_t *dvbfront=&dvb->hw_frontend;

        dvbfront->id="hw_frontend";
        dvbfront->vendor="VLSI";
        switch (dvb->dvbtype) {
        default:
        case DVB_S:
                dvbfront->model="1893";
                dvbfront->source=DMX_SATELLITE_FE;
                break;
        case DVB_C:
                dvbfront->model="1820";
                dvbfront->source=DMX_CABLE_FE;
                break;
        }

        ret=dvb->hw_demux.dmx.add_frontend(&dvb->hw_demux.dmx, &dvb->hw_frontend);
        if (ret<0)
                return ret;
        
        ret=dvb->hw_demux.dmx.add_frontend(&dvb->hw_demux.dmx, &dvb->mem_frontend);
        if (ret<0)
                return ret;

        ret=dvb->hw_demux.dmx.connect_frontend(&dvb->hw_demux.dmx, &dvb->hw_frontend);
        if (ret<0)
                return ret;

        return 0;
}


static void dvb_unregister(struct dvb_struct *dvb)
{
        dvb->hw_demux.dmx.close(&dvb->hw_demux.dmx);
        dvb->hw_demux.dmx.remove_frontend(&dvb->hw_demux.dmx, &dvb->hw_frontend);
        dvb->hw_demux.dmx.remove_frontend(&dvb->hw_demux.dmx, &dvb->mem_frontend);
        dmx_unregister_demux(&dvb->hw_demux.dmx);

        /*
        dvb->sw_demux.dmx.close(&dvb->sw_demux.dmx);
        dvb->sw_demux.dmx.remove_frontend(&dvb->sw_demux.dmx, &dvb->mem_frontend);
        dmx_unregister_demux(&dvb->sw_demux.dmx);
        */

        dvb_unregister_device(&dvb->dvb_dev);
}


/*
 * Local variables:
 * c-indent-level: 8
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -8
 * c-argdecl-indent: 8
 * c-label-offset: -8
 * c-continued-statement-offset: 8
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
