#include "DVB.hh"

DVB::~DVB() {
        delete [] lnbs;
	delete [] diseqcs;
	delete [] rotors;
	delete [] tps;
	delete [] chans;
	delete [] bouqs;
	delete [] sats;
	delete [] swis;
	delete [] ntws;
#ifdef NAPI
	//close(fd_video);
	//close(fd_audio);
	close(fd_frontend);
	close(fd_sec);
	close(fd_demuxa);
	close(fd_demuxv);
#else
	close(fsi);
#endif
	close(fdvb);
}


void *DVB::fthread_start(void *) {
        return 0;
}

static void *dummy_start(void *dvb) {
        ((DVB *)dvb)->fthread_start(0);
	return 0;
}

int DVB::start_fthread(void) {
        return pthread_create(&fthread, NULL, &dummy_start, (void *) this);
} 


void DVB::init(char *dvbname="/dev/video0", char *siname="/dev/vbi0",int minor = 0) {
	int failed =0;

#ifndef NAPI
        if (fdvb>0) close(fdvb);
	if (fsi>0) close(fsi);
        fdvb=open(dvbname, O_RDWR);
	if (fdvb<0) {
	  cerr << "Could not open " << dvbname << "\n";
	  return;
	} else ;//cerr << "Opened " << dvbname << "  fd=" << fdvb << endl;
	
	fsi=open(siname, O_RDWR);
	if (fsi<0) {
	  cerr << "Could not open " << siname << "\n";
	  close(fdvb);
	  fdvb = -1;
	  return;
	} else;// cerr << "Opened " << siname << "  fd=" << fsi << endl;
	if (fdvb >= 0) osd.init(fdvb);
#endif

#ifdef NAPI
	//if (fd_audio > 0) close(fd_audio);
        //if (fd_video > 0) close(fd_video);
        if (fd_sec > 0) close(fd_sec);
        if (fd_frontend > 0) close(fd_frontend);
        if (fd_demuxa > 0) close(fd_demuxa);
        if (fd_demuxv > 0) close(fd_demuxv);
	char devname[80];

	/*
	sprintf(devname,"%s%d",VIDEO_DEV,minor);
	fd_video=open(devname, O_RDWR);
	if (fd_video < 0) {
		cerr << "Could not open " << devname << endl;
		perror(devname);
		fd_video = -1;
		failed = 1;
	}

	sprintf(devname,"%s%d",AUDIO_DEV,minor);
	fd_audio=open(devname, O_RDWR);
	if (fd_audio < 0) {
		cerr << "Could not open " << devname << endl;
		perror(devname);
		fd_audio = -1;
		failed = 1;
	}
	*/
	sprintf(devname,"%s%d",FRONT_DEV,minor);
	fd_frontend=open(devname, O_RDWR);
	if (fd_frontend < 0) {
		cerr << "Could not open " << devname << endl;
		perror(devname);
		fd_frontend = -1;
		failed = 1;
	}

	sprintf(devname,"%s%d",SEC_DEV,minor);
	fd_sec=open(devname, O_RDWR);
	if (fd_sec < 0) {
		cerr << "Could not open " << devname << endl;
		perror(devname);
		fd_sec = -1;
		failed = 1;
	}

	sprintf(devname,"%s%d",DEMUX_DEV,minor);
	fd_demuxa=open(devname, O_RDWR);
	if (fd_demuxa < 0) {
		cerr << "Could not open " << devname << endl;
		perror(devname);
		fd_demuxa = -1;
		failed = 1;
	}

	fd_demuxv=open(devname, O_RDWR);
	if (fd_demuxv < 0) {
		cerr << "Could not open " << devname << endl;
		perror(devname);
		fd_demuxv = -1;
		failed = 1;
	}

#else
	ioctl(fdvb, VIDIOCGFRONTEND, &front);
#endif	

	for (int i=0; i<NK; i++)
	        num[i]=0;

	lnbs   =new Lnb[maxs[LNB]];
	diseqcs=new DiSEqC[maxs[DIS]];
	rotors =new Rotor[maxs[ROTOR]];
	tps    =new Transponder[maxs[TRANS]];
	chans  =new Channel[maxs[CHAN]];;
	bouqs  =new Bouquet[maxs[BOU]];;
	sats   =new Sat[maxs[SAT]];;
	swis   =new Switch[maxs[SWI]];;
	ntws   =new Network[maxs[NTW]];;

	start_fthread();
}

int DVB::SetTP(unsigned int tpid, unsigned int satid)
{
        int i;
	Transponder *tp=0;
	Sat *sat=0;
	Lnb *lnb=0;

	for (i=0; i<num[TRANS]; i++)  
	        if (tps[i].id==tpid &&
		    // accept NOID for backwards compatibility 
		    (tps[i].satid==satid || satid==NOID)) {
		        tp=&tps[i];
			break;
		}
	if (!tp)
	        return -1;

	for (i=0; i<num[SAT]; i++) {
		if (sats[i].id==tp->satid) {
		        sat=&sats[i];
			break;
		}
	}
	if (!sat)
	        return -1;

	for (i=0; i<num[LNB]; i++)  
               if (lnbs[i].id==sat->lnbid) {
		        lnb=&lnbs[i];
			break;
	       }
	if (!lnb)
	        return -1;

#ifdef NAPI
	static const uint8_t rfectab[9] = {1,2,3,0,4,0,5,0,0};
	                
	if (tp->freq < lnb->slof) {
		qpsk.iFrequency = (tp->freq - lnb->lof1);
		scmds.continuousTone = SEC_TONE_OFF;
	} else {
		qpsk.iFrequency = (tp->freq - lnb->lof2);
		scmds.continuousTone = SEC_TONE_ON;
	}
	if (tp->pol) scmds.voltage = SEC_VOLTAGE_18;
	else scmds.voltage = SEC_VOLTAGE_13;
	set_diseqc_nb(lnb->diseqcnr);
	qpsk.SymbolRate = tp->srate;
	qpsk.FEC_inner = rfectab[tp->fec];
#else
	switch (front.type) {
	case FRONT_DVBS:
	        if (tp->freq < lnb->slof) {
		        front.freq=(tp->freq - lnb->lof1)*1000UL;
		        front.ttk=0;
		} else {
		        front.freq=(tp->freq - lnb->lof2)*1000UL;
			front.ttk=1;
		}
		front.diseqc=lnb->diseqcnr;
		front.volt=tp->pol;
		break;
	case FRONT_DVBC:
	        front.freq=tp->freq;	        
	        front.qam=tp->qam;
	        break;
	}

	front.srate=tp->srate;
	front.fec=tp->fec;
	front.AFC=1;
#endif

        return 0;
}

int DVB::SetChannel(int chnr) 
{
	Channel *chan=0;
	
	//cerr << "setchannel" << endl;
	
	if (chnr>=num[CHAN])
	        return -1;
	chan=&chans[chnr];

#ifdef NAPI
	get_front();
        set_vpid(chan->vpid);
        set_apid(chan->apids[0]);
#else
	front.pnr=chan->pnr;
	front.video_pid=chan->vpid;
	front.audio_pid=chan->apids[0];
	front.channel_flags=chan->type;
#endif
	SetTP(chan->tpid, chan->satid);
	set_front();

	cerr  << chan->pnr << ": " << chan->name << endl;

 	/*
	osd.Open(80, 500, 640, 540, 2, 0, 2);
	osd.SetColor(0, 0, 0, 0, 255);
	osd.SetColor(1, 240, 240, 240, 255);

	{
	  ostrstream fstr;
	  fstr << dec << chan->id << ": " << chan->name << ends;
	  osd.Text(0, 0, 3, 1, fstr.str());
	}
	*/
	/*
	VIDEOSTATE vstate;

	ioctl(fdvb, VIDIOCGVIDEOSTATE, &vstate);
	
	cout << dec << "HSize=" << vstate.HSize << endl;
	cout << dec << "VSize=" << vstate.VSize << endl;
	cout << dec << "BitRate=" << vstate.BitRate << endl;
	cout << dec << "BytesDecoded=" << vstate.BytesDecoded << endl;
	cout << dec << "InitDone=" << vstate.InitDone << endl;
	*/
        return 0;

}

int DVB::SetChannel(uint16_t sid, uint16_t onid, 
		    uint16_t tpid, uint16_t satid)
{
        int chnr, i;
	Transponder *tp=0;
	Sat *sat=0;
	Lnb *lnb=0;
	Channel *chan=0;

	for (chnr=0; chnr<num[CHAN]; chnr++)  
	        if (chans[chnr].pnr==sid   && 
		    (onid==0xffff  || chans[chnr].onid==onid) && 
		    (satid==0xffff || chans[chnr].satid==satid)  && 
		    (tpid==0xffff  || chans[chnr].tpid==tpid)
		    ) {
		        chan=&chans[chnr];
			break;
		}
	if (!chan)
	        return -1;

	if (tpid==NOID)
 	        tpid=chan->tpid;
	if (onid==NOID)
 	        onid=chan->onid;
	if (satid==NOID)
 	        satid=chan->satid;

	for (i=0; i<num[TRANS]; i++)  
	        if (tps[i].id==tpid && tps[i].satid==satid) {
		        tp=&tps[i];
			break;
		}
	if (!tp)
	        return -1;

	for (i=0; i<num[SAT]; i++) {
		if (sats[i].id==tp->satid) {
		        sat=&sats[i];
			break;
		}
	}
	if (!sat)
	        return -1;

	for (i=0; i<num[LNB]; i++)  
               if (lnbs[i].id==sat->lnbid) {
		        lnb=&lnbs[i];
			break;
	       }
	if (!lnb)
	        return -1;

	get_front();
#ifdef NAPI
	static const uint8_t rfectab[9] = {1,2,3,0,4,0,5,0,0};
	                
	if (tp->freq < lnb->slof) {
		qpsk.iFrequency = (tp->freq - lnb->lof1);
		scmds.continuousTone = SEC_TONE_OFF;
	} else {
		qpsk.iFrequency = (tp->freq - lnb->lof2);
		scmds.continuousTone = SEC_TONE_ON;
	}
	if (tp->pol) scmds.voltage = SEC_VOLTAGE_18;
	else scmds.voltage = SEC_VOLTAGE_13;
	set_diseqc_nb(lnb->diseqcnr);
	qpsk.SymbolRate = tp->srate;
	qpsk.FEC_inner = rfectab[tp->fec];

	set_front();
        set_vpid(chan->vpid);
        set_apid(chan->apids[0]);
#else
	front.pnr=chan->pnr;
	front.video_pid=chan->vpid;
	front.audio_pid=chan->apids[0];
	front.channel_flags=chan->type;

	switch (front.type) {
	case FRONT_DVBS:
	        if (tp->freq < lnb->slof) {
		        front.freq=(tp->freq - lnb->lof1)*1000UL;
		        front.ttk=0;
		} else {
		        front.freq=(tp->freq - lnb->lof2)*1000UL;
			front.ttk=1;
		}
		front.diseqc=lnb->diseqcnr;
		front.volt=tp->pol;
		break;
	case FRONT_DVBC:
	        front.freq=tp->freq;	        
	        front.qam=tp->qam;
	        break;
	}

	front.srate=tp->srate;
	front.fec=tp->fec;
	front.AFC=1;
	set_front();
#endif
	return chnr;
}

bool DVB::Tune(int &newProgNr, int &oldProgNr, uint16_t ONID,
	       uint16_t TSID, int SID)
{
        if ((newProgNr=SetChannel(SID, TSID, 0xffff, ONID))<0)
	        return false;
	else
	        return true;
}



int DVB::GetChannel(int chnr, struct channel *) 
{
        int i;
	Channel *chan=0;
	Transponder *tp=0;
	Sat *sat=0;
	Lnb *lnb=0;

	if (chnr>=num[CHAN])
	        return -1;
	chan=&chans[chnr];
	
	for (i=0; i<num[TRANS]; i++)  
               if (tps[i].id==chan->tpid) {
		        tp=&tps[i];
			break;
	       }
	if (!tp)
	        return -1;

	for (i=0; i<num[SAT]; i++)  
               if (sats[i].id==tp->satid) {
		        sat=&sats[i];
			break;
	       }
	if (!sat)
	        return -1;

	for (i=0; i<num[LNB]; i++)  
               if (lnbs[i].id==sat->lnbid) {
		        lnb=&lnbs[i];
			break;
	       }
	if (!lnb)
	        return -1;
	cerr << "ok\n";
#ifdef NAPI
#else
	ioctl(fdvb, VIDIOCGFRONTEND, &front);
#endif


#ifdef NAPI
#else
	switch (front.type) {
	case FRONT_DVBS:
	        if (tp->freq < lnb->slof) {
		        front.freq=(tp->freq - lnb->lof1)*1000UL;
		        front.ttk=0;
		} else {
		        front.freq=(tp->freq - lnb->lof2)*1000UL;
			front.ttk=1;
		}
		front.diseqc=lnb->diseqcnr;
		front.volt=tp->pol;
		break;
	case FRONT_DVBC:
	        front.freq=tp->freq;	        
	        front.qam=tp->qam;
	        break;
	   
	}
	front.srate=tp->srate;
	front.fec=tp->fec;
	front.pnr=chan->pnr;
	front.video_pid=chan->vpid;
	front.audio_pid=chan->apids[0];
	front.AFC=1;
	front.channel_flags=chan->type;
#endif

	set_front();

	/*
	osd.Open(80, 500, 640, 540, 2, 0, 2);
	osd.SetColor(0, 0, 0, 0, 255);
	osd.SetColor(1, 240, 240, 240, 255);
	
	{
	  ostrstream fstr;
	  fstr << dec << chan->id << ": " << chan->name << ends;
	  osd.Text(0, 0, 3, 1, fstr.str());
	}
	*/
	/*
	VIDEOSTATE vstate;

	ioctl(fdvb, VIDIOCGVIDEOSTATE, &vstate);
	
	cout << dec << "HSize=" << vstate.HSize << endl;
	cout << dec << "VSize=" << vstate.VSize << endl;
	cout << dec << "BitRate=" << vstate.BitRate << endl;
	cout << dec << "BytesDecoded=" << vstate.BytesDecoded << endl;
	cout << dec << "InitDone=" << vstate.InitDone << endl;
	*/
        return 0;

}


static dvb_sdt_service_t *find_service(dvb_pat_prog_t *prog, dvb_sdt_t *sdt) {
        dvb_sdt_service_t *service;
	  
	if (!sdt || !prog ) 
	        return 0;
	service = sdt->service;
	while (service) {
	        if (prog->prog_nr == service->service_id)
		        return service;
		service=service->next;
	}
	return 0;
}
 
static void dvb2txt(char *out, char *in, int len)
{
        //char *chartable=0;
        unsigned char *txt=(unsigned char *) in;
        if (!len)
	        return;
	switch (*txt) {
	case 0x01 ... 0x0f:
	        txt++;
	        len--;
	        break;
	case 0x10:
	        txt+=3;
                len-=3;
	        break;
	}
	while (len) {
	        switch(*txt) {
		case 0x7f ... 0xa0:
		        len--;
			txt++;
			break;
		case 0x00:
		        len=1;
		default:
		        *(out++)=(char) *(txt++);
			len--;
			break;
		}
	}
}


int DVB::AddChannels(Transponder &tp, dvb_pat_t &pat, dvb_sdt_t &sdt)
{
        dvb_pmt_stream_t *stream;
        dvb_pat_prog_t *prog;
	dvb_sdt_service_t *service;
	Channel chan;

	prog=pat.prog;
	while (prog) {
		if (prog->prog_nr) {
		        chan.pnr=prog->prog_nr;
		        if ((service=find_service(prog, &sdt))) {
			        chan.type|=service->free_ca_mode;
			        if (service->descr && service->descr->service
				    && service->descr->service->service_name) {
				        dvb2txt(chan.name,
						service->descr->service->service_name,
						maxname);
					cerr << "Found: " << chan.name << endl;
				}
			}
			
			if (prog->prog.pmt) {
			        chan.pcrpid=prog->prog.pmt->pcr_pid;
				stream = prog->prog.pmt->stream;
				while (stream) {
				        switch (stream->stream_type) {
					case 1: 
					case 2: 
					        chan.vpid=stream->pid;
						break;
					case 3: 
					case 4: 
					        if (chan.apidnum>=MAXAPIDS) {
						        cerr << "Need more apids\n";
							break;
						}
						chan.apids[chan.apidnum++]=stream->pid;
						break; 
					case 6: 
					        chan.ttpid=stream->pid;
						break;
					default:
					  //cerr << "Unknown: " <<  HEX(4) << (int) stream->stream_type << "  PID: " << HEX(2) << (int) stream->pid << endl;
					  break;
					}
					stream = stream->next;
				}
			}
			chan.tpid=tp.id;
			chan.satid = tp.satid;
			AddChannel(chan);
			chan.clearall();
		}
	        prog=prog->next;
	}  

        return 0;
}


int DVB::scan_tp(struct Transponder &tp) 
{
        dvb_sdt_t sdt;
        dvb_pat_t pat;
	dvb_pat_prog_t *prog;
	int pmtnum=0;

	memset(&pat, 0, sizeof(pat));
        if (GetPAT(&pat)<0)
	       return -1;

	if(fdvb >=  0) osd.Text(0, 120, 2, 1, "PAT");
	cerr << "PAT" << endl;

	prog=pat.prog;
	while (prog) {
		if (!prog->prog_nr) {
		        dvb_nit_t *nit = (dvb_nit_t *)calloc (1, sizeof (dvb_nit_t));
			if (GetNIT(nit) >= 0) {
			        if (prog->prog.nit)
				        free(prog->prog.nit);
				prog->prog.nit = nit;
			} else 
			        free(nit);

			if (fdvb >= 0) osd.Text(64, 120, 2, 1, "NIT");

			cerr << "NIT\n";

			// FIXME: fix NIT parsing 
			//if (front.type!=FRONT_DVBC)
			        tp.freq=(__u32) (1000000*nit->freq+0.5);
		} else {
		  fprintf(stderr,"PMT: %04x:%04x\n", prog->prog_nr, prog->pid);
		        dvb_pmt_t *pmt = (dvb_pmt_t *)calloc (1, sizeof (dvb_pmt_t));
			if (GetPMT(pmt, prog->pid) >= 0) {
			        pmtnum++;
			        if (prog->prog.pmt)
				        free (prog->prog.pmt);
				prog->prog.pmt = pmt;
			} else 
			        free (pmt);
		}
	        prog=prog->next;
	}

	if(fdvb >= 0) osd.Text(128, 120, 2, 1, "PMT");

	cerr << "freq:" << tp.freq << endl;
	/* Ignore TPs with no PMT or SDT */
	if (!pmtnum) {
	        free_pat(&pat);
	        return -1;
	}
	memset(&sdt, 0, sizeof(dvb_sdt_t));
	if (GetSDT(&sdt)<0) {
	        free_pat(&pat);
	        return -1;
	}
	if(fdvb >= 0) osd.Text(196, 120, 2, 1, "SDT");

	tp.id=(tp.satid<<16)|pat.ts_id;
	AddTP(tp);
	AddChannels(tp, pat, sdt);
	
	free_sdt(&sdt);
	free_pat(&pat);
        return 0;
}

int DVB::scan_sat(Sat &sat) 
{
	struct Transponder tp;
        uint srates[2]= { 27500000, 22000000 }; 
	uint freq, i;
	int pol, res, ttk;
	uint srate=0, ofreq;
	Lnb &lnb=*sat.lnb;
	

	set_lnb(lnb.diseqcnr);
	ostrstream satname;
			
	satname << "on " << sat.name << ends;
	if(fdvb >= 0) osd.Text(100, 40, 2, 1, satname.str());

	for (pol=0; pol<2; pol++)
	for (freq=sat.fmin; freq<=sat.fmax; freq+=1000) {
	        if (freq<lnb.slof) {
		        ofreq=(freq-lnb.lof1)*1000;
			ttk=0;
		} else {
		        ofreq=(freq-lnb.lof2)*1000;
			ttk=1;
		}
		
		{
		        ostrstream fstr;
			fstr << dec << (freq/1000) << " MHz " << ends;
			cerr << fstr.str();
			if(fdvb >= 0) osd.Text(400, 40, 2, 1, fstr.str());
		}
		for (i=0, res=-1; i<2 && res<0; i++) {
		        srate=srates[i];
			res=set_tp(&ofreq, ttk, pol, srate);

			if (!res) {
			        ostrstream fstr;
				fstr << "Signal lock at " << dec << (freq/1000)
				     << " MHz" << ends;
				cerr << fstr.str() << endl;
				if(fdvb >= 0){
					osd.Text(0, 80, 2, 1, fstr.str());
					osd.FillBlock(0, 120, 300, 159, 0);
				}
				tp.name[0]=0;
				tp.id=NOID;
				tp.type=1;
				tp.freq=freq;
				tp.pol=pol;
				tp.qam=-1;
				tp.srate=srate;
				tp.fec=8;
				tp.satid=sat.id;
				scan_tp(tp);
				freq+=8000;
			}
		}
	}
	
	return 0;
}

int DVB::scan_cable(Sat &sat) 
{

        struct Transponder tp;
        uint srates[2]= { 6900000 }; 
	uint freq, i;
	int res, qam=2;
	uint srate=0, ofreq;
	
	for (freq=sat.fmin; freq<=sat.fmax; freq+=4000000) {
		{
		        ostrstream fstr;
			fstr << dec << (freq/1000) << " KHz" << ends;
			if(fdvb >= 0) osd.Text(400, 40, 2, 1, fstr.str());
		}
		for (i=0, res=-1; i<1 && res<0; i++) {
		        ofreq=freq;
		        srate=srates[i];
			res=set_tp(&ofreq, 0, 0, srate);

			if (!res) {
			        ostrstream fstr;
				fstr << "Signal lock at " << dec << (freq/1000)
				     << " KHz" << ends;
				if(fdvb >= 0){
					osd.Text(0, 80, 2, 1, fstr.str());
					osd.FillBlock(0, 120, 300, 159, 0);
				}
				tp.name[0]=0;
				tp.id=NOID;
				tp.type=2;
				tp.pol=-1;
				tp.freq=freq;
				tp.qam=qam;
				tp.srate=srate;
				tp.fec=8;
				tp.satid=sat.id;
				scan_tp(tp);
			}
		}
	}
	
	return 0;
}

int DVB::scan_lnb(struct Lnb &lnb) {
        int i;

	for (i=0; i<num[SAT]; i++) {
#ifdef NAPI
	        if (sats[i].lnbid==lnb.id)
			scan_sat(sats[i]);
#else
	        if (sats[i].lnbid==lnb.id)
		        switch (front.type) {
			case FRONT_DVBS:
			        scan_sat(sats[i]);
				break;
			case FRONT_DVBC:
			  //cerr << "scan_cable\n";
			        scan_cable(sats[i]);
				break;
			}
#endif			  
	}
	return 0;
}


int DVB::scan_all_tps() 
{
        int itp;
	
	if(fdvb >= 0){
		osd.Open(80, 100, 640, 400, 2, 0, 1);
		osd.SetColor(0, 80, 80, 80, 230);
		osd.SetColor(1, 40, 40, 40, 255);
		osd.SetColor(2, 0, 140, 0, 255);
		osd.Text(180, 0, 3, 1, "TP Scan");
	}

	get_front();
#ifdef NAPI
	set_apid(0);
	set_vpid(0);
#else
	front.pnr=0;
	front.video_pid=0;
	front.audio_pid=0;
	front.channel_flags=0;
#endif
	for (itp=0; itp<num[TRANS]; itp++) {
	  cerr << "TP " << dec << itp << "  id=" << hex << tps[itp].id << endl;
	  if (SetTP(tps[itp].id, tps[itp].satid))
	    continue;
	  set_front();
	  scan_tp(tps[itp]);
	}
	return 0;
}

int DVB::scan() {
        int i;
	
	if (!num[LNB])
	        return -1;
	
	if(fdvb >= 0){
		osd.Open(80, 100, 640, 400, 2, 0, 1);
		osd.SetColor(0, 80, 80, 80, 230);
		osd.SetColor(1, 40, 40, 40, 255);
		osd.SetColor(2, 0, 140, 0, 255);
		osd.Text(180, 0, 3, 1, "Channel Scan");
	}

#ifndef NAPI
	get_front();
#endif
	for (i=0; i<num[LNB]; i++) {
	        ostrstream lnbname;
#ifdef NAPI
		lnbname << "LNB" << HEX(1) << lnbs[i].id << ends;

#else
		lnbname << (front.type==FRONT_DVBS ? "LNB" : "CABLE" ) 
			<< HEX(1) << lnbs[i].id << ends;
#endif
		if(fdvb >= 0) osd.Text(0, 40, 2, 1, lnbname.str());

		if (scan_lnb(lnbs[i])<0){
		  cerr << "done" << endl;
		  return -1;
		}
	}
	cerr << "done" << endl;
	return 0;
}


int DVB::AddLNB(int id, int t, uint l1, uint l2, uint sl,
		int dnr, int dis, int sw)
{
        if (num[LNB]>=maxs[LNB])
	        return -1;
	
	cerr << "new LNB nr " << num[LNB] << endl;
	lnbs[num[LNB]].init(t, l1, l2, sl, dnr, dis, sw);
	lnbs[num[LNB]].id=id;
	num[LNB]++;
	return 0;
}

int DVB::AddTP(Transponder &tp)
{
        if (num[TRANS]>=maxs[TRANS])
	        return -1;

	if (!tp.id)
	        tp.id=num[TRANS];
	for (int i=0; i<num[TRANS]; i++)
	           if (tps[i].id==tp.id && tps[i].satid==tp.satid 
		       && tps[i].onid==tp.onid)
		        return -1;
	tps[num[TRANS]]=tp;
	num[TRANS]++;
	return 0;
}

int DVB::AddChannel(Channel &chan)
{
        int i;

        for (i=0; i<num[CHAN]; i++) {
 	        if (chan.pnr==chans[i].pnr && chan.tpid==chans[i].tpid) {
		        cerr << "Channel exists, only updating info\n";
			chans[i]=chan;
			return 0;
		}
	}
        if (num[CHAN]>=maxs[CHAN])
	        return -1;
	chan.id=num[CHAN];
	chans[num[CHAN]]=chan;
	num[CHAN]++;
	return 0;
}

int DVB::AddSat(int id, unsigned int lnbid, char *name, uint fmin, uint fmax)
{
        int i;

	if (num[SAT]==maxs[SAT]) 
	        return -1;
	for (i=0; i<num[LNB]; i++) {
	  if (lnbs[i].id==lnbid) {
		        sats[num[SAT]].id=id;
		        sats[num[SAT]].lnb=&lnbs[i];
		        sats[num[SAT]].lnbid=lnbs[i].id;
		        strncpy(sats[num[SAT]].name, name, maxname);
			sats[num[SAT]].name[maxname]=0;
			sats[num[SAT]].fmin=fmin;
			sats[num[SAT]].fmax=fmax;
			num[SAT]++;
			return 0;
		}
	}
	return -1;

}


ostream &operator<<(ostream &stream, DVB &x) {
  int i,j,k,l;
  
  for (i=0; i<x.num[LNB]; i++) {
    stream << x.lnbs[i];
    
    for (j=0; j<x.num[SAT]; j++)
      if (x.sats[j].lnbid==x.lnbs[i].id) {
	stream << x.sats[j];
	
	for (k=0; k<x.num[TRANS]; k++) 
	  if (x.tps[k].satid==x.sats[j].id) {
	    stream << x.tps[k];
	    
	    for (l=0; l<x.num[CHAN]; l++) 
	      if (x.chans[l].tpid==x.tps[k].id)
		stream << x.chans[l];
	  }
      }
  }
  return stream;
}


static char *names[] = {
        "LNB","DISEQC","ROTOR","TRANSPONDER","CHANNEL",
	"BOUQUET","SAT","SWITCH","NETWORK", NULL
};
int findkey(char *name, char *keys[]);

istream &operator>>(istream &ins, DVB &x)
{
  while(!ins.eof()){
    char keybuf[MAXNAM];
    ins >> keybuf;
    int n=findkey(keybuf, names);

    if (n<0) {
      cerr << "Error: " << keybuf << " is not a valid keyword at " 
	   << endl;
      exit(0);
    } else {
      if (x.num[n]< maxs[n]){
	//cerr << "number " << x.num[n] << " of type " << n << endl;
	x.num[n]++;
	switch (n){
	case LNB:
	  x.lnbs[x.num[n]-1].name[0]='\0';//shouldn`t be necessary, but is!?
	  ins >> x.lnbs[x.num[n]-1];
	  break;
	case DIS:
	  ins >> x.diseqcs[x.num[n]-1];
	  break;
	case ROTOR:
	  ins >> x.rotors[x.num[n]-1];
	  break;
	case TRANS:
	  ins >> x.tps[x.num[n]-1];
	  break;
	case CHAN:
	  ins >> x.chans[x.num[n]-1];
	  break;
	case BOU:
	  ins >> x.bouqs[x.num[n]-1];
	  break;
	case SAT:
	  ins >> x.sats[x.num[n]-1];
	  break;
	}
      } else {
	      cerr << "not enough channels" << endl;
	      break;
      }
    }
  } 
    
  return ins;
}




void hdump(unsigned char *buf, int n) 
{
	int i;
	
	for (i=0; i<n; i++)
		cerr << HEX(2) << (int) buf[i] << " ";
	cerr << endl;
}


void DVB::bar2(int x, int y, int w, int h, int val, int col1, int col2) 
{
	        int sep=(w*val)>>16;
		if(fdvb >= 0){
			osd.FillBlock(x, y, x+w-1-sep, y+h-1, col1);
			osd.FillBlock(x+w-1-sep, y, 515, y+h-1, col2);
		}
}
#ifndef NAPI

__u16 DVB::SetFilter(__u16 pid, __u16 section, __u16 mode) 
{ 
	struct bitfilter filt = {
		pid,
			{ section, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,  
				  0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000},
			mode,0,
				FILTER_MEM,
				{},
				};
	if (ioctl(fdvb, VIDIOCSBITFILTER, &filt)<0)
		return 0xffff;
	return filt.handle;
}

int DVB::SetFilter(uint16_t pid, unsigned char *filter, unsigned char *mask,
		   uint32_t timeout, uint32_t flags) 
{
	struct bitfilter filt = {
		pid, {},
			0, 0,
				FILTER_MEM,
				{},
				};
	int i;
	
	for (i=0; i<16; i++) 
		filt.data[i]=(filter[i]<<8)|mask[i];
	
	filt.mode=4;
	
	if (ioctl(fdvb, VIDIOCSBITFILTER, &filt)<0)
		return 0xffff;
	return filt.handle;
}

int DVB::CloseFilter(int h) 
{
	uint16_t handle=h;
	
	if (ioctl(fdvb, VIDIOCSSHUTDOWNFILTER, &handle)<0)
		return -1;
	return 0;
}

#else
__u16 DVB::SetFilter(__u16 pid, __u16 section, __u16 mode) 
{ 
	struct dmxSctFilterParams secFilterParams;
	int fd_section=open("/dev/ost/demux", O_RDWR);
	
	secFilterParams.pid=pid;
	memset(&secFilterParams.filter.filter, 0, DMX_FILTER_SIZE);
	memset(&secFilterParams.filter.mask, 0, DMX_FILTER_SIZE);
	secFilterParams.timeout = 0;
	secFilterParams.flags=DMX_IMMEDIATE_START;
	
	secFilterParams.filter.filter[0]=(section>>8)&0xff;
	secFilterParams.filter.mask[0]=section&0xff;
	
	if (ioctl(fd_section, DMX_SET_FILTER, &secFilterParams) < 0)  
		return 0xffff;
	return fd_section;
} 

int DVB::SetFilter(uint16_t pid, unsigned char *filter, 
		   unsigned char *mask,
		   uint32_t timeout, uint32_t flags) 
{
	
	struct dmxSctFilterParams secFilterParams;
	int fd_section=open("/dev/ost/demux", O_RDWR);
	
	secFilterParams.pid=pid;
	memset(&secFilterParams.filter.filter, 0, DMX_FILTER_SIZE);
	memset(&secFilterParams.filter.mask, 0, DMX_FILTER_SIZE);
	secFilterParams.timeout = timeout;
	secFilterParams.flags=DMX_IMMEDIATE_START;
	
	for (int i = 0; i < DMX_FILTER_SIZE; i++){
		secFilterParams.filter.filter[i]=filter[i];
		secFilterParams.filter.mask[i]=mask[i];
	}
	if (ioctl(fd_section, DMX_SET_FILTER, &secFilterParams) < 0)  
		return 0xffff;
	return fd_section;
}


int DVB::CloseFilter(int h) 
{
	close(h);
	return 0;
}

#endif


int DVB::GetSection(unsigned char *buf, ushort PID, unsigned char sec,
		    unsigned char secnum, unsigned char &msecnum) 
{
	int seclen=0;
	__u16 handle, pid;
	unsigned char section, sectionnum=0xff, maxsec=0;
	struct pollfd pfd;
	int loopc = 0;

	if ((handle=SetFilter(PID, (sec<<8)|0x00ff, 0))==0xffff)
		return -1;
	do {
		seclen=0;
#ifdef NAPI
		pfd.fd = handle;
#else
		pfd.fd=fsi;
#endif
		pfd.events=POLLIN;
		if (poll(&pfd, 1, 20000)==0) {
			cerr << "Timeout\n";
			break;
		}
		loopc++;
		
#ifndef NAPI
		read(fsi, buf, 8);
		seclen=(buf[6]<<8)|buf[7];
		seclen&=0xfff;
		pid=(buf[4]<<8)|buf[5];
		read(fsi, buf, seclen);
#else
		pid = PID;
		read(handle, buf, 3);
		seclen = 0;
		seclen |= ((buf[1] & 0x0F) << 8); 
		seclen |= (buf[2] & 0xFF);
		
		read(handle, buf+3, seclen);
		seclen+=3;
#endif	
		section=buf[0];
		sectionnum=buf[6];
		maxsec=buf[7];
	} while ( loopc < maxsec*2 && 
		 (section != sec || pid != PID || sectionnum != secnum));
	msecnum=maxsec;
	//cerr << "secnum: " << HEX(2) << (int)sectionnum 
	//     << ", secmax: " << HEX(2) << (int) msecnum << "\n";
	CloseFilter(handle);
	return seclen;
}

int DVB::GetSection(uint8_t *buf, 
		    uint16_t PID, uint8_t *filter, uint8_t *mask,
		    uint8_t secnum, uint8_t &msecnum) 
{
	int seclen=0;
	uint16_t handle, pid;
	uint8_t section, sectionnum=0xff, maxsec=0;
	struct pollfd pfd;
	int loopc = 0;

	if ((handle=SetFilter(PID, filter, mask, 0, 0))==0xffff)
		return -1;
	do {
		seclen=0;
#ifdef NAPI
		pfd.fd=handle;
#else
		pfd.fd=fsi;
#endif	
		pfd.events=POLLIN;
		if (poll(&pfd, 1, 20000)==0)
			break;
		loopc++;
		// read header
#ifndef NAPI
		read(fsi, buf, 8);
		seclen=(buf[6]<<8)|buf[7];
		pid=(buf[4]<<8)|buf[5];
		// read section
		read(fsi, buf, seclen);
#else
		pid = PID;
		read(handle, buf, 3);
		seclen = 0;
		seclen |= ((buf[1] & 0x0F) << 8); 
		seclen |= (buf[2] & 0xFF);
		
		read(handle, buf+3, seclen);
		seclen+=3;
#endif	
		
		
		section=buf[0];
		sectionnum=buf[6];
		maxsec=buf[7];
	} while (loopc < maxsec*2 && ((section&mask[0]!=filter[0]) ||
		 pid!=PID || sectionnum!=secnum));
	msecnum=maxsec;
	CloseFilter(handle);
	return seclen;
}

int DVB::GetSection(uint8_t *buf, 
		    uint16_t PID, uint8_t TID, uint16_t TIDExt, 
		    uint16_t FilterTIDExt, 
		    uint8_t secnum, uint8_t &msecnum) 
{
	uint8_t filter[16], mask[16];
	
	memset(filter, 0, 16);
	memset(mask, 0, 16);
	
	filter[0]=TID;
	mask[0]=0xff;
	if (TIDExt!=0xffff) {
		filter[1]=(TIDExt>>8);
		filter[2]=TIDExt&0xff;
		mask[1]=(FilterTIDExt>>8);
		mask[2]=FilterTIDExt&0xff;
		}
	
	return GetSection(buf, PID, filter, mask, secnum, msecnum);
}		

int DVB::setBitFilter(bitfilter bfilter, uint16_t PID,
		      uint8_t TID, uint16_t TIDExt, 
		      uint16_t FilterTIDExt) 
{
	
	uint8_t filter[16], mask[16];
	
	memset(filter, 0, 16);
	memset(mask, 0, 16);
	
	filter[0]=TID;
	mask[0]=0xff;
	if (TIDExt!=0xffff) {
		filter[1]=(TIDExt>>8);
		filter[2]=TIDExt&0xff;
		mask[1]=(FilterTIDExt>>8);
		mask[2]=FilterTIDExt&0xff;
	}
	
	SetFilter(PID, filter, mask, 0, 0);
	return 1;
}		

int DVB::GetSDT(dvb_sdt_t *sdt) 
{
	unsigned char buf[MAXSECSIZE], msec=0, sec=0;
	int ret=-1;
	
	
	while (sec<=msec) {
		if (GetSection(buf, PID_SDT, TID_SDT_ACT, sec, msec)>0)
			ret&=parse_sdt(buf, sdt);
		sec++;
	}
	return ret;
}

int DVB::GetPAT(dvb_pat_t *pat) 
{
	unsigned char buf[MAXSECSIZE], msec=0, sec=0;
	int ret=-1;
	
	while (sec<=msec) {
		if (GetSection(buf, PID_PAT, TID_PAT, sec, msec)>0)
			ret&=parse_pat(buf, pat);
		sec++;
	}
	return ret;
}

int DVB::GetPMT(dvb_pmt_t *pmt, unsigned short pid) 
{
	unsigned char buf[MAXSECSIZE], sec=0, msec=0;
	int ret=-1;
	while (sec<=msec) {
		if (GetSection(buf, pid, TID_PMT, sec, msec)>0)
			ret&=parse_pmt(buf, pmt);
		sec++;
	}
	return ret;
}

int DVB::GetNIT(dvb_nit_t *nit) 
{
	unsigned char buf[MAXSECSIZE], sec=0, msec=0;
	int ret=-1;
	while (sec<=msec) {
		if (GetSection(buf, PID_NIT, TID_NIT_ACT, sec, msec)>0)
			ret&=parse_nit(buf, nit);
		//hdump(buf, 0x120);
		sec++;
	}
	return ret;
}


void DVB::get_front(void) 
{
#ifdef NAPI
	set_vpid(0);
	set_apid(0);
	scmds.voltage = SEC_VOLTAGE_13; 
	scmds.miniCommand = SEC_MINI_NONE;
	scmds.continuousTone = SEC_TONE_OFF;
	scmds.numCommands=1;
	scmds.commands=&scmd;
	
	qpsk.iFrequency=(12666000-10600000);
	qpsk.SymbolRate=22000000;
	qpsk.FEC_inner=0;

#else
	ioctl(fdvb, VIDIOCGFRONTEND, &front);
#endif
}	        
  
#ifdef NAPI
feStatus DVB::tune_it(struct qpskParameters *qpsk)
{
	feStatus stat = 0;
	int res;
	struct qpskEvent event;
	int count = 0;

	if (ioctl(fd_frontend, QPSK_TUNE, qpsk) <0)
		perror("setfront front");
	if ( (res = ioctl(fd_frontend, QPSK_GET_EVENT, &event))  
	     == -EBUFFEROVERFLOW)
		res = ioctl(fd_frontend, QPSK_GET_EVENT, &event);
	if (res < 0)
		perror("qpsk ge event");
	
	while( !(stat & FE_HAS_SIGNAL) && count < 10){
		if (ioctl(fd_frontend, FE_READ_STATUS, &stat) < 0)
			perror("set_tp stat");
		count++;
	}
	return stat;
}
#endif

void DVB::set_front(void) 
{
#ifdef NAPI
	scmds.miniCommand = SEC_MINI_NONE;
	scmds.numCommands=1;
	scmds.commands=&scmd;
	feStatus stat = 0;
	int i;
	__u32 soff = qpsk.SymbolRate/16000;

	if (ioctl(fd_sec, SEC_SEND_SEQUENCE, &scmds) < 0)
		perror("setfront sec");
	udelay(100000);

	int freq = qpsk.iFrequency;
	//	cerr << freq << endl;
	
	stat = tune_it(&qpsk);

	for (i=2; i<10; i++) {    
		if (stat & FE_HAS_SIGNAL) {
			//cerr << "got signal" << endl;
			break;

		}
                if (i&1) 
                        qpsk.iFrequency = freq+soff*(i/2);
                else
                        qpsk.iFrequency = freq-soff*(i/2);

		stat = tune_it(&qpsk);
        }
	
#else
	ioctl(fdvb, VIDIOCSFRONTEND, &front);
#endif
}	        

#ifdef NAPI
void DVB::set_diseqc_nb(int nr) 
{
	scmd.type=0;
	scmd.u.diseqc.addr = 0x10;
	scmd.u.diseqc.cmd = 0x38;
	scmd.u.diseqc.numParams = 1;
	scmd.u.diseqc.params[0] = 0xF0 | ((nr * 4) & 0x0F) | 
	  (scmds.continuousTone == SEC_TONE_ON ? 1 : 0) |
	  (scmds.voltage==SEC_VOLTAGE_18 ? 2 : 0);
}
#endif

int DVB::set_lnb(int dis) 
{
#ifdef NAPI
	get_front();
	set_diseqc_nb(dis);		
	if(ioctl(fd_sec, SEC_SEND_SEQUENCE, &scmds) < 0)
		perror("set_lnb");
#else
	ioctl(fdvb, VIDIOCGFRONTEND, &front);
	front.diseqc=dis;
#endif		
	return 0;
}	        
  
void DVB::set_vpid(ushort vpid) 
{  
#ifdef NAPI
        if (vpid==0 || vpid==0xffff) {
	        ioctl(fd_demuxa, DMX_STOP, 0);
	        return;
	}

	pesFilterParamsV.pid     = vpid;
	pesFilterParamsV.input   = DMX_IN_FRONTEND; 
	pesFilterParamsV.output  = DMX_OUT_DECODER; 
	pesFilterParamsV.pesType = DMX_PES_VIDEO; 
	pesFilterParamsV.flags   = DMX_IMMEDIATE_START;
	if (ioctl(fd_demuxv, DMX_SET_PES_FILTER, 
		  &pesFilterParamsV) < 0)
		perror("set_vpid");
#else
	front.video_pid=vpid;
	ioctl(fdvb, VIDIOCSFRONTEND, &front);
#endif
}

void DVB::set_apid(ushort apid) 
{  
#ifdef NAPI
        if (apid==0 || apid==0xffff) {
	        ioctl(fd_demuxa, DMX_STOP, apid);
	        return;
	}
	pesFilterParamsA.pid = apid;
	pesFilterParamsA.input = DMX_IN_FRONTEND; 
	pesFilterParamsA.output = DMX_OUT_DECODER; 
	pesFilterParamsA.pesType = DMX_PES_AUDIO; 
	pesFilterParamsA.flags = DMX_IMMEDIATE_START;
	if (ioctl(fd_demuxa, DMX_SET_PES_FILTER, 
		  &pesFilterParamsA) < 0)
		perror("set_apid");
#else
	front.audio_pid=apid;
	ioctl(fdvb, VIDIOCSFRONTEND, &front);
#endif
}


int DVB::set_tp(uint *freq, int ttk, int pol, uint srate) 
{
#ifdef NAPI
	if (ttk) scmds.continuousTone = SEC_TONE_ON;
	else scmds.continuousTone = SEC_TONE_OFF;
	qpsk.iFrequency = *freq/1000;
	qpsk.SymbolRate = srate;
	if (pol) scmds.voltage=SEC_VOLTAGE_18;
	else scmds.voltage=SEC_VOLTAGE_13;
	qpsk.FEC_inner=0;
	set_vpid(0);  
	set_apid(0);
	if (ioctl(fd_sec, SEC_SEND_SEQUENCE, &scmds) < 0)
		perror("set_tp sec");
	feStatus stat = 0;

	stat = tune_it(&qpsk);
	
	if (stat & FE_HAS_SIGNAL) return 0;
	
#else
	front.qam=2;
	front.ttk=ttk;
	front.srate=srate;
	front.volt=pol;
	front.fec=8;
	front.AFC=0;
	front.video_pid=0;
	front.audio_pid=0;
	front.tt_pid=0;
	front.freq=*freq;
	front.channel_flags=DVB_CHANNEL_FTA;
	
	ioctl(fdvb, VIDIOCSFRONTEND, &front);
	udelay(400000);
	ioctl(fdvb, VIDIOCGFRONTEND, &front);
	/*
	  cout << "freq: " << dec << (*freq/1000+10600000) 
	  << "  sync: " << HEX(2) << front.sync 
	  << "  agc: " << HEX(4) << front.agc 
	  << "  nest: " << HEX(4) << front.nest 
	  << "\n";
	*/
	osd.Text(0, 230, 2, 1, "Signal Quality");
	bar2(200, 240, 256, 16, front.nest, 2, 1);
	if ((front.sync&0x07)==0x07) {
		/*
		  front.freq-=front.afc;
		  ioctl(fdvb, VIDIOCSFRONTEND, &front);
		  udelay(10000);
		  ioctl(fdvb, VIDIOCGFRONTEND, &front);
		*/
		//*freq=front.curfreq;
		return 0;
	}
#endif
		return -1;
}
