/*
 * dvb.c: driver for the Siemens Fujitsu DVB PCI card
 *
 * Copyright (C) 1999,2000 Ralph  Metzler 
 *                       & Marcus Metzler for convergence integrated media GmbH
 *
 *  originally based on code by:
 *
 * Copyright (C) 1998,1999 Christian Theiss <mistert@rz.fh-augsburg.de>
 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 * 

 * The author can be reached at ralph@convergence.de, 

 * the project's page is at http://linuxtv.org/dvb/
 */

//#define _NOHANDSHAKE

#define __KERNEL_SYSCALLS__
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/fs.h>
#include <linux/unistd.h>
#include <linux/byteorder/swabb.h>
#include <linux/poll.h>
#include <linux/slab.h>
#include <stdarg.h>

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/interrupt.h>
#include <linux/ptrace.h>
#include <linux/ioport.h>
#include <linux/in.h>
#include <linux/malloc.h>
#include <linux/string.h>
#include <linux/pci.h>
#include <asm/system.h>
#include <asm/bitops.h>
#include <asm/io.h>
#include <asm/dma.h>
#include <asm/semaphore.h>
#include <linux/init.h>

#include <linux/netdevice.h>
#include <linux/inetdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>


#include "tuner.h"
#include "dvb.h"
#include "dvb_dpram4k.h"
#include "dvb_comcode.h"
#include "dvb_types.h"

#include "saa7146_core.h"
#ifdef V4L2
#include "saa7146_v4l2.h"
#else
#include "saa7146_v4l.h"
#endif
#include "saa7146.h"
#include "i2c.h"

#ifdef MODULE
MODULE_PARM(debug,"i");
MODULE_PARM(fdebug,"i");
MODULE_PARM(vidmem,"l");
MODULE_PARM(vidlow,"l");
MODULE_PARM(outstream,"i");
MODULE_PARM(vidmode,"i");
MODULE_PARM(napi,"i");
MODULE_PARM(init_chan,"i");
#endif

static int dvb_register_frontend(struct dvb_struct *dvb);
static void net_ipmpe(struct net_device *dev, u8 *pkt, int pkt_len);
static void net_fsection(struct net_device *dev, u8 *pkt, u16 pkt_len, u16 port);

static int  outcom(struct dvb_struct *dvb, int type, int com, int num, ...);
static int  SetFront(struct dvb_struct *dvb, struct frontend *front);
static int SetPIDs(struct dvb_struct *dvb, u16 vpid, u16 apid, u16 ttpid, u16 subpid, u16 pcrpid);
static void SetMode(struct dvb_struct *dvb, int mode);

static void init_p2a(p2a_t *p);
static void init_a2p(a2p_t *p,
                     void (*write_cb)(u8 const *buf, 
                                      long int count, void *data), 
                     void *dataA, void *dataV, int type,u16 pidv,u16 pida );

static void av_pes_cb(u8 const *buf, long int count,void *data);
static void av_pes_cb_napi(u8 const *buf, long int count,void *data);
static void av_pes_to_pes(a2p_t *p, u8 *buf,int count);
static long kpes_to_av_pes(u8 const *buf, long int count, void * data);
static void es_to_av_pes(trans_t *t, const u8 *buf,long int l);

void twrite(u8 const *buf);
void init_p2t(p2t_t *p, dvb_demux_feed_t *feed);
long int find_pes_header(u8 const *buf, long int length, int *frags);
void pes_to_ts(u8 const *buf, long int length, u16 pid, p2t_t *p);
void p_to_t(u8 const *buf, long int length, u16 pid, u8 *counter, dvb_demux_feed_t *feed);



static unsigned short get_pid(u8 *pid);
static void init_trans(trans_t *p, dvb_demux_filter_t *filter,        
                       long (*fkt)(u8 const *, long int, void *), 
                       void *dummy);
static int set_trans_filt(trans_t *p, int filtn, u16 pid, u8 *mask, 
                   u8 *filt, int pes);

static void clear_trans_filt(trans_t *p,int filtn);
static int filt_is_set(trans_t *p, int filtn);
static int pes_is_set(trans_t *p, int filtn);
static int pes_is_started(trans_t *p, int filtn);
static int pes_is_start(trans_t *p, int filtn);
static int filt_is_ready(trans_t *p,int filtn);

static void trans_filt(const u8 *buf, int count, trans_t *p);
static void ts_filter(trans_t *p);
static void pes_filter(trans_t *p, int filtn, int off);
static void sec_filter(trans_t *p, int filtn, int off);
static int get_filt_buf(trans_t *p, int filtn, u8 **buf); 
static section_t *get_filt_sec(trans_t *p, int filtn); 

static int ts_av_pes(trans_t *t,const u8 *buf, int icount);


static u32 vidmem	= 0;
static u32 vidlow	= 0;

static int debug = 0;
#define dprintk	if (debug) printk

static int fdebug = 0;
#define fprintk	if (fdebug) printk

static int outstream=PS_STREAM;
static int vidmode=CVBS_RGB_OUT;
static int napi=1;
static int init_chan=0;


static int num_dvb = 0;

#define saacomm(x,y) dvb->saa->command(dvb->i2cbus, (x), (y))

struct dvb_struct dvbs[MAX_NUM_DVB];


/****************************************************************************
 * General helper functions
 ****************************************************************************/

/* taken from linux/drivers/sound/sound_firmware.c */
/* Maybe the kernel should offer this somewhere else, not just in soundcore */

static int errno;

static int do_firmread(const char *fn, char **fp)
{
        int fd;
	long l;
	char *dp;
       
	fd = open(fn, 0, 0);
	if (fd == -1)
	{
                dprintk(KERN_INFO "Unable to load '%s'.\n", fn);
		return 0;
	}
	l = lseek(fd, 0L, 2);
	if (l <= 0 || l > 300*1024)
	{
	        printk(KERN_INFO "Firmware too large '%s'\n", fn);
		sys_close(fd);
		return 0;
	}
	lseek(fd, 0L, 0);
	*fp= dp = vmalloc(l);
	if (dp == NULL)
	{
		printk(KERN_INFO "Out of memory loading '%s'.\n", fn);
		sys_close(fd);
		return 0;
	}
	if (read(fd, dp, l) != l)
	{
		printk(KERN_INFO "Failed to read '%s'.\n", fn);
		vfree(dp);
		sys_close(fd);
		return 0;
	}
	close(fd);
	*fp = dp;
	return (int) l;
}

static
int firmread(const char *fn, char **fp)
{
	int r;
	mm_segment_t fs = get_fs();
        
	set_fs(get_ds());
	r = do_firmread(fn, fp);
	set_fs(fs);
	return r;
}

static inline void ddelay(int i) 
{
        current->state=TASK_INTERRUPTIBLE;
        schedule_timeout((HZ*i)/100);
}


/****************************************************************************
 * GPIO and DEBI functions
 ****************************************************************************/

#define saaread(x)      saa7146_read(saamem, (x))
#define saawrite(x,y) 	saa7146_write(saamem, (y), (x))

inline static void 
setgpio(struct dvb_struct *dvb, int port, u32 data)
{
        u32 val;

        val=saa7146_read(dvb->saa->mem, GPIO_CTRL);
        val&=~(0xff << (8*(port)));
        val|=(data)<<(8*(port));
        saa7146_write(dvb->saa->mem, GPIO_CTRL, val);
}

/* This DEBI code is based on the Stradis driver 
   by Nathan Laredo <laredo@gnu.org> */

static int debiwait_maxwait = 5000;

static int wait_for_debi_done(struct dvb_struct *dvb)
{
        struct saa7146 *saa=dvb->saa;
        void *saamem=saa->mem;
	int i;

	/* wait for registers to be programmed */
	for (i = 0; i < 100000; i++)
                if (saaread(MC2) & 2)
                        break;;
	/* wait for transfer to complete */
	for (i = 0; i < 500000 &&
	     (saaread(PSR) & SPCI_DEBI_S); i++)
		saaread(MC2);
	if (i > debiwait_maxwait)
		printk("wait-for-debi-done maxwait: %d\n",
			debiwait_maxwait = i);
	
	if (i == 500000)
		return -1;
	return 0;
}

static int debiwrite(struct dvb_struct *dvb, u32 config, 
                     int addr, u32 val, int count)
{
        struct saa7146 *saa=dvb->saa;
        void *saamem=saa->mem;
	u32 cmd;

	if (count <= 0 || count > 32764)
		return -1;
	if (wait_for_debi_done(dvb) < 0)
		return -1;
	saawrite(config, DEBI_CONFIG);
	if (count <= 4)		/* immediate transfer */
		saawrite(val, DEBI_AD);
	else			/* block transfer */
		saawrite(virt_to_bus(saa->debi), DEBI_AD);
	saawrite((cmd = (count << 17) | (addr & 0xffff)), DEBI_COMMAND);
	saawrite((2 << 16) | 2, MC2);
	return 0;
}

static u32 debiread(struct dvb_struct *dvb, u32 config, int addr, int count)
{
        struct saa7146 *saa=dvb->saa;
        void *saamem=saa->mem;
	u32 result = 0;

	if (count > 32764 || count <= 0)
		return 0;
	if (wait_for_debi_done(dvb) < 0)
		return 0;
	saawrite(virt_to_bus(saa->debi), DEBI_AD);
	saawrite((count << 17) | 0x10000 | (addr & 0xffff),
		 DEBI_COMMAND);
	saawrite(config, DEBI_CONFIG);
	saawrite((2 << 16) | 2, MC2);
	if (count > 4)	
		return count;
	wait_for_debi_done(dvb);
	result = saaread(DEBI_AD);
        result &= (0xffffffffUL >> ((4-count)*8));
	return result;
}

/* DEBI during interrupt */

static void iwdebi(struct dvb_struct *dvb, u32 config, int addr, u32 val, int count)
{
        if (count>4 && val)
                memcpy(dvb->saa->debi, (char *) val, count);
        debiwrite(dvb, config, addr, val, count);
}

static u32 irdebi(struct dvb_struct *dvb, u32 config, int addr, u32 val, int count)
{
        u32 res;

        res=debiread(dvb, config, addr, count);
        if (count<=4) {
                memcpy(dvb->saa->debi, (char *) &res, count);
        }
        return res;
}

/* DEBI outside interrupts, only for count<=4! */

static void wdebi(struct dvb_struct *dvb, u32 config, 
                  int addr, u32 val, int count)
{
        unsigned long flags;

        spin_lock_irqsave(&dvb->debilock, flags);
        debiwrite(dvb, config, addr, val, count);
        spin_unlock_irqrestore(&dvb->debilock, flags);
}

static u32 rdebi(struct dvb_struct *dvb, u32 config, 
                 int addr, u32 val, int count)
{
        unsigned long flags;
        u32 res;

        spin_lock_irqsave(&dvb->debilock, flags);
        res=debiread(dvb, config, addr, count);
        spin_unlock_irqrestore(&dvb->debilock, flags);
        return res;
}


static inline char chtrans(char c)
{
        if (c<32 || c>126)
                c=0x20;
        return c;
}


/* handle mailbox registers of the dual ported RAM */

static inline void 
ARM_ResetMailBox(struct dvb_struct *dvb)
{
        unsigned long flags;

        spin_lock_irqsave(&dvb->debilock, flags);
        debiread(dvb, DEBINOSWAP, IRQ_RX, 2);
        debiwrite(dvb, DEBINOSWAP, IRQ_RX, 0, 2);
        spin_unlock_irqrestore(&dvb->debilock, flags);
}

static inline void 
ARM_ClearMailBox(struct dvb_struct *dvb)
{
        iwdebi(dvb, DEBINOSWAP, IRQ_RX, 0, 2);
}

static inline void 
ARM_ClearIrq(struct dvb_struct *dvb)
{
	irdebi(dvb, DEBINOSWAP, IRQ_RX, 0, 2);
}

static void reset_arm(struct dvb_struct *dvb)
{
        setgpio(dvb, RESET_LINE, GPIO_OUTLO);
        setgpio(dvb, RESET_LINE, GPIO_OUTHI);
        
        ddelay(50);

        printk("dvb: ARM RESET\n");

        ARM_ClearIrq(dvb);
}

static void arm_error(struct dvb_struct *dvb)
{
        dvb->arm_errors++;
        if (dvb->arm_errors<3)
                return;
        reset_arm(dvb);
        dvb->arm_errors=0;
}

static int AV_StartRecord(struct dvb_struct *dvb, int av)
{
        if (dvb->playing||(dvb->recording&av))
                return -EBUSY;

        outcom(dvb, COMTYPE_REC_PLAY, __Stop, 0);
        ARM_ResetMailBox(dvb); //FIXME: why is this needed with CI installed????
       
        init_a2p(&dvb->ps_ts_stream,av_pes_cb,(void *)dvb,
                 (void *)dvb, outstream, 0xa0, 0x50);
        outcom(dvb, COMTYPE_REC_PLAY, __Record, 2, AV_PES, 0);
        wdebi(dvb, DEBINOSWAP, IRQ_RX, 0, 2);
        dvb->recording|=av;
        return 0;
}

static int AV_StartRecordNAPI(struct dvb_struct *dvb, int av,
                              dvb_demux_feed_t *dvbdmxfeed)
{
        dvb_demux_t *dvbdmx=dvbdmxfeed->demux;
  
        if (dvb->playing||(dvb->recording&av))
                return -EBUSY;

        printk("pids %04x %04x\n", 
               dvbdmx->pesfilter[0]->pid,
               dvbdmx->pesfilter[1]->pid);
        dvbdmx->recording=1;
        init_a2p(&dvb->ps_ts_stream, 
                 av_pes_cb_napi,
                 (void *)dvbdmx->pesfilter[0],
                 (void *)dvbdmx->pesfilter[1],
                 (dvbdmxfeed->ts_type & 
                  TS_PAYLOAD_ONLY) ? 
                 PES_STREAM : TS_STREAM,
                 dvbdmx->pesfilter[1]->pid,
                 dvbdmx->pesfilter[0]->pid);
        outcom(dvb, COMTYPE_REC_PLAY, __Record, 2, AV_PES, 0);
        wdebi(dvb, DEBINOSWAP, IRQ_RX, 0, 2);
        dvb->recording|=av;
        return 0;
}

static int AV_StartPlay(struct dvb_struct *dvb, int av)
{
        if (dvb->recording)
                return -EBUSY;
        if (dvb->playing&av)
                return -EBUSY;

        outcom(dvb, COMTYPE_REC_PLAY, __Stop, 0);
        ARM_ResetMailBox(dvb); //FIXME: why is this needed with CI installed????

        if (dvb->playing == RP_NONE) {
                init_trans(&dvb->transi, dvb->p2a_filter, 
                           kpes_to_av_pes, (void *) dvb);
                init_p2a(&dvb->transi.pesto[0]);
                init_p2a(&dvb->transi.pesto[1]);
                dvb->transi.which_pesto = 0;
        }

        dvb->playing|=av;
        switch (dvb->playing) {
        case RP_AUDIO:
                outcom(dvb, COMTYPE_REC_PLAY, __Play, 2, AudioPES, 0);
                break;
        case RP_VIDEO:
                outcom(dvb, COMTYPE_REC_PLAY, __Play, 2, VideoPES, 0);
                break;
        case RP_AV:
                outcom(dvb, COMTYPE_REC_PLAY, __Play, 2, AV_PES, 0);
                break;
        }
        return dvb->playing;
}

static void AV_Stop(struct dvb_struct *dvb, int av)
{
        if (!(dvb->playing&av) && !(dvb->recording&av))
                return;

        outcom(dvb, COMTYPE_REC_PLAY, __Stop, 0);

        if (dvb->playing) {
                dvb->playing&=~av;
                switch (dvb->playing) {
                case RP_AUDIO:
                        outcom(dvb, COMTYPE_REC_PLAY, __Play, 2, AudioPES, 0);
                        break;
                case RP_VIDEO:
                        outcom(dvb, COMTYPE_REC_PLAY, __Play, 2, VideoPES, 0);
                        break;
                case RP_NONE:
                        SetMode(dvb, dvb->vidmode);
                        break;
                }
        } else {
                dvb->recording&=~av;
                switch (dvb->recording) {
                case RP_AUDIO:
                        outcom(dvb, COMTYPE_REC_PLAY, __Record, 2, AudioPES, 0);
                        break;
                case RP_VIDEO:
                        outcom(dvb, COMTYPE_REC_PLAY, __Record, 2, VideoPES, 0);
                        break;
                }
        }
}

/****************************************************************************
 * Buffer handling
 ****************************************************************************/

static inline void 
dvb_buf_init(struct dvb_buf *dbuf, u8 *data, int len)
{
        dbuf->head=dbuf->tail=0;
        dbuf->buf=data;
        dbuf->length=len;
        init_waitqueue_head(&dbuf->queue);
        spin_lock_init (&(dbuf->lock));
        dbuf->lock=SPIN_LOCK_UNLOCKED;
}

static inline 
int dvb_buf_empty(dvb_buf_t *buf)
{
        return (buf->head==buf->tail);
}

static inline 
int dvb_buf_free(dvb_buf_t *buf)
{
        int free;
  
        free=buf->head - buf->tail;
        if (free<=0)
                free+=buf->length;
        return free;
}

static inline 
int dvb_buf_avail(dvb_buf_t *buf)
{
        int avail;
  
        avail=buf->tail - buf->head;
        if (avail<0)
                avail+=buf->length;
        return avail;
}

static long 
dvb_buf_read(dvb_buf_t *dbuf, 
             char *buf, unsigned long count, 
             int nonblock, int usermem)
{
        unsigned long todo=count;
        int free, split;
        
	if (dvb_buf_empty(dbuf) && nonblock)
	        return -EWOULDBLOCK;

        while (todo > 0) {
                if (dvb_buf_empty(dbuf)) {
                        if (nonblock)
                                return count-todo;
                        if (wait_event_interruptible(dbuf->queue,
                                                     (dbuf->head!=dbuf->tail))<0)
                                return count-todo;
                }
                split=dbuf->length;
                free=dbuf->tail - dbuf->head;
                if (free < 0) {
                        free+=dbuf->length;
                        split=dbuf->length - dbuf->head;
                }
                if (free > todo)
                        free=todo;
                if (split < free) {
                        if (usermem) { 
                                if (copy_to_user(buf, dbuf->buf+dbuf->head, split))
                                        return -EFAULT;
                        } else
                                memcpy(buf, dbuf->buf+dbuf->head,split);
                        buf+=split;
                        dbuf->head=0;
                        todo-=split;
                        free-=split;
                }
                if (free) {
                        if (usermem) {
                                if (copy_to_user(buf, dbuf->buf+dbuf->head, free))
                                        return -EFAULT;
                        } else
                                memcpy(buf, dbuf->buf+dbuf->head, free);
                                  
                        dbuf->head = (dbuf->head + free) % dbuf->length;
                        todo-=free;
                        buf+=free;
                }
        }
        return count;
}

static long 
dvb_buf_write(dvb_buf_t *dbuf, 
              const char *buf, unsigned long count, 
              int nonblock, int usermem)
{
        unsigned long todo = count;
        int free, split;
    
        while (todo > 0) {
                if (dvb_buf_free(dbuf)<=2) {
                        if (nonblock)
                                return count-todo;
                        if (wait_event_interruptible(dbuf->queue,
                                                     (dvb_buf_free(dbuf)>2)))
                        	return count-todo;
                }   
                free = dbuf->head - dbuf->tail;
                split=dbuf->length;
                if (free<=0) {
                        free+=dbuf->length;
                        split-=dbuf->tail;
                }
                free-=2; 

                if (free > todo)
                        free = todo;
                        
                if (split < free) {
                        if (!usermem) 
                                memcpy(dbuf->buf+dbuf->tail, buf, split);
                        else
                                if (copy_from_user(dbuf->buf+dbuf->tail,
                                                   buf, split)) 
                                        return -EFAULT;
                        buf += split;
                        todo -= split;
                        free -= split;
                        dbuf->tail = 0;
                }
                if (!usermem) 
                        memcpy(dbuf->buf+dbuf->tail, buf, free);
                else
                        if (copy_from_user(dbuf->buf+dbuf->tail, buf, free)) 
                                return -EFAULT;
                dbuf->tail = (dbuf->tail + free)%dbuf->length;
                todo -= free;
                buf += free;
        }

	return count-todo;
}

static void
dvb_buf_put(struct dvb_buf *db, u8 *buf, int len)
{
        int split, fsize;

        fsize=db->head - db->tail;
        if (fsize <= 0) {
                fsize+=db->length;
                split=db->length-db->tail;
        } else 
                split=0;
        if (len>=fsize) {
          //printk("buffer overflow\n");
                return;
        }
        if (split>=len)
                split=0;
        if (split) {
                memcpy(db->buf + db->tail, buf, split);
                len-=split;
                db->tail=0;
        }
        memcpy(db->buf + db->tail, split + buf, len);
        db->tail=(db->tail+len)%db->length;
}


static int 
TTBStop(struct dvb_struct *dvb)
{
        printk ("function : %s\n", __FUNCTION__);
        if (--dvb->feeding)
                return dvb->feeding;
        saa7146_write(dvb->saa->mem, MC1, MASK_20); // DMA3 off
        saa7146_write(dvb->saa->mem, MC1, MASK_28); // RPS0 off
        saa7146_write(dvb->saa->mem, IER, 
                      saa7146_read(dvb->saa->mem, IER) & ~MASK_10 );
        saa7146_write(dvb->saa->mem, IER, 
                      saa7146_read(dvb->saa->mem, IER)& ~MASK_07);
        return 0;
}

#define TS_WIDTH  (4*188)
#define TS_HEIGHT (1024/4)
static int
TTBStart(struct dvb_struct *dvb)
{
        struct saa7146 *saa=dvb->saa;

        printk ("function : %s\n", __FUNCTION__);
        if (dvb->feeding++)
                return dvb->feeding;

      	saa7146_write(saa->mem, MC1, MASK_20); // DMA3 off

        memset(saa->grabbing, 0x00, TS_HEIGHT*TS_WIDTH);

        saa7146_write(saa->mem, PCI_BT_V1, 0x001c0000);

        dvb->tsf=0;
        saa7146_write(dvb->saa->mem, DD1_INIT, 0x02000680);
        saa7146_write(dvb->saa->mem, MC2, 
                      (MASK_09 | MASK_25 | MASK_10 | MASK_26));

        saa7146_write(saa->mem, BRS_CTRL, 0x60000000);	
      	saa7146_write(saa->mem, MC2, (MASK_08 | MASK_24));
        ddelay(1);

#if 1
        saa7146_write(saa->mem, BASE_ODD3, 0);
        saa7146_write(saa->mem, BASE_EVEN3, TS_WIDTH*TS_HEIGHT/2);
        saa7146_write(saa->mem, PROT_ADDR3, TS_WIDTH*TS_HEIGHT);	
        saa7146_write(saa->mem, BASE_PAGE3, virt_to_bus(saa->page_table[0])|ME1|0xb0);
        saa7146_write(saa->mem, PITCH3, TS_WIDTH);	
#else
        saa7146_write(saa->mem, BASE_ODD3, 0xceb3e000);
        saa7146_write(saa->mem, BASE_EVEN3, 1024+0xceb3e000+2048*TS_HEIGHT/2);
        saa7146_write(saa->mem, PROT_ADDR3, 0xceb3e000+2048*TS_HEIGHT+1024);	
        saa7146_write(saa->mem, BASE_PAGE3, 0xb0);
        saa7146_write(saa->mem, PITCH3, 2048);	
#endif
        saa7146_write(saa->mem, NUM_LINE_BYTE3, ((TS_HEIGHT/2)<<16)|TS_WIDTH);
      	saa7146_write(saa->mem, MC2, (MASK_04 | MASK_20));

        // VPE
        saa7146_write(dvb->saa->mem, IER, saa7146_read(dvb->saa->mem, IER)|MASK_10);

     	saa7146_write(saa->mem, MC1, (MASK_04 | MASK_20)); // DMA3 on

        // FIDB
        saa7146_write(dvb->saa->mem, IER, saa7146_read(dvb->saa->mem, IER)|MASK_07);
        return dvb->feeding;
}


/****************************************************************************
 * IRQ handling
 ****************************************************************************/


/* data format of fsection buffer: (all big endian)

   u16  ID="SI"
   u16  filter handle
   u16  PID  
   u16  data length
   u8[] data

 */

static void fsection(struct dvb_struct *dvb, int handle, u8 *data, u16 len)
{
        int flen;
        u8 header [8]={ 
          'S', 'I', (handle>>8), handle&0xff, (handle>>8), handle&0xff, 
          (len>>8), (len&0xff)
        };

#if 0
        dvb_demux_filter_t *dvbdmxfilter;

        if ((handle<MAXFILT) && dvb->handle2filter[handle]) {
                dvbdmxfilter=dvb->handle2filter[handle];
                header[4]=(dvbdmxfilter->pid>>8);
                header[5]=(dvbdmxfilter->pid&0xff);
                
                dvbdmxfilter->feed->cb.sec(data, len, 0, 0, 
                                           &dvbdmxfilter->filter, DMX_OK); 
        }

#endif
        /*
        if ((handle<MAXFILT) && dvb->hfilter[handle]) {
                header[4]=(dvb->hfilter[handle]->pid>>8);
                header[5]=(dvb->hfilter[handle]->pid&0xff);
        } 
        */
        if ((handle>=MAXFILT) || !dvb->hfilter[handle])
                return;
        header[4]=(dvb->hfilter[handle]->pid>>8);
        header[5]=(dvb->hfilter[handle]->pid&0xff);
        flen = dvb->fsec.head - dvb->fsec.tail;
        if (flen <= 0) 
                flen += dvb->fsec.length;
        if (flen<len+12)
                return;

        dvb_buf_put(&dvb->fsec, header, 8);
        dvb_buf_put(&dvb->fsec, data, len);

        dprintk("head: %08x, ", dvb->fsec.head);
        dprintk("tail: %08x, ", dvb->fsec.tail);
        dprintk("len: %08x, ", len);
        dprintk("\n");

        wake_up_interruptible(&dvb->fsec.queue);
        return;
}
/*
static void CI_out(struct dvb_struct *dvb, u8 *data, u16 len)
{
        int flen;
        u8 header[8]={ 
          'S', 'I', 0, 0, 0xcc, 0xcc, 
          (len>>8), (len&0xff)
        };
      
        flen = dvb->fsec.head - dvb->fsec.tail;
        if (flen <= 0) 
                flen += dvb->fsec.length;
        if (flen<len+12)
                return;

        dvb_buf_put(&dvb->fsec, header, 8);
        dvb_buf_put(&dvb->fsec, data, len);

        wake_up_interruptible(&dvb->fsec.queue);
        return;
}
*/
void CI_handle(struct dvb_struct *dvb, u8 *data, u16 len) 
{
  //CI_out(dvb, data, len);

        if (len<3)
                return;
        switch (data[0]) {
        case CI_MSG_CI_INFO:
                if (data[2]!=1 && data[2]!=2)
                        break;
                switch (data[1]) {
                case 0:
                        dvb->ci_slot[data[2]-1]=0;
                        break;
                case 1:
                        dvb->ci_slot[data[2]-1]|=CI_MODULE_PRESENT;
                        break;
                case 2:
                        dvb->ci_slot[data[2]-1]|=CI_MODULE_OK;
                        break;
                }
                break;
        case CI_SWITCH_PRG_REPLY:
                dvb->ci_stat=data[1];
                break;
        default:
                break;
        }

}

static inline int
DvbDmxFilterCallback(__u8 * buffer1, size_t buffer1_len,
                     __u8 * buffer2, size_t buffer2_len,
                     dvb_demux_filter_t *dvbdmxfilter,
                     dmx_success_t success,
                     struct dvb_struct *dvb)
{
        switch(dvbdmxfilter->type) {
        case DMX_TYPE_SEC:
                return dvbdmxfilter->feed->cb.sec(buffer1, buffer1_len,
						  buffer2, buffer2_len,
						  &dvbdmxfilter->filter,
						  DMX_OK); 
        case DMX_TYPE_TS:
                if (!(dvbdmxfilter->feed->ts_type & TS_PACKET)) 
                        return 0;
                if (dvbdmxfilter->feed->ts_type & TS_PAYLOAD_ONLY) 
                        return dvbdmxfilter->feed->cb.ts(buffer1, buffer1_len,
                                                         buffer2, buffer2_len,
                                                         &dvbdmxfilter->feed->feed.ts,
                                                         DMX_OK); 
                else
                        pes_to_ts(buffer1, buffer1_len, 
                                  dvbdmxfilter->feed->pid, 
                                  &dvb->p2t_filter[dvbdmxfilter->index]);
	default:
	        return 0;
        }
}


u8 pshead[0x26] = {
        0x00, 0x00, 0x01, 0xba, 0x5f, 0xff, 0xfe, 0xe6, 
        0xc4, 0x01, 0x01, 0x89, 0xc3, 0xf8, 0x00, 0x00,
        0x01, 0xbb, 0x00, 0x12, 0x80, 0xc4, 0xe1, 0x00,
        0xe1, 0xff, 0xb9, 0xe0, 0xe8, 0xb8, 0xc0, 0x20,
        0xbd, 0xe0, 0x44, 0xbf, 0xe0, 0x02,
};								


static void vpeirq(struct saa7146* saa, void *data)
{
        struct dvb_struct *dvb=(struct dvb_struct *) data;
  
        if (dvb->cardtype!=DVB_CARD_TT_BUDGET)
                return;
        /*
        dvb_buf_put(&dvb->avin, 
                    (u8 *)dvb->saa->grabbing,
                    TS_HEIGHT*TS_WIDTH);
        wake_up_interruptible(&dvb->avin.queue);
        */
        //printk("vpeirq %08x\n", saa7146_read(dvb->saa->mem, PCI_VDP3));
}

static void fidbirq(struct saa7146* saa, void *data)
{
        struct dvb_struct *dvb=(struct dvb_struct *) data;
        u8 *mem;
  
        if (dvb->cardtype!=DVB_CARD_TT_BUDGET)
                return;

        mem=(dvb->tsf ? TS_HEIGHT*TS_WIDTH/2 :0)+(u8 *)dvb->saa->grabbing;

        if (dvb->recording && mem[0]==0x47)
                dvb_buf_put(&dvb->avin, mem, TS_HEIGHT*TS_WIDTH/2);
        wake_up_interruptible(&dvb->avin.queue);

        dvb->tsf^=1;
        saa7146_write(dvb->saa->mem, DD1_INIT, 0x02000600|(dvb->tsf ? 0x40:0x80));
        saa7146_write(dvb->saa->mem, MC2, 
                      (MASK_09 | MASK_25 | MASK_10 | MASK_26));

        // FIXME: use bottom half or tasklet
        if (dvb->feeding && mem[0]==0x47)
                DvbDmxSWFilterPackets(&dvb->hw_demux, mem, 512);
}

static void debiirq(struct saa7146* saa, void *data)
{
        struct dvb_struct *dvb=(struct dvb_struct *) data;
        int type=dvb->debitype;
        int handle=(type>>8)&0x1f;
        
        dprintk("DEBI irq\n");        

        saa7146_write(dvb->saa->mem, IER, 
                      saa7146_read(dvb->saa->mem, IER) & ~MASK_19 );
        saa7146_write(dvb->saa->mem, ISR, MASK_19 );

        if (type==-1) {
                printk("DEBI irq oops\n");
                ARM_ClearMailBox(dvb);
                ARM_ClearIrq(dvb);
                return;
        }

        dvb->debitype=-1;

        switch (type&0xff) {
        case DATA_MPEG_RECORD:
        {
                // new API filter?
                if (dvb->hw_demux.recording) {
                        av_pes_to_pes(&(dvb->ps_ts_stream), 
                                      (u8 *)dvb->saa->debi,
                                      dvb->debilen);
                        break;
                }

                // old API filter?
                if (dvb->ps_ts_stream.type != AV_PES_STREAM)
                        av_pes_to_pes(&(dvb->ps_ts_stream), 
                                      (u8 *)dvb->saa->debi,
                                      dvb->debilen);
                else {
                        //dvb_buf_put(&dvb->avin, pshead, 0x26);
                        dvb_buf_put(&dvb->avin, (u8 *)dvb->saa->debi, 
                                    dvb->debilen);
                }
                wake_up_interruptible(&dvb->avin.queue);
                break;
        }

        case DATA_FSECTION:
        case DATA_PIPING:
                // new API filter?
                if ((handle<DVB_DEMUX_FILTER_MAX) && dvb->handle2filter[handle]) {
                        DvbDmxFilterCallback((u8 *)dvb->saa->debi, dvb->debilen, 0, 0, 
                                             dvb->handle2filter[handle], DMX_OK, dvb); 
                        break;
                }

                // old API filter?
                if ((handle>=MAXFILT) || !dvb->hfilter[handle])
                        break;
                if (dvb->hfilter[handle]->flags&FILTER_UDP)
                        net_fsection(&dvb->net, (void*)dvb->saa->debi, 
                                     dvb->debilen, 3000+handle);
                if (dvb->hfilter[handle]->flags&FILTER_MEM)
                        fsection(dvb, handle, (void*)dvb->saa->debi, dvb->debilen);
                
                break;

        case DATA_IPMPE:
                if ((handle>=MAXFILT) || !dvb->hfilter[handle])
                        break;
                if (dvb->hfilter[handle]->flags&FILTER_MEM)
                        fsection(dvb, handle, (void*)dvb->saa->debi, dvb->debilen);
                net_ipmpe(&dvb->net, (void*)dvb->saa->debi, dvb->debilen);
                break;

        case DATA_COMMON_INTERFACE:
                CI_handle(dvb, (u8 *)dvb->saa->debi, dvb->debilen);
#if 0
        {
                int i;

                printk("%02x ", *(u8 *)dvb->saa->debi);
                printk("%02x ", *(1+(u8 *)dvb->saa->debi));
                for (i=2; i<dvb->debilen; i++)
                  printk("%02x ", (*(i+(char *)dvb->saa->debi)));
                for (i=2; i<dvb->debilen; i++)
                  printk("%c", chtrans(*(i+(char *)dvb->saa->debi)));
                printk("\n");
        }
#endif
                break;
        case DATA_TELETEXT:
        {
                fsection(dvb, handle, (void*)dvb->saa->debi, dvb->debilen);
                break;
        }


        case DATA_MPEG_PLAY:
        case DATA_BMP_LOAD:
        default:
                break;
        }
        spin_lock(&dvb->debilock);
        ARM_ClearMailBox(dvb);
        spin_unlock(&dvb->debilock);
}

static int
avpes_play(void *dest, dvb_buf_t *buf, int dlen)
{
        int len, split=0;
        u16 sync, blen;
        
        if (!dlen || (dvb_buf_avail(buf) < 2)) 
                return -1;

        sync=((u16)*(u8 *)(buf->buf+((buf->head)%buf->length)))<<8;
        sync|=*(u8 *)(buf->buf+((buf->head+1)%buf->length));
        
        while (sync!=0x4156 && !dvb_buf_empty(buf)) {
                buf->head=(buf->head+1)%buf->length;
                sync=((u16)*(u8 *)(buf->buf+((buf->head)%buf->length)))<<8;
                sync|=*(u8 *)(buf->buf+((buf->head+1)%buf->length));
        } 
        
        len=dvb_buf_avail(buf);
        if (len<8)
                return -1;
                
        blen=((u16)*(u8 *)(buf->buf+((buf->head+6)%buf->length)))<<8;
        blen|=*(u8 *)(buf->buf+((buf->head+7)%buf->length));
        blen+=8;
        if (len<blen || blen > dlen) {
                dprintk("buffer empty\n");
                return -1;
        }
        if (blen>DATA_BUFF_SIZE) {
                if (!dvb_buf_empty(buf))
                        buf->head=(buf->head+1)%buf->length;
                dprintk("packet too large\n");
                return -1;
        }
        len=blen;
        if (buf->head + blen > buf->length)
                split=buf->length-buf->head;
        if (split>0) {
                memcpy(dest, buf->buf+buf->head, split);
                buf->head=0;
                len-=split;
        }
        memcpy(split + dest, 
               buf->buf + buf->head, len);
        buf->head = (buf->head +len)%buf->length;
        
        wake_up_interruptible(&buf->queue);
        return blen;
}


static void gpioirq(struct saa7146* saa, void *data)
{
        struct dvb_struct *dvb=(struct dvb_struct *) data;
        
        dprintk("GPIO0 irq\n");        

        if (dvb->debitype !=-1)
                printk("GPIO0 irq oops\n");
       
	ARM_ClearIrq(dvb);

        spin_lock(&dvb->debilock);
        saa7146_write(dvb->saa->mem, IER, 
                      saa7146_read(dvb->saa->mem, IER) & ~MASK_19 );
        saa7146_write(dvb->saa->mem, ISR, MASK_19 );

        dvb->debitype = irdebi(dvb, DEBINOSWAP, IRQ_STATE, 0, 2);
        dvb->debilen  = irdebi(dvb, DEBINOSWAP, IRQ_STATE_EXT, 0, 2);

        dprintk("GPIO0 irq %d %d\n", dvb->debitype, dvb->debilen);

        switch (dvb->debitype&0xff) {
        case DATA_MPEG_RECORD:
        {
                int len;

                len=(dvb->debilen+3)&(~3);
                if (dvb_buf_free(&dvb->avin) <= len) {
                        printk("record buffer full\n");
                        break;
                }

                saa7146_write(dvb->saa->mem, IER, 
                              saa7146_read(dvb->saa->mem, IER) | MASK_19 );
                irdebi(dvb, DEBISWAB, DATA_BUFF_BASE, 0, len);
                spin_unlock(&dvb->debilock);
                return;
        }

        case DATA_MPEG_PLAY:
        {
                int len;

                if (!dvb->playing) {
                        dprintk("not playing\n");
                        iwdebi(dvb, DEBINOSWAP, IRQ_STATE_EXT, 0, 2);
                        break;
                }

                spin_lock(&dvb->avout.lock);
                //#define NEW_OUT
#ifdef NEW_OUT
                //printk("debitype  %04x\n",dvb->debitype);
                if (dvb->debitype&0x200) {
                        len=avpes_play(dvb->saa->debi, &dvb->vout, dvb->debilen);
                        if (len<0 && dvb->debitype&0x100)
                          len=avpes_play(dvb->saa->debi, &dvb->aout, dvb->debilen);
                } else {
                        len=avpes_play(dvb->saa->debi, &dvb->aout, dvb->debilen);
                        if (len<0 && dvb->debitype&0x200)
                          len=avpes_play(dvb->saa->debi, &dvb->vout, dvb->debilen);
                }
#else
                len=avpes_play(dvb->saa->debi, &dvb->avout, dvb->debilen);
#endif
                spin_unlock(&dvb->avout.lock);

                if (len<0) {
                        iwdebi(dvb, DEBINOSWAP, IRQ_STATE_EXT, 0, 2);
                        break;
                } else
                        iwdebi(dvb, DEBINOSWAP, IRQ_STATE_EXT, len, 2);

                wait_for_debi_done(dvb);
                saa7146_write(dvb->saa->mem, IER, 
                              saa7146_read(dvb->saa->mem, IER) | MASK_19 );
                iwdebi(dvb, DEBISWAB, DATA_BUFF_BASE, 0, len);
                spin_unlock(&dvb->debilock);
                return;
        }

        case DATA_BMP_LOAD:
        {
                int len=dvb->debilen;
                
                if (!len) {
                        dvb->bmp_state=BMP_LOADED;
                        wake_up_interruptible(&dvb->bmpq);
                        break;
                }
                if (len>dvb->bmplen)
                        len=dvb->bmplen;
                if (len>6*1024)
                        len=6*1024;
                iwdebi(dvb, DEBINOSWAP, IRQ_STATE_EXT, len, 2);
                memcpy(dvb->saa->debi, dvb->bmpbuf+dvb->bmpp, len);
                dvb->bmpp+=len;
                dvb->bmplen-=len;
                wait_for_debi_done(dvb);
                saa7146_write(dvb->saa->mem, IER, 
                              saa7146_read(dvb->saa->mem, IER) | MASK_19 );
                iwdebi(dvb, DEBISWAB, DATA_BUFF_BASE, 0, len);
                spin_unlock(&dvb->debilock);
                return;
        }

        case DATA_COMMON_INTERFACE:
        case DATA_TELETEXT:
        case DATA_FSECTION:
        case DATA_DEBUG_MESSAGE:
        case DATA_IPMPE:
        case DATA_PIPING:
        {
                u16 len=(dvb->debilen+3)&(~3);
                
                if (!len || len>6*1024) 
                        break;
                
                saa7146_write(dvb->saa->mem, IER, 
                              saa7146_read(dvb->saa->mem, IER) | MASK_19);
                irdebi(dvb, DEBISWAB, DATA_BUFF_BASE, 0, len);
                spin_unlock(&dvb->debilock);
                return;
        }
        default:
                printk("gpioirq %d %d\n", dvb->debitype, dvb->debilen);
                break;
        }      
        ARM_ClearMailBox(dvb);
        dvb->debitype=-1;
        spin_unlock(&dvb->debilock);
}



/****************************************************************************
 * DEBI command polling 
 ****************************************************************************/


static int OutCommand(struct dvb_struct *dvb, u16* buf, int length)
{
        int i;
        u32	start;

        start = jiffies;
        while ( rdebi(dvb, DEBINOSWAP, COMMAND, 0, 2 ) )
        {
                ddelay(1);
                if ((jiffies - start) > HZ) {
                        printk("outcommand error 1\n");
                        arm_error(dvb);
                        return -1;
                }
        }

#ifndef _NOHANDSHAKE
        start = jiffies;
        while ( rdebi(dvb, DEBINOSWAP, HANDSHAKE_REG, 0, 2 ) ) 
        {
                ddelay(1);
                if ((jiffies - start) > HZ/5) {
                        printk("outcommand error 2\n");
                        arm_error(dvb);
                        return -1;
                }
        }
#endif

        start = jiffies;
        while ( rdebi(dvb, DEBINOSWAP, MSGSTATE, 0, 2) & OSDQFull )
        {
                ddelay(1);
                if ((jiffies - start) > HZ/2)
                {
                        printk("outcommand error 3\n");
                        arm_error(dvb);
                        return -1;
                }
        }
        for (i=2; i<length; i++)
                wdebi(dvb, DEBINOSWAP, COMMAND + 2*i, (u32) buf[i], 2);

        if (length)
                wdebi(dvb, DEBINOSWAP, COMMAND + 2, (u32) buf[1], 2);
        else
                wdebi(dvb, DEBINOSWAP, COMMAND + 2, 0, 2);
        wdebi(dvb, DEBINOSWAP, COMMAND, (u32) buf[0], 2);
        return 0;
}

inline static int 
SOutCommand(struct dvb_struct *dvb, u16* buf, int length)
{
        int ret;
        
        down_interruptible(&dvb->dcomlock);
        ret=OutCommand(dvb, buf, length);
        up(&dvb->dcomlock);
        return ret;
}


static int outcom(struct dvb_struct *dvb, int type, int com, int num, ...)
{
	va_list args;
        u16 buf[num+2];
        int i;

        buf[0]=(( type << 8 ) + com); 
        buf[1]=num;

        if (num) 
        {
                va_start(args, num);
                for (i=0; i<num; i++)
                        buf[i+2]=va_arg(args, u32);
                va_end(args);
        }

        return SOutCommand(dvb, buf, num+2);
}

int SendCICommand(struct dvb_struct *dvb, u8 subcom, u8 *Params, u8 ParamLen)
{
        int i;
        u16 CommandBuffer[18] = { ((COMTYPE_COMMON_IF << 8) + subcom),
                                  16, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
        
	for(i=0; (i<ParamLen)&&(i<32); i++)	
	{
		if(i%2 == 0)
			CommandBuffer[(i/2)+2] = (u16)(Params[i]) << 8;
		else
			CommandBuffer[(i/2)+2] |= Params[i];
	}

        return SOutCommand(dvb, CommandBuffer, 18);
}


static int CommandRequest(struct dvb_struct *dvb, u16 *Buff, int length, u16 *buf, int n)
{
        s16 i;
        u32 start;
       
        down_interruptible(&dvb->dcomlock);
        OutCommand(dvb, Buff, length);
        
        start = jiffies;
        while ( rdebi(dvb, DEBINOSWAP, COMMAND, 0, 2) ) 
        {
#ifdef _NOHANDSHAKE
                ddelay(1);
#endif
                if ((jiffies - start) > HZ/2) {
                        printk("commandrequest error\n");
                        up(&dvb->dcomlock);
                        arm_error(dvb);
                        return -1;
                }
        }
        
#ifndef _NOHANDSHAKE
        start = jiffies;
        while ( rdebi(dvb, DEBINOSWAP, HANDSHAKE_REG, 0, 2 ) ) 
        {
                ddelay(1);
                if ((jiffies - start) > HZ/5) {
                        printk("commandrequest error\n");
                        up(&dvb->dcomlock);
                        arm_error(dvb);
                        return -1;
                }
        }
#endif

        for (i=0; i<n; i++)
                buf[i] = rdebi(dvb, DEBINOSWAP, COM_BUFF + 2*i, 0, 2);
        
	up(&dvb->dcomlock);
        return 0;
}


static inline int 
RequestParameter(struct dvb_struct *dvb, u16 tag, u16* Buff, s16 length)
{
        return CommandRequest(dvb, &tag, 0, Buff, length);
}

static int
GetDecoderStatus(struct dvb_struct *dvb, PVIDEOSTATE vstate)
{
        if (RequestParameter(dvb, (COMTYPE_REQUEST << 8) + VideoState1, 
                             (u16*) vstate, 16)<0)
                return 0;
        if (RequestParameter(dvb, (COMTYPE_REQUEST << 8) + VideoState2, 
                             16+(u16*) vstate, 16)<0)
                return 0;
        if (RequestParameter(dvb, (COMTYPE_REQUEST << 8) + VideoState3, 
                             32+(u16*) vstate, 16)<0)
                return 0;

        return 0;
}




/****************************************************************************
 * Firmware commands 
 ****************************************************************************/


inline static int SendDAC(struct dvb_struct *dvb, u8 addr, u8 data)
{
        return outcom(dvb, COMTYPE_AUDIODAC, AudioDAC, 2, addr, data);
}

int static SetVolume(struct dvb_struct *dvb, u8 volleft, u8 volright)
{
        int err;
        
        if (dvb->dvbtype!=DVB_S)
                return -EOPNOTSUPP;

        if (volleft > 0x45)
                volleft=0x45;
        if (volright > 0x45)
                volright=0x45;
        err=SendDAC(dvb, 3, 0x80 + volleft);
        if (err)
          return err;
        return SendDAC(dvb, 4, volright);

}

inline static int ResetBlend(struct dvb_struct *dvb, u8 windownr)
{
        return outcom(dvb, COMTYPE_OSD, SetNonBlend, 1, windownr);
}

inline static int SetColorBlend(struct dvb_struct *dvb, u8 windownr)
{
        return outcom(dvb, COMTYPE_OSD, SetCBlend, 1, windownr); 
}

inline static int SetWindowBlend(struct dvb_struct *dvb, u8 windownr, u8 blending)
{
        return outcom(dvb, COMTYPE_OSD, SetWBlend, 2, windownr, blending); 
}

inline static int SetBlend_(struct dvb_struct *dvb, u8 windownr,
                     OSDPALTYPE colordepth, u16 index, u8 blending)
{
        return outcom(dvb, COMTYPE_OSD, SetBlend, 4,
                      windownr, colordepth, index, blending);
} 

inline static int SetColor_(struct dvb_struct *dvb, u8 windownr,
                     OSDPALTYPE colordepth, u16 index, u16 colorhi, u16 colorlo)
{
        return outcom(dvb, COMTYPE_OSD, SetColor, 5,
                      windownr, colordepth, index, colorhi, colorlo);
} 

inline static int BringToTop(struct dvb_struct *dvb, u8 windownr)
{
        return outcom(dvb, COMTYPE_OSD, WTop, 1, windownr);
} 

inline static int SetFont(struct dvb_struct *dvb, u8 windownr, u8 fontsize,
                   u16 colorfg, u16 colorbg)
{
        return outcom(dvb, COMTYPE_OSD, Set_Font, 4,
                      windownr, fontsize, colorfg, colorbg);
} 

static int FlushText(struct dvb_struct *dvb)
{
        u32 start;

        down_interruptible(&dvb->dcomlock);
        start = jiffies;
        while ( rdebi(dvb, DEBINOSWAP, BUFF1_BASE, 0, 2 ) ) {
                ddelay(1); 
                if ((jiffies - start) > HZ/5) {
                        printk("outtext error\n");
                        up(&dvb->dcomlock);
                        arm_error(dvb);
                        return -1;
                }
        }
        up(&dvb->dcomlock);
        return 0;
}

static int WriteText(struct dvb_struct *dvb, u8 win, u16 x, u16 y, u8* buf)
{
        int i, ret;
        u32 start;
        int length=strlen(buf)+1;
        u16 cbuf[5] = { (COMTYPE_OSD<<8) + DText, 3, win, x, y };
        
        down_interruptible(&dvb->dcomlock);
        start = jiffies;
        while ( rdebi(dvb, DEBINOSWAP, BUFF1_BASE, 0, 2 ) ) {
                ddelay(1);
                if ((jiffies - start) > HZ/5) {
                        printk("outtext error\n");
                        up(&dvb->dcomlock);
                        arm_error(dvb);
                        return -1;
                }
        }
#ifndef _NOHANDSHAKE
        start = jiffies;
        while ( rdebi(dvb, DEBINOSWAP, HANDSHAKE_REG, 0, 2 ) ) {
                ddelay(1);
                if ((jiffies - start) > 200) {
                        printk("outtext error\n");
                        up(&dvb->dcomlock);
                        arm_error(dvb);
                        return -1;
                }
        }
#endif
        for (i=0; i<length/2; i++)
                wdebi(dvb, DEBINOSWAP, BUFF1_BASE + i*2, 
                      swab16(*(u16 *)(buf+2*i)), 2);
        if (length&1)
                wdebi(dvb, DEBINOSWAP, BUFF1_BASE + i*2, 0, 2);
        ret=OutCommand(dvb, cbuf, 5);
        up(&dvb->dcomlock);
        return ret;
}

inline static int DrawLine(struct dvb_struct *dvb, u8 windownr, 
                    u16 x, u16 y, u16 dx, u16 dy, u16 color)
{
        return outcom(dvb, COMTYPE_OSD, DLine, 6,
                      windownr, x, y, dx, dy, color);
} 

inline static int DrawBlock(struct dvb_struct *dvb, u8 windownr, 
                    u16 x, u16 y, u16 dx, u16 dy, u16 color)
{
        return outcom(dvb, COMTYPE_OSD, DBox, 6,
                      windownr, x, y, dx, dy, color);
} 

inline static int HideWindow(struct dvb_struct *dvb, u8 windownr)
{
        return outcom(dvb, COMTYPE_OSD, WHide, 1, windownr);
} 

inline static int MoveWindowRel(struct dvb_struct *dvb, u8 windownr, u16 x, u16 y)
{
        return outcom(dvb, COMTYPE_OSD, WMoveD, 3, windownr, x, y);
} 

inline static int MoveWindowAbs(struct dvb_struct *dvb, u8 windownr, u16 x, u16 y)
{
        return outcom(dvb, COMTYPE_OSD, WMoveA, 3, windownr, x, y);
} 

inline static int DestroyOSDWindow(struct dvb_struct *dvb, u8 windownr)
{
        return outcom(dvb, COMTYPE_OSD, WDestroy, 1, windownr);
} 

static void DestroyOSDWindows(struct dvb_struct *dvb)
{
        int i;

        for (i=1; i<7; i++)
                outcom(dvb, COMTYPE_OSD, WDestroy, 1, i);
} 

static inline int 
CreateOSDWindow(struct dvb_struct *dvb, u8 windownr,
                           DISPTYPE disptype, u16 width, u16 height)
{
        return outcom(dvb, COMTYPE_OSD, WCreate, 4,
                      windownr, disptype, width, height);
} 


static OSDPALTYPE bpp2pal[8]={Pal1Bit, Pal2Bit, 0, Pal4Bit, 0, 0, 0, Pal8Bit}; 
static DISPTYPE   bpp2bit[8]={BITMAP1, BITMAP2, 0, BITMAP4, 0, 0, 0, BITMAP8}; 

static inline int 
LoadBitmap(struct dvb_struct *dvb, u16 format, u16 dx, u16 dy, int inc, u8* data)
{
        int bpp;
        int i;
        int d, delta; 
        u8 c;
        DECLARE_WAITQUEUE(wait, current);
        
        if (dvb->bmp_state==BMP_LOADING) {
                add_wait_queue(&dvb->bmpq, &wait);
                while (1) {
                        set_current_state(TASK_INTERRUPTIBLE);
                        if (dvb->bmp_state!=BMP_LOADING
                            || signal_pending(current))
                                break;
                        schedule();
                }
                current->state=TASK_RUNNING;
                remove_wait_queue(&dvb->bmpq, &wait);
        }
        if (dvb->bmp_state==BMP_LOADING)
                return -1;
        dvb->bmp_state=BMP_LOADING;
        if      (format==BITMAP8) { bpp=8; delta = 1; } 
        else if (format==BITMAP4) { bpp=4; delta = 2; }
        else if (format==BITMAP2) { bpp=2; delta = 4; }
        else if (format==BITMAP1) { bpp=1; delta = 8; }
        else {
          dvb->bmp_state=BMP_NONE;
          return -1;
        }
        dvb->bmplen= ((dx*dy*bpp+7)&~7)/8; 
        dvb->bmpp=0;
        if (dvb->bmplen>32768) {
                dvb->bmp_state=BMP_NONE;
                return -1;
        }
        for (i=0; i<dy; i++) {
                if (copy_from_user(dvb->bmpbuf+1024+i*dx, data+i*inc, dx)) { 
                        dvb->bmp_state=BMP_NONE;
                        return -1;
                }
                
        }
        if (format != BITMAP8) {
                for (i=0; i<dx*dy/delta; i++) {
                        c = ((u8 *)dvb->bmpbuf)[1024+i*delta+delta-1];
                        for (d=delta-2; d>=0; d--) {
                                c |= (((u8 *)dvb->bmpbuf)[1024+i*delta+d] << ((delta-d-1)*bpp));
                        ((u8 *)dvb->bmpbuf)[1024+i] = c;
                        }
                }
        }
        dvb->bmplen+=1024;
        return outcom(dvb, COMTYPE_OSD, LoadBmp, 3, format, dx, dy);
} 

static int 
BlitBitmap(struct dvb_struct *dvb, u16 win, u16 x, u16 y, u16 trans)
{
        DECLARE_WAITQUEUE(wait, current);
        
        if (dvb->bmp_state==BMP_NONE)
                return -1;
        if (dvb->bmp_state==BMP_LOADING) {
                add_wait_queue(&dvb->bmpq, &wait);
                while (1) {
                        set_current_state(TASK_INTERRUPTIBLE);
                        if (dvb->bmp_state!=BMP_LOADING
                            || signal_pending(current))
                                break;
                        schedule();
                }
                current->state=TASK_RUNNING;
                remove_wait_queue(&dvb->bmpq, &wait);
        }
        if (dvb->bmp_state==BMP_LOADED)
                return outcom(dvb, COMTYPE_OSD, BlitBmp, 4, win, x, y, trans);
        return -1;
} 

static inline int 
ReleaseBitmap(struct dvb_struct *dvb)
{
        if (dvb->bmp_state!=BMP_LOADED)
                return -1;
        dvb->bmp_state=BMP_NONE;
        return outcom(dvb, COMTYPE_OSD, ReleaseBmp, 0);
} 

static u32 RGB2YUV(u16 R, u16 G, u16 B)
{
        u16 y, u, v;
        u16 Y, Cr, Cb;

        y = R * 77 + G * 150 + B * 29;  // Luma=0.299R+0.587G+0.114B 0..65535
        u = 2048+B * 8 -(y>>5);    // Cr 0..4095
        v = 2048+R * 8 -(y>>5);    // Cb 0..4095

        Y=y/256;
        Cb=u/16;
        Cr=v/16;

        return Cr|(Cb<<16)|(Y<<8);
}

static void
OSDSetColor(struct dvb_struct *dvb, u8 color, u8 r, u8 g, u8 b, u8 blend)
{
        u16 ch, cl;
        u32 yuv;

        yuv=blend ? RGB2YUV(r,g,b) : 0;
        cl=(yuv&0xffff);
        ch=((yuv>>16)&0xffff);
        SetColor_(dvb, dvb->osdwin, bpp2pal[dvb->osdbpp[dvb->osdwin]],
                  color, ch, cl);
        SetBlend_(dvb, dvb->osdwin, bpp2pal[dvb->osdbpp[dvb->osdwin]],
                  color, ((blend>>4)&0x0f));
}

static int
OSDSetBlock(struct dvb_struct *dvb, int x0, int y0, int x1, int y1, int inc, u8 *data)
{
        uint w, h, bpp, bpl, size, lpb, bnum, brest;
        int i;

        w=x1-x0+1; h=y1-y0+1;
        if (inc<=0)
                inc=w; 
        if (w<=0 || w>720 || h<=0 || h>576) 
                return -1;
        bpp=dvb->osdbpp[dvb->osdwin]+1; 
        bpl=((w*bpp+7)&~7)/8; 
        size=h*bpl;
        lpb=(32*1024)/bpl; 
        bnum=size/(lpb*bpl);
        brest=size-bnum*lpb*bpl;

        for (i=0; i<bnum; i++) {
                LoadBitmap(dvb, bpp2bit[dvb->osdbpp[dvb->osdwin]], w, lpb, inc, data); 
                BlitBitmap(dvb, dvb->osdwin, x0, y0+i*lpb, 1);
                data+=lpb*inc; 
                //ddelay(3); 
        }
        if (brest) {
                LoadBitmap(dvb, bpp2bit[dvb->osdbpp[dvb->osdwin]], w, brest/bpl, inc, data); 
                BlitBitmap(dvb, dvb->osdwin, x0, y0+bnum*lpb, 1);
        }
        ReleaseBitmap(dvb);
        return 0;
}

static int 
OSD_DrawCommand(struct dvb_struct *dvb, struct drawcmd *dc)
{
        switch (dc->cmd) {
        case OSD_Close:
                DestroyOSDWindow(dvb, dvb->osdwin);
                return 0;
        case OSD_Open:
                dvb->osdbpp[dvb->osdwin]=(dc->color-1)&7;
                CreateOSDWindow(dvb, dvb->osdwin, bpp2bit[dvb->osdbpp[dvb->osdwin]],
                                dc->x1-dc->x0+1, dc->y1-dc->y0+1);
                MoveWindowAbs(dvb, dvb->osdwin, dc->x0, dc->y0);
                SetColorBlend(dvb, dvb->osdwin);
                return 0;
        case OSD_Show:
                MoveWindowRel(dvb, dvb->osdwin, 0, 0);
                return 0;
        case OSD_Hide:
                HideWindow(dvb, dvb->osdwin);
                return 0;
        case OSD_Clear:
                DrawBlock(dvb, dvb->osdwin, 0, 0, 720, 576, 0);
                return 0;
        case OSD_Fill:
                DrawBlock(dvb, dvb->osdwin, 0, 0, 720, 576, dc->color);
                return 0;
        case OSD_SetColor:
                OSDSetColor(dvb, dc->color, dc->x0, dc->y0, dc->x1, dc->y1); 
                return 0;
        case OSD_SetPalette:
        {      
                int i, len=dc->x0-dc->color+1;
                u8 colors[len*4];

                if (copy_from_user(colors, dc->data, sizeof(colors)))
                        return -EFAULT;
                for (i=0; i<len; i++)
                        OSDSetColor(dvb, dc->color+i,
                                    colors[i*4]  , colors[i*4+1],
                                    colors[i*4+2], colors[i*4+3]);
                return 0;
        }
        case OSD_SetTrans: 
                return 0;
        case OSD_SetPixel:
                DrawLine(dvb, dvb->osdwin,
                         dc->x0, dc->y0, dc->x0, dc->y0,
                         dc->color);
                return 0;
        case OSD_GetPixel: // hmm, firmware patch? do we need this?
                return 0;

        case OSD_SetRow:   
                dc->y1=dc->y0;
        case OSD_SetBlock:
                OSDSetBlock(dvb, dc->x0, dc->y0, dc->x1, dc->y1, dc->color, dc->data);
                return 0;

        case OSD_FillRow:
                DrawBlock(dvb, dvb->osdwin, dc->x0, dc->y0,
                          dc->x1-dc->x0+1, dc->y1,
                          dc->color);
                return 0;
        case OSD_FillBlock:
                DrawBlock(dvb, dvb->osdwin, dc->x0, dc->y0,
                          dc->x1-dc->x0+1, dc->y1-dc->y0+1,
                          dc->color);
                return 0;
        case OSD_Line:
                DrawLine(dvb, dvb->osdwin,
                         dc->x0, dc->y0, dc->x1, dc->y1,
                         dc->color);
                return 0;
        case OSD_Query: 
                return 0;
        case OSD_Test:
                return 0;
        case OSD_Text:
        {
                char textbuf[240];
                
                if (strncpy_from_user(textbuf, dc->data, 240)<0)
                        return -EFAULT;
                textbuf[239]=0;
                if (dc->x1>3) 
                        dc->x1=3;
                SetFont(dvb, dvb->osdwin, dc->x1,
                        (u16) (dc->color&0xffff), (u16) (dc->color>>16));
                FlushText(dvb);
                WriteText(dvb, dvb->osdwin, dc->x0, dc->y0, textbuf);
                return 0;
        }
        case OSD_SetWindow:
                if (dc->x0<1 || dc->x0>7)
                        return -EINVAL;
                dvb->osdwin=dc->x0;
                return 0;
        case OSD_MoveWindow:
                MoveWindowAbs(dvb, dvb->osdwin, dc->x0, dc->y0);
                SetColorBlend(dvb, dvb->osdwin);
                return 0;
        default:
                return -EINVAL;
        }
}


/* get version of the firmware ROM of the AV7110 */

static u32 
firmversion(struct dvb_struct *dvb)
{
        u16 Buffer[18];

        u16 tag = ((COMTYPE_REQUEST << 8) + ReqVersion);
        

        RequestParameter(dvb, tag, Buffer, 16);
        
        printk("dvb: firmware = %08x\n", (Buffer[0] << 16) + Buffer[1]);

        return (Buffer[0] << 16) + Buffer[1];
}

static int 
waitdebi(struct dvb_struct *dvb, int adr, int state)
{
        int k;
        
        for (k=0; k<100; k++, udelay(500)) {
                if (irdebi(dvb, DEBINOSWAP, adr, 0, 2) == state) 
                        return 0;
        }
        return -1;
}

static int 
load_dram(struct dvb_struct *dvb, u32 *data, int len)
{
        int i;
        int blocks, rest;
        u32 base, bootblock=BOOT_BLOCK;
        
        blocks=len/BOOT_MAX_SIZE;
        rest=len % BOOT_MAX_SIZE;
        base=DRAM_START_CODE;
        
        for (i=0; i<blocks; i++)
        {
                if (waitdebi(dvb, BOOT_STATE, BOOTSTATE_BUFFER_EMPTY) < 0)
                        return -1;
                dprintk("Writing DRAM block %d\n",i);
                iwdebi(dvb, DEBISWAB, bootblock,
                       i*(BOOT_MAX_SIZE)+(u32)data,
                       BOOT_MAX_SIZE);
                bootblock^=0x1400;
                iwdebi(dvb, DEBISWAB, BOOT_BASE, cpu_to_be32(base), 4);
                iwdebi(dvb, DEBINOSWAP, BOOT_SIZE, BOOT_MAX_SIZE, 2);
                iwdebi(dvb, DEBINOSWAP, BOOT_STATE, BOOTSTATE_BUFFER_FULL, 2);
                base+=BOOT_MAX_SIZE;
        }
        
        if (rest > 0)
        {
                if (waitdebi(dvb, BOOT_STATE, BOOTSTATE_BUFFER_EMPTY) < 0)
                        return -1;
                if (rest>4)
                        iwdebi(dvb, DEBISWAB, bootblock, i*(BOOT_MAX_SIZE)+(u32)data, rest);
                else
                        iwdebi(dvb, DEBISWAB, bootblock, i*(BOOT_MAX_SIZE)-4+(u32)data, rest+4);
                
                iwdebi(dvb, DEBISWAB, BOOT_BASE, cpu_to_be32(base), 4);
                iwdebi(dvb, DEBINOSWAP, BOOT_SIZE, rest, 2);
                iwdebi(dvb, DEBINOSWAP, BOOT_STATE, BOOTSTATE_BUFFER_FULL, 2);
        }
        if (waitdebi(dvb, BOOT_STATE, BOOTSTATE_BUFFER_EMPTY) < 0)
                return -1;
        iwdebi(dvb, DEBINOSWAP, BOOT_SIZE, 0, 2);
        iwdebi(dvb, DEBINOSWAP, BOOT_STATE, BOOTSTATE_BUFFER_FULL, 2);
        if (waitdebi(dvb, BOOT_STATE, BOOTSTATE_BOOT_COMPLETE) < 0)
                return -1;
        return 0;
}

static int 
bootarm(struct dvb_struct *dvb)
{
        u32 *lfirm;
        int len;
        int result;
        

        iwdebi(dvb, 0x6e0000, DPRAM_BASE, 0x76543210, 4);
        if (irdebi(dvb, DEBINOSWAP, DPRAM_BASE, 0, 4)!=0x10325476) {
                printk("dvb: debi test in bootarm() failed!!\n");;
                return -1;
        }
        dprintk("bootarm: debi test OK\n");;
        
        dprintk("bootarm: load boot code\n");
        if (!(len=firmread("Boot_up.axf", (char**) &lfirm)))
                if (!(len=firmread(DVB_FIRM_PATH "Boot_up.axf", (char**) &lfirm)))
                        return -1;
        
        setgpio(dvb, ARM_IRQ_LINE, GPIO_IRQLO);
        //setgpio(dvb, DEBI_DONE_LINE, GPIO_INPUT);
        //setgpio(dvb, 3, GPIO_INPUT);

        setgpio(dvb, RESET_LINE, GPIO_OUTLO);
        mdelay(1);
        
        iwdebi(dvb, DEBISWAB, DPRAM_BASE, (u32) lfirm, len);
        iwdebi(dvb, DEBINOSWAP, BOOT_STATE, BOOTSTATE_BUFFER_FULL, 2);
        
        vfree(lfirm);
        wait_for_debi_done(dvb);
        setgpio(dvb, RESET_LINE, GPIO_OUTHI);
        current->state=TASK_INTERRUPTIBLE;
        schedule_timeout(HZ);
        
        dprintk("bootarm: load dram code\n");
        if (!(len=firmread("arm_firm/Root", (char**) &lfirm)) &&
            !(len=firmread("Root", (char**) &lfirm)) &&
            !(len=firmread(DVB_FIRM_PATH "Root", (char**) &lfirm)))
                return -1;
        result=load_dram(dvb, lfirm, len);
        vfree(lfirm);
        if (result<0)
                return -1;
        
        setgpio(dvb, RESET_LINE, GPIO_OUTLO);
        mdelay(1);
        
        dprintk("bootarm: load dpram code\n");
        if (!(len=firmread("arm_firm/Dpram", (char**) &lfirm)) &&
            !(len=firmread("Dpram", (char**) &lfirm)) &&
            !(len=firmread(DVB_FIRM_PATH "Dpram", (char**) &lfirm)))
                        return -1;
        iwdebi(dvb, DEBISWAB, DPRAM_BASE, (u32) lfirm, len);
        vfree(lfirm);
        wait_for_debi_done(dvb);
        
        setgpio(dvb, RESET_LINE, GPIO_OUTHI);
        
        ddelay(80);

        printk("dvb: ARM firmware successfully loaded.\n");

        ARM_ResetMailBox(dvb);
        return 0;
}

static int
SetPIDs(struct dvb_struct *dvb, u16 vpid, u16 apid, u16 ttpid, u16 subpid, u16 pcrpid)
{
        return outcom(dvb, COMTYPE_PIDFILTER, MultiPID, 5, 0xfffe, vpid, apid, 0, 0);
        /*
        if (dvb->front.tt_pid)
                outcom(dvb, COMTYPE_TELETEXT, TTXDisable, 0);
        if (ttpid) {
                outcom(dvb, COMTYPE_TELETEXT, TTXEnable, 2, 29, ttpid);
                outcom(dvb, COMTYPE_TELETEXT, ModifyTable, 4, 
                       0xffff, 0xffff, 0xffff, 0);
        }
        */
}


static void 
SetMode(struct dvb_struct *dvb, int mode)
{
        outcom(dvb, COMTYPE_ENCODER, LoadVidCode, 1, mode);
        
        if (!dvb->playing)
                SetPIDs(dvb, dvb->front.video_pid, 
                        dvb->front.audio_pid, dvb->front.tt_pid, 0xffff, 0xffff);
}

inline static void 
TestMode(struct dvb_struct *dvb, int mode)
{
        outcom(dvb, COMTYPE_ENCODER, SetTestMode, 1, mode);
}

inline static void 
VidMode(struct dvb_struct *dvb, int mode)
{
        outcom(dvb, COMTYPE_ENCODER, SetVidMode, 1, mode);
}
           

static int inline
vidcom(struct dvb_struct *dvb, u32 com, u32 arg)
{
        return outcom(dvb, 0x80, 0x02, 4, 
                      (com>>16), (com&0xffff), 
                      (arg>>16), (arg&0xffff));
}

static int inline
audcom(struct dvb_struct *dvb, u32 com)
{
        return outcom(dvb, 0x80, 0x03, 4, 
                      (com>>16), (com&0xffff));
}

inline static void 
Set22K(struct dvb_struct *dvb, int state)
{
        if (dvb->cardtype==DVB_CARD_TT_SIEMENS)
                outcom(dvb, COMTYPE_AUDIODAC, (state ? ON22K : OFF22K), 0);
        else // DVB_CARD_TT_BUDGET
                setgpio(dvb, 3, (state ? GPIO_OUTHI : GPIO_OUTLO));
}


/* Diseqc functions only for TT Budget card */

inline static void 
DiseqcSendBit(struct dvb_struct *dvb, int data)
{
        setgpio(dvb, 3, GPIO_OUTHI);
        udelay(data ? 500 : 1000);
        setgpio(dvb, 3, GPIO_OUTLO);
        udelay(data ? 1000 : 500);
}

static void 
DiseqcSendByte(struct dvb_struct *dvb, int data)
{
        int i, par=1, d;

        for (i=7; i>=0; i--) 
        {
                d=(data>>i)&1;
                par^=d;
                DiseqcSendBit(dvb, d);
        }
        DiseqcSendBit(dvb, par);
}

static void 
DiseqcSendMsg(struct dvb_struct *dvb, int frame, int adr, int com, int data, int swi)
{
        setgpio(dvb, 3, GPIO_OUTLO);

        mdelay(16);

        DiseqcSendByte(dvb, frame);
        DiseqcSendByte(dvb, adr);
        DiseqcSendByte(dvb, com);
        DiseqcSendByte(dvb, data);

        mdelay(16);
        
        if (swi) 
                DiseqcSendByte(dvb, 0xff);
        else {
                setgpio(dvb, 3, GPIO_OUTHI);
                udelay(12500);
                setgpio(dvb, 3, GPIO_OUTLO);
        }

        ddelay(2);

        Set22K(dvb, dvb->front.ttk);
}


inline static void
SetDiSEqC(struct dvb_struct *dvb, int input)
{
        dprintk("setdis: %d\n", input);
        if (input<0 || input>15)
                return;
        if (dvb->cardtype==DVB_CARD_TT_SIEMENS) {
                outcom(dvb, COMTYPE_AUDIODAC, SendDiSEqC, 6, 4,
                       (input&4) ? 1 : 0,
                       0xe0, 0x10, 0x38, 0xf0+input);
                outcom(dvb, COMTYPE_AUDIODAC, SendDiSEqC, 6, 4,
                       (input&4) ? 1 : 0,
                       0xe1, 0x10, 0x38, 0xf0+input);
        } else {
                DiseqcSendMsg(dvb, 0xe0, 0x10, 0x38, 0xf0+input, (input&4));
                DiseqcSendMsg(dvb, 0xe1, 0x10, 0x38, 0xf0+input, (input&4));
        }
}

inline static int
SendDiSEqCMsg(struct dvb_struct *dvb, int len, u8 *msg, int burst)
{
        int i;
        u16 buf[18] = { ((COMTYPE_AUDIODAC << 8) + SendDiSEqC),
                        16, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
        
        if (len>10)
                len=10;
	buf[1] = len+2;
        buf[2] = len;

        if (burst!=-1)
          buf[3]=burst ? 0xff : 0;
        else
          buf[3]=0xffff;

        for (i=0; i<len; i++)
                buf[i+4]=msg[i];
        
        return SOutCommand(dvb, buf, 18);
}

static struct dvb_filter *
AllocHWFilter(struct dvb_struct *dvb)
{
        int i;

        for (i=0; i<MAXFILT-3; i++)
                if (!dvb->filter[i].state) {
                        dvb->filter[i].state=1;
                        return &dvb->filter[i]; 
                }
        return NULL;
}

static int 
FreeHWFilter(struct dvb_struct *dvb, struct dvb_filter *filter)
{
        if (!filter || !filter->state)
                return -1;
        filter->state=0;
        return 0;
}

static int
SetBitFilter(struct dvb_struct *dvb, struct bitfilter *filter)
{
        u16 buf[20];
        int ret;
        u16 handle;
        struct dvb_filter *dvbfilter;

        if (!(dvbfilter=AllocHWFilter(dvb)))
                return -1;

        memcpy(buf + 4, filter->data, 32);
        buf[0] = (COMTYPE_PID_FILTER << 8) + AddPIDFilter; 
        buf[1] = 16;	
        buf[2] = filter->pid;
        buf[3] = filter->mode;
          
        ret=CommandRequest(dvb, buf, 20, &handle, 1);          
        if (ret<0)
                return ret;

        filter->handle=handle;
        dvb->hfilter[handle]=dvbfilter;
        dvbfilter->pid=filter->pid;
        dvbfilter->flags=filter->flags;
        
        return ret;
}

static int
ShutDownFilter(struct dvb_struct *dvb, u16 handle)
{
        u16 buf[3];
        u16 answ[2];
        int ret;
      
        if (FreeHWFilter(dvb, dvb->hfilter[handle])<0)
                return -1;
        dvb->hfilter[handle]=NULL;

        buf[0] = (COMTYPE_PID_FILTER << 8) + DelPIDFilter; 
        buf[1] = 1;	
        buf[2] = handle;
        ret=CommandRequest(dvb, buf, 3, answ, 2);          

        if (answ[1] != handle) {
                printk("dvb: filter shutdown error :%d\n", answ[1]);
                ret=-1;
        }
        return ret;
}

static int
ShutDownFilters(struct dvb_struct *dvb)
{
        u16 handle;

        for (handle=0; handle<MAXFILT; handle++)
                ShutDownFilter(dvb, handle);
        return 0;
}
                


/****************************************************************************
 * I2C client commands
 ****************************************************************************/


static int tuner_command(struct dvb_struct *dvb, unsigned int cmd, void *arg)
{
        if (!dvb->tuner)
                return -1;
        return dvb->tuner->driver->command( dvb->tuner, cmd, arg);
}


static int decoder_command(struct dvb_struct *dvb, unsigned int cmd, void *arg)
{
        if (!dvb->decoder)
                return -1;
        return dvb->decoder->driver->command( dvb->decoder, cmd, arg);
}


int static 
GetFront(struct dvb_struct *dvb)
{
        dprintk("getfront\n");
        decoder_command(dvb, DVB_GET_FRONTEND, &dvb->front); 
        return 0;
}

/*
static void
SetPower(struct dvb_struct *dvb, int state)
{
        struct frontend front;

        memcpy(&front, &(dvb->front), sizeof(struct frontend));
        front.power=state;
        SetFront(dvb, &front);
}                

int static 
checklock(struct dvb_struct *dvb)
{
        decoder_command(dvb, DVB_GET_FRONTEND, &dvb->front); 
        if ((dvb->front.sync&0x1f)==0x1f)
                return 0;
        else 
                return -1;
}
*/



/****************************************************************************
 * Tuning
 ****************************************************************************/

int static 
AFC(struct dvb_struct *dvb)
{
        u8 sync;
        int i;
        u32 sfreq=dvb->front.curfreq;
        u32 soff=dvb->front.srate/16;

        ddelay(10);
        decoder_command(dvb, DVB_GET_FRONTEND, &dvb->front); 
        if ((dvb->front.sync&0x1f)==0x1f)
                return 0;
        
        decoder_command(dvb, DVB_RESET, 0); 
        ddelay(40);
        decoder_command(dvb, DVB_GET_FRONTEND, &dvb->front); 
        if ((dvb->front.sync&0x1f)==0x1f)
                return 0;
        decoder_command(dvb, DVB_RESET, 0); 
        ddelay(40);
        decoder_command(dvb, DVB_GET_FRONTEND, &dvb->front); 
        
        for (i=2; i<20; i++) {    
                sync=dvb->front.sync;
                //printk("%02x \n", sync);
                if ((sync&0x1f)==0x1f)
                        break;
                if (i&1) 
                        sfreq=dvb->front.freq+soff*(i/2);
                else
                        sfreq=dvb->front.freq-soff*(i/2);
                tuner_command(dvb, TUNER_SET_TVFREQ, &sfreq); 
                mdelay(1);
                decoder_command(dvb, DVB_RESET, 0); 
                dprintk("sfreq = %d \n", sfreq);
                mdelay(1);
                decoder_command(dvb, DVB_GET_FRONTEND, &dvb->front); 
        }
        if (i==10) 
          return -1;
        
        dprintk("afc = %d \n", dvb->front.afc);
        
        
        if (dvb->front.afc) {
                sfreq-=dvb->front.afc;
                tuner_command(dvb, TUNER_SET_TVFREQ, &sfreq); 
                mdelay(1);
                decoder_command(dvb, DVB_RESET, 0); 
                dprintk("sfreq = %d \n", sfreq);
                //decoder_command(dvb, DVB_GET_FRONTEND, &dvb->front); 
                dprintk("afc = %d \n", dvb->front.afc);
        }
        
        dvb->front.curfreq=sfreq;
        return 0;
}

static inline void 
decoder_writereg(struct dvb_struct *dvb, u8 adr, u8 val)
{
        u8 msg[2]={ adr, val }; 
  
        decoder_command(dvb, DVB_WRITEREG, &msg); 
}


/* I moved these here (from VES1893.c) since the usage of the CTRL regs
   depends on the cards hardware */
static void 
SetVoltage(struct dvb_struct *dvb, int power, int voltage)
{
        decoder_writereg(dvb, 0x1f, (power ? 0x20 : 0)|(voltage ? 0x10 : 0));
          
}

static int
SetFront(struct dvb_struct *dvb, struct frontend *front)
{
        dprintk("setfront\n");
        
        decoder_command(dvb, DVB_GET_FRONTEND, &dvb->front); 
        if (dvb->dvbtype==DVB_S) {
                if (dvb->front.diseqc!=front->diseqc 
                    || (dvb->front.sync&0x1f)!=0x1f
                    || dvb->front.ttk!=front->ttk
                    || dvb->front.volt!=front->volt 
                    || dvb->front.power!=front->power
                    || dvb->front.freq!=front->freq)
                        front->flags|=FRONT_TP_CHANGED;

                if (front->flags & FRONT_TP_CHANGED)
                if (dvb->cardtype==DVB_CARD_TT_SIEMENS)
                        ShutDownFilters(dvb);

                //if (dvb->front.diseqc!=front->diseqc) 
                if (front->flags & FRONT_TP_CHANGED)
                        SetDiSEqC(dvb,
                                  (front->diseqc*4) |
                                  (front->ttk ? 1 : 0) |
                                  (front->volt ? 2 : 0));
                if (dvb->front.ttk!=front->ttk 
                    || dvb->front.diseqc!=front->diseqc) {
                        Set22K(dvb, front->ttk);
                }
                if (dvb->front.power!=front->power 
                    || dvb->front.volt!=front->volt) {
                        SetVoltage(dvb, front->power, front->volt);
                }
        }


        if (dvb->dvbtype==DVB_C) {
          /*
                if (dvb->front.power!=front->power)
                        outcom(dvb, COMTYPE_AUDIODAC, MainSwitch, 1,
                               front->power);
          */    
                if (dvb->front.qam!=front->qam)
                        front->flags|=FRONT_TP_CHANGED;
        }
        if (dvb->front.freq!=front->freq) {
                front->flags|=FRONT_FREQ_CHANGED|FRONT_TP_CHANGED;
                tuner_command(dvb, TUNER_SET_TVFREQ, &front->freq); 
                front->curfreq=front->freq;
        }

        decoder_command(dvb, DVB_SET_FRONTEND, front); 

        if (dvb->cardtype==DVB_CARD_TT_SIEMENS)
                SetPIDs(dvb, front->video_pid, front->audio_pid, 
                        front->tt_pid, 0xffff, 0xffff);

        if ((front->channel_flags&DVB_CHANNEL_CA)) {
                if (front->flags & FRONT_TP_CHANGED)
                        SendCICommand(dvb, CI_CMD_SYSTEM_READY, 0, 0);
                if (front->pnr!=dvb->front.pnr || 
                    (front->flags & FRONT_TP_CHANGED)) {
                        u8 pnr[2];
                        pnr[0]=front->pnr>>8;
                        pnr[1]=(front->pnr&0xff);
                        SendCICommand(dvb, CI_CMD_FAST_PSI, pnr, 2);
                }
        } 

        if (front->flags & FRONT_TP_CHANGED) {
                decoder_command(dvb, DVB_RESET, 0); 
                //SendCICommand(dvb, CI_CMD_SYSTEM_READY, 0, 0);
        }

        dvb->front=*front;
        /* Not fully tested with DVB-C yet */
        if ((dvb->dvbtype==DVB_S) && front->AFC &&
            (front->flags & FRONT_TP_CHANGED)) {
                dvb->front.freq=front->freq;
                front->AFC=0;
                AFC(dvb);
        }
        dvb->front.flags=0;
        
        return 0;
}
                
static int
SetPIDArray(struct dvb_struct *dvb, u16 *pid)
{
        dvb->front.video_pid=(pid[1]==0xffff) ? 0 : pid[1];
        dvb->front.audio_pid=(pid[0]==0xffff) ? 0 : pid[0];
        return SetPIDs(dvb, pid[1], pid[0], pid[2], pid[3], pid[4]);
}


static void
InitFront(struct dvb_struct *dvb)
{
        struct frontend front;
        int ttype=0;

        dprintk("initfront\n");

        memset(&(dvb->front), 0xff, sizeof(struct frontend));
        memset(&front, 0x00, sizeof(struct frontend));

        decoder_command(dvb, DVB_INIT_FRONTEND, 0); 

        front.pnr=0;
        front.channel_flags=0;
        front.type=dvb->front.type=dvb->dvbtype;
        switch (dvb->dvbtype) {
        case DVB_S:
                ttype=TUNER_SP5659;
                tuner_command(dvb, TUNER_SET_TYPE, &ttype); 

                if (dvb->cardtype==DVB_CARD_TT_SIEMENS) {
                        SendDAC(dvb, 2, 1);
                        SendDAC(dvb, 2, 0);
                        mdelay(100);

                        SendDAC(dvb, 0, 0x44);
                        SendDAC(dvb, 1, 0x81);
                        SendDAC(dvb, 2, 0xf0);
                        SetVolume(dvb, 0x45, 0x45);
                }

                saa7146_write(dvb->saa->mem, MC2, 
                              (MASK_09 | MASK_25 | MASK_10 | MASK_26));
                front.power=1;
                front.AFC=1;
                front.fec=8;
                if (!init_chan)
                        break;

                if (init_chan==2) {
                        // Astra n-tv default
                        front.channel_flags=DVB_CHANNEL_FTA;
                        front.volt=0;
                        front.ttk=1; 
                        front.diseqc=0;    
                        front.freq=front.curfreq=(12666000-10600000)*1000;
                        front.srate=22000000;
                        front.video_pid=162;
                        front.audio_pid=96;
                        front.tt_pid=0x1012;
                        front.inv=0;
                } else {
                        // Astra PRO7
                        front.channel_flags=DVB_CHANNEL_FTA;
                        front.volt=0;
                        front.ttk=1;
                        front.diseqc=0;
                        front.freq=front.curfreq=(12480000-10600000)*1000;
                        front.srate=27500000;
                        front.video_pid=255;
                        front.audio_pid=256;
                        front.inv=0;
                }
                
                SetFront(dvb, &front);
                break;
        case DVB_C:

                ttype=TUNER_DVBC;

                if (dvb->cardtype==DVB_CARD_TT_SIEMENS) {
                        outcom(dvb, COMTYPE_AUDIODAC, MainSwitch, 1, 1);
                        outcom(dvb, COMTYPE_AUDIODAC, ADSwitch, 1, 1); //0 for analog
                }
                tuner_command(dvb, TUNER_SET_TYPE, &ttype); 
                decoder_writereg(dvb, 0x0f, 0x28); //use 0x68 for analog 

                if (!init_chan)
                        break;

                front.power=1;
                front.freq=394000000;        
                front.srate=6900000;
                front.video_pid=0x262;
                front.audio_pid=0x26c;
                front.qam=2;
                front.inv=0;

                SetFront(dvb, &front);
                break;
        }
        if (dvb->cardtype==DVB_CARD_TT_SIEMENS)
                VidMode(dvb, vidmode);
        if (napi)
                dvb_register_frontend(dvb);
        //printk("dvb: sync=%02x\n",dvb->front.sync);
        //SendCICommand(dvb, CI_CMD_GET_SLOT_INFO, 0, 0);
}




/****************************************************************************
 * I/O buffer management and control
 ****************************************************************************/


static void section_reset(struct dvb_struct *dvb)
{
        dvb->fsec.head=dvb->fsec.tail;
}

static unsigned int section_poll(struct dvb_struct *dvb,
                                 struct file *file, poll_table *wait)
{
        unsigned int mask=0;

        if (dvb->fsec.head != dvb->fsec.tail)
                return (POLLIN | POLLRDNORM);

        poll_wait(file, &dvb->fsec.queue, wait);

        if (dvb->fsec.head != dvb->fsec.tail)
                mask |= (POLLIN | POLLRDNORM);

        return mask;
}


static long section_read(struct dvb_struct *dvb, char *buf,
                         unsigned long count, int nonblock)
{

        DECLARE_WAITQUEUE(wait, current);
        unsigned long todo = count;
        int fsize, split;
        
        while (todo == count) {
                if (dvb->fsec.head == dvb->fsec.tail) {
                        add_wait_queue(&dvb->fsec.queue, &wait);
                        while (1) {
                                set_current_state(TASK_INTERRUPTIBLE);
                                if (dvb->fsec.head != dvb->fsec.tail
                                    || signal_pending(current))
                                        break;
                                schedule();
                        }
                        current->state=TASK_RUNNING;
                        remove_wait_queue(&dvb->fsec.queue, &wait);
                        if (dvb->fsec.head == dvb->fsec.tail)
                                return (count-todo);
                }

                split=dvb->fsec.length;
                //spin_lock_irqsave(&dvb->fsec.lock, flags);
                fsize=dvb->fsec.tail - dvb->fsec.head;
                if (fsize < 0) {
                        fsize+=dvb->fsec.length;
                        split=dvb->fsec.length - dvb->fsec.head;
                }
                //spin_unlock_irqrestore(&dvb->fsec.lock, flags);

                if (fsize > todo)
                        fsize=todo;
                if (split < fsize) {
                        if (copy_to_user(buf, dvb->fsec.buf+dvb->fsec.head, split))
                                return -EFAULT;
                        buf+=split;
                        dvb->fsec.head=0;
                        todo-=split;
                        fsize-=split;
                }
                if (fsize) {
                        if (copy_to_user(buf, dvb->fsec.buf+dvb->fsec.head,
                                         fsize))
                                return -EFAULT;
                        dvb->fsec.head = (dvb->fsec.head + fsize) & (dvb->fsec.length-1);
                        todo-=fsize;
                        buf+=fsize;
                }
        }
	return (count-todo);
}


static long av_record(struct dvb_struct *dvb, char *buf,
                      unsigned long count, int nonblock)
{
        if (dvb->cardtype!=DVB_CARD_TT_BUDGET)
                AV_StartRecord(dvb,RP_AV);
        else 
                if (!dvb->recording) {
                        dvb->recording=1;
                        TTBStart(dvb);
                }
        return dvb_buf_read(&dvb->avin, buf, count, nonblock, 1);
}

static long av_play(struct dvb_struct *dvb, const char *buf,
                    unsigned long count, int nonblock, int usermem)
{
        return dvb_buf_write(&dvb->avout, buf, count, nonblock, usermem);
}

static long dvb_play(struct dvb_struct *dvb, const char *buf,
                     unsigned long count, int nonblock)
{
        if (!dvb->transi.pesto[0].what){
                switch (buf[0]){

                case 0x41:
                        if ( buf[0] == 0x41 && buf[1] == 0x56){
                                dvb->transi.pesto[0].what = 1;
                                printk(KERN_ERR "dvb: Playing AV_PES\n");
                        } else {
                                dvb->transi.pesto[0].what = 2;
                                dvb->transi.pesto[0].nonblock = nonblock;
                                printk(KERN_ERR "dvb: Playing PES\n");
                        }
                        break;

                case 0x47:
                        dvb->transi.pesto[0].what = 3;
                        dvb->transi.pesto[0].nonblock = nonblock;
                        dvb->transi.pesto[1].nonblock = nonblock;
                        printk(KERN_ERR "dvb: Playing TS\n");
                        break;
                        
                case 0x00:
                        if (buf[1] == 0x00 &&
                            buf[2] == 0x01 &&
                            buf[3] == (char) 0xb3){
                                dvb->transi.pesto[0].what = 4;
                                dvb->transi.pesto[0].nonblock = nonblock;
                                printk(KERN_ERR "dvb: Playing ES\n");
                                break;
                        } 
                                
                default:
                        dvb->transi.pesto[0].what = 2;
                        dvb->transi.pesto[0].nonblock = nonblock;
                        printk(KERN_ERR "dvb: Playing PES\n");
                        break;
                }             
        }   

        switch (dvb->transi.pesto[0].what){
        case 1:
                return av_play(dvb, buf, count, nonblock, 1);

        case 2:
                return kpes_to_av_pes(buf, count,(void *)&dvb->transi);
                
        case 3:
                count =  ts_av_pes(&dvb->transi,buf,count);
                return count;

        case 4:
                es_to_av_pes(&dvb->transi, buf, count);
                return count;

        default:
                return 0;
        }
}




/****************************************************************************
 * PES AV_PES conversion 
 ****************************************************************************/

static void init_av_pes (av_pes_packet_t *p)
{
	p->length = 0;
	p->reserved = AV_RES;
	p->counter = 0;
	p->flags = 0x00;
}

static void nlength_av_pes(av_pes_packet_t *p)
{
	short *ll = (short *) p->llength;
	short l = p->length;
	if ((p->flags & PTS_FLAG) && (p->stream_id == AV_VIDEO)) l+= 4;
	*ll = htons(l);
}

static int find_format(trans_t *p, u8 *buf)
{
	u8 headr[4];
	int found=0;
        int c = 0;
	int form = -1;
	int hsize,vsize;
        int sw;
        int w = p->which_pesto;
        int bit_rate;

	while (c < MAX_AV-8 && !found) {
		memcpy(headr,buf+c,4);
                if (headr[0] == 0x00 && headr[1] == 0x00 && headr[2] == 0x01)
                        if ( headr[3] == 0xB3 ){ 
				found = 1;
                                c+=4;
			}
		if (! found)  c++;
	}
        
        if (!found) return -1;

        memcpy(headr,buf+c,4);
        c += 4;

	hsize = ((headr[1] &0xF0) >> 4) | (headr[0] << 4);
	vsize = ((headr[1] &0x0F) << 8) | (headr[2]);
        p->pesto[w].hsize = hsize;
        p->pesto[w].vsize = vsize;

        sw = (int)((headr[3]&0xF0) >> 4) ;

        switch( sw ){
	case 1:
		printk("MPEG: ASPECT: 1:1");
                p->pesto[w].aspect = 100;        
		break;
	case 2:
		printk("MPEG: ASPECT: 4:3");
                p->pesto[w].aspect = 133;        
		break;
	case 3:
		printk("MPEG: ASPECT: 16:9");
                p->pesto[w].aspect = 177;        
		break;
	case 4:
		printk("MPEG: ASPECT: 2.21:1");
                p->pesto[w].aspect = 221;        
		break;

        case 5 ... 15:
		printk("MPEG: ASPECT: reserved");
                p->pesto[w].aspect = 0;        
		break;

        default:
                p->pesto[w].aspect = 0;        
                return -1;
	}

        printk("  Size = %dx%d",hsize,vsize);

        sw = (int)(headr[3]&0x0F);

        switch ( sw ) {
	case 1:
		printk("  FRAMERATE: 23.976");
                p->pesto[w].framerate = 24000/1001;
		form = -1;
		break;
	case 2:
		printk("  FRAMERATE: 24");
                p->pesto[w].framerate = 24;
		form = -1;
		break;
	case 3:
		printk("  FRAMERATE: 25");
                p->pesto[w].framerate = 25;
		form = VIDEO_MODE_PAL;
		break;
	case 4:
		printk("  FRAMERATE: 29.97");
                p->pesto[w].framerate = 30000/1001;
		form = VIDEO_MODE_NTSC;
		break;
	case 5:
		printk("  FRAMERATE: 30");
                p->pesto[w].framerate = 30;
		form = VIDEO_MODE_NTSC;
		break;
	case 6:
		printk("  FRAMERATE: 50");
                p->pesto[w].framerate = 50;
		form = VIDEO_MODE_PAL;
		break;
	case 7:
		printk("  FRAMERATE: 60");
                p->pesto[w].framerate = 60;
		form = VIDEO_MODE_NTSC;
		break;
	}

        if (c < MAX_AV-3){
                memcpy(headr,buf+c,3);
                
                bit_rate = 400*((headr[0] << 10) | (headr[1] << 2) | 
                                ((headr[2] & 0xC0) >> 6));
	
                printk("  BRate: %d.%d Mbit/s",bit_rate/1000000,
                       (bit_rate%1000000)/10000);
        }
	
        printk("\n");
        p->pesto[w].video_format = form;

        return form;
}

static int cwrite_av_pes(trans_t *t, av_pes_packet_t *p)
{
	long count;
	u8 sync[2] = { 0x41, 0x56} ;
        int w;
	u8 *buf;
        int mode;
	count = 0;

        w = t->which_pesto;
	buf = t->pesto[w].buf;
	nlength_av_pes(p);
	buf[0] = sync[0];
	buf[1] = sync[1];
	count = 2;
	if (p->gap){
		p->flags |= (p->gap & 0x03) << 2;
	}
	buf[count] = p->stream_id;
	count++;
	buf[count] = p->counter;
	count++;
	buf[count] = p->reserved;
	count++;
	buf[count] = p->flags;
	count++;
	buf[count] = p->llength[0];
	count++;
	buf[count] = p->llength[1];
	count++;

        if (t->pesto[w].video_format < 0  
            && p->stream_id == AV_VIDEO){
                mode = find_format(t, p->payload);
              
                switch (mode){
                case VIDEO_MODE_PAL:
                  printk("MPEG: PAL mode\n");
                  SetMode((struct dvb_struct *)t->pointer, mode);
                  break;
                case VIDEO_MODE_NTSC:
                  printk("MPEG: NTSC mode\n");
                  SetMode((struct dvb_struct *)t->pointer, mode);
                  break;
                default:
                        return 0;
                }
        }

	if ((p->flags & PTS_FLAG) && (p->stream_id == AV_VIDEO)){
		memcpy(buf+count,p->pts,4);
		count += 4;
		memcpy(buf+count,p->payload,p->length);
		count += p->length;
	} else {
		memcpy(buf+count,p->payload,p->length);
		count += p->length;
	}

	return count;

}

#ifdef NEW_OUT
static void write_a_pes(trans_t *t, av_pes_packet_t *p)
{
	u32 length, count;
        int w = t->which_pesto;

	length = cwrite_av_pes(t, p);
        if (length)
                count = dvb_buf_write(&((struct dvb_struct *)t->pointer)->aout,
                                      t->pesto[w].buf, length, t->pesto[w].nonblock,
                                      0);
}

static void write_v_pes(trans_t *t, av_pes_packet_t *p)
{
	u32 length, count;
        int w = t->which_pesto;

	length = cwrite_av_pes(t, p);
        if (length)
                count = dvb_buf_write(&((struct dvb_struct *)t->pointer)->vout,
                                      t->pesto[w].buf, length, t->pesto[w].nonblock,
                                      0);
}
#endif

static void write_av_pes(trans_t *t, av_pes_packet_t *p)
{
	u32 length, count;
        int w = t->which_pesto;

	length = cwrite_av_pes(t, p);
        if (length)
                count = av_play((struct dvb_struct *)t->pointer, 
                                t->pesto[w].buf, length, t->pesto[w].nonblock,
                                0);
}


static void es_to_av_pes(trans_t *t, const u8 *buf, long int l)
{
	long int count, size;
        av_pes_packet_t *apsv;


        apsv = &t->pesto[0].apsv;
        t->which_pesto = 0;

	init_av_pes(apsv);
	apsv->stream_id = AV_VIDEO;
	count = 0;
	while (count < l){
                size = l-count;
		if ( size > MAX_AV) size = MAX_AV;
		count += size;
		apsv->length = size;
		memcpy(apsv->payload,buf,size);
                write_av_pes(t, apsv);
	}

}

static void pts2pts(u8 *av_pts, u8 *pts)
{
		av_pts[0] = ((pts[0] & 0x06) << 5) | 
			((pts[1] & 0xFC) >> 2); 
		av_pts[1] = ((pts[1] & 0x03) << 6) |
			((pts[2] & 0xFC) >> 2); 
		av_pts[2] = ((pts[2] & 0x02) << 6) |
			((pts[3] & 0xFE) >> 1);
		av_pts[3] = ((pts[3] & 0x01) << 7) |
			((pts[4] & 0xFE) >> 1);
}

static void kwrite_video(trans_t *t)
{
        int w = t->which_pesto;

	int gap = t->pesto[w].apsv.length & 3;
	t->pesto[w].apsv.gap = gap;
	t->pesto[w].apsv.length -= gap;
#ifdef NEW_OUT
	write_v_pes(t, &t->pesto[w].apsv);
#else
	write_av_pes(t, &t->pesto[w].apsv);
#endif
	if (gap) memmove(t->pesto[w].apsv.payload, 
                         t->pesto[w].apsv.payload+
                         t->pesto[w].apsv.length,gap);
	t->pesto[w].apsv.flags = gap;
	t->pesto[w].apsv.length = gap;
	t->pesto[w].apsv.counter++;
}



static void check_mp1_pts(trans_t *p, int type)
{
        int c=6;
        u8 check,flag2;
        int w = p->which_pesto;

        u8 *buf = NULL;

        switch (type){
        case AV_AUDIO:
                buf = p->pesto[w].apsa.payload;
                break;
        case AV_PCM:
                buf = p->pesto[w].apsp.payload;
                break;
        }

        flag2 = 0;

        while ( buf[c] == 0xFF) c++;

        if ( (buf[c] & 0xC0) == 0x40){
                c++;
                check = buf[c];
                c++;
        }
  
        check = buf[c];
        c++;
        if ( (check & 0x30) ){
                flag2 = (check & 0xF0) << 2;
        } 
  

        if (flag2 & PTS_DTS_FLAGS){
                switch (type){
                case AV_AUDIO:
                        p->pesto[w].apsa.flags |= PTS_FLAG;
                        break;
                case AV_PCM:
                        p->pesto[w].apsp.flags |= PTS_FLAG;
                        break;
                }
                
        }
        
}


static void kwrite_audio(trans_t *t)
{
        int w = t->which_pesto;

        if (t->pesto[w].mpeg == 1){
                check_mp1_pts(t,AV_AUDIO);
        }

#ifdef NEW_OUT
        write_a_pes(t, &t->pesto[w].apsa);
#else
        write_av_pes(t, &t->pesto[w].apsa);
#endif
	t->pesto[w].apsa.flags = 0;
	t->pesto[w].apsa.length = 0;
	t->pesto[w].apsa.counter ++;
}

static void kwrite_pcm(trans_t *t)
{
        int l; 
        int w = t->which_pesto;
        
        l = t->pesto[w].apsp.payload[8] & 0xFF;
        if ( t->pesto[w].apsp.payload[l+9] == 0xA0 ) {
                if (t->pesto[w].mpeg == 1)
                        check_mp1_pts(t, AV_AUDIO);
                write_av_pes(t, &t->pesto[w].apsp);
                t->pesto[w].apsp.counter ++;
        }
	t->pesto[w].apsp.flags = 0;
	t->pesto[w].apsp.length = 0;
}

static void init_p2a(p2a_t *p)
{
	p->found = 0;
	p->cid = 0;
	p->mpeg = 0;
        p->video_format = -1;
	p->what = 0;
	init_av_pes(&p->apsv);
	init_av_pes(&p->apsa);
	init_av_pes(&p->apsp);
	p->apsv.stream_id = AV_VIDEO;
	p->apsa.stream_id = AV_AUDIO;
	p->apsp.stream_id = AV_PCM;
}


#define NB_CHECK  if (t->pesto[0].nonblock && \
                  (dvb_buf_free(&((struct dvb_struct *)t->pointer)->avout)<8*1024)) \
                        return c;


static long 
kpes_to_av_pes(u8 const *buf, long int count, void * data)
{

	int l,max;
	short *pl;
	int c=0;
	u8 headr[3] = { 0x00, 0x00, 0x01} ;
        trans_t *t;
        p2a_t *p;
        

        t = (trans_t *)data;
        p = &t->pesto[t->which_pesto];

        NB_CHECK;

        while (c<count) {
                while (c < count && p->found <4 ) {
                        switch ( p->found ){
                        case 0:
                        case 1:
                                if (buf[c] == 0x00) p->found++;
                                else p->found = 0;
                                c++;
			break;
                        case 2:
                                if (buf[c] == 0x01) p->found++;
                                else if (buf[c] == 0x00) {
                                        p->found = 2;
                                } else p->found = 0;
                                c++;
                                break;
                        case 3:
                                p->cid = 0;
                                switch (buf[c]){
                                case PROG_STREAM_MAP:
                                case PRIVATE_STREAM2:
                                case PROG_STREAM_DIR:
                                case ECM_STREAM     :
                                case EMM_STREAM     :
                                case PADDING_STREAM :
                                case DSM_CC_STREAM  :
                                case ISO13522_STREAM:
                                        p->done = 1;
                                case PRIVATE_STREAM1:
                                case VIDEO_STREAM_S ... VIDEO_STREAM_E:
                                case AUDIO_STREAM_S ... AUDIO_STREAM_E:
                                  p->found++;
                                  p->cid = buf[c];
                                  c++;
                                  break;
                                default:
                                  p->found = 1;
                                  break;
                                }
                                break;
                                
                        default:
                          break;
                        }
                }
                if (c == count) return count;

                if (p->found == 4) {
			
                        if (count-c > 1) {
                                pl = (short *) (buf+c);
                                p->plength = ntohs(*pl);
                                p->plen[0] = buf[c];
                                c++;
                                p->plen[1] = buf[c];
                                c++;
                                p->found+=2;
                        } else {
                                p->plen[0] = buf[c];
                                p->found++;
                                return c;
                        }
                }
                
                if (p->found == 5) {
                        p->plen[1] = buf[c];
                        c++;
                        pl = (short *) p->plen;
                        p->plength = ntohs(*pl);
                        p->found++;
                }
                
                if (c == count) return c;

                while (!p->done && 
                       (c < count && p->found > 5 
                        && (p->mpeg == 0 ||
                            (p->mpeg == 1 && p->found < 7) ||
                            (p->mpeg == 2 && p->found < 9)))){
                        if ( p->found == 6 ){
                                p->flag1 = buf[c];
                                c++;
                                p->found++;
                                if ( (p->flag1 & 0xC0) == 0x80 ) p->mpeg = 2;
                                else {
                                        p->found2 = 0;
                                        p->mpeg = 1;
                                        p->flag2 = 0;				
                                }
                        }
                        if (c == count) return c;
                        
                        if ( p->found == 7 && p->mpeg == 2){
                                p->flag2 = buf[c];
                                c++;
                                p->found++;
                        }
                        if (c == count) return c;
                        if ( p->found == 8  && p->mpeg == 2) {
                                p->hlength = buf[c];
                                c++;
                                p->found++;
                        }
                }
                
                NB_CHECK;
                
                if ( p->done || 
                     (p->mpeg == 2 && p->found >= 9)  || 
                     (p->mpeg == 1 && p->found >= 7) ) { 
                        switch (p->cid){

                        case VIDEO_STREAM_S ... VIDEO_STREAM_E:
                                if (p->mpeg == 2 && 
                                    (p->flag2 & PTS_ONLY) &&  
                                    p->found < 14){
                                        while (c < count && p->found < 14) {
                                                p->pts[p->found-9] = buf[c];
                                                c++;
                                                p->found++;
                                        }
                                        if (c == count) return c;
                                        if (p->apsv.length > (p->apsv.flags & 0x03))
                                                kwrite_video(t);
                                        pts2pts(p->apsv.pts,p->pts);
                                        p->apsv.flags |= PTS_FLAG;
                                }
                                while (p->mpeg == 2 && c < count && 
                                       p->found < 9+p->hlength) {
                                        c++;
                                        p->found++;
                                }


                                if (p->mpeg == 1 && p->found2 < 2000) {

                                        if (p->found == 7) p->check = p->flag1;
                                        while (!p->found2 && c < count && 
                                               p->check == 0xFF) {
                                                p->check = buf[c];
                                                c++;
                                                p->found++;
                                        }

                                        if ( c == count) return c;
                                        
                                        if ( (p->check & 0xC0) == 0x40 && 
                                             !p->found2){
                                                p->check = buf[c];
                                                c++;
                                                p->found++;
                                                p->found2 = 1;
                                                if ( c == count) return c;
                                                p->check = buf[c];
                                                c++;
                                                p->found++;
                                                p->found2 = 2;
                                                if ( c == count) return c;
                                        }
                                        
                                        if (p->found2 == 1){
                                                p->check = buf[c];
                                                c++;
                                                p->found++;
                                                p->found2 = 2;
                                                if ( c == count) return c;
                                        }
                                        
                                        
                                        if ( (p->check & 0x30) && 
                                             p->check != 0xFF){
                                                p->flag2 = (p->check & 0xF0) << 2;
                                                p->pts[0] = p->check;
                                                p->found2 = 3;
                                        } 
                                        
                                        if ( c == count) return c;
                                        if (p->found2 > 2){
                                                if ((p->flag2 & PTS_DTS_FLAGS)
                                                    == PTS_ONLY){
                                                         while (c < count && 
                                                                p->found2 < 7){
                                                                 p->pts[p->found2-2] =
                                                                   buf[c];
                                                                 c++;
                                                                 p->found++;
                                                                 p->found2++;
                                                         }
                                                         if ( c == count) return c;
                                                } else if ((p->flag2 & PTS_DTS_FLAGS) 
                                                           == PTS_DTS){
                                                  while (c < count && 
                                                         p->found2< 12){
                                                          if (p->found2< 7)
                                                                  p->pts[p->found2-2] =
                                                                    buf[c];
                                                          c++;
                                                          p->found++;
                                                          p->found2++;
                                                  }
                                                  if ( c == count) return c;
                                                }
                                                if (p->found2 >= 7){
                                                        if (p->apsv.length > 
                                                            (p->apsv.flags & 0x03))
                                                          kwrite_video(t);
                                                        pts2pts(p->apsv.pts,p->pts);
                                                        p->apsv.flags |= PTS_FLAG;
                                                }
                                                p->found2 = 2000;
                                                
                                        }
                                        
                                }

                                while (c < count && (p->found < p->plength+6 ||
                                        !p->plength)){
                                        NB_CHECK;

                                        max = MAX_AV - ((p->apsv.flags & PTS_FLAG)>>2) 
                                          - p->apsv.length;
                                        l = count-c;
                                        if (p->plength &&
                                            l+p->found > p->plength+6)
                                          l = p->plength+6-p->found;
                                        
                                        if (l < max){
                                          memcpy(p->apsv.payload+p->apsv.length,
                                                 buf+c,l);
                                          p->apsv.length += l;
                                          p->found += l;
                                          c += l;
                                        } else {
                                          memcpy(p->apsv.payload+p->apsv.length,
                                                 buf+c,max);
                                          p->apsv.length += max;
                                          p->found += max;
                                          c += max;
                                          kwrite_video(t);
                                        }
                                }					
				
                                break;
                                
                        case AUDIO_STREAM_S ... AUDIO_STREAM_E:			
                                if (p->mpeg == 2 && p->found == 9){
                                  memcpy(p->apsa.payload,headr,3);
                                  p->apsa.payload[3] = p->cid;
                                  memcpy(p->apsa.payload+4,p->plen,2);
                                  p->apsa.payload[6] = p->flag1;
                                  p->apsa.payload[7] = p->flag2;
                                  p->apsa.payload[8] = p->hlength;
                                  if (p->flag2 & PTS_ONLY && p->mpeg == 2)
                                          p->apsa.flags |= PTS_FLAG;
                                  p->apsa.length = 9;
                                }
                          
                                if (p->mpeg == 1 && p->found == 7) {
                                        memcpy(p->apsa.payload,headr,3);
                                        p->apsa.payload[3] = p->cid;
                                        memcpy(p->apsa.payload+4,p->plen,2);
                                        p->apsa.payload[6] = p->flag1;
                                        p->apsa.length = 7;
                                        if (c == count) return c;
                                }
                          
                                while (c < count && p->found < p->plength+6){
                                        NB_CHECK;

                                        max = MAX_AVA - p->apsa.length;
                                        l = count -c;
                                        if (l+p->found > p->plength+6)
                                                l = p->plength+6-p->found;
                                        if (l < max){
                                                memcpy(p->apsa.payload+p->apsa.length,
                                                       buf+c,l);
                                                p->apsa.length += l;
                                                p->found += l;
                                                c += l;
                                                if (p->found == p->plength+6)
                                                        kwrite_audio(t);
                                        } else {
                                                memcpy(p->apsa.payload+p->apsa.length,
                                                       buf+c,max);
                                                p->apsa.length += max;
                                                p->found += max;
                                                c += max;
                                                kwrite_audio(t);
                                        }	
                                }				
                                break;
                                
                        case PRIVATE_STREAM1:
                                if (p->mpeg == 2 && p->found == 9){
                                        memcpy(p->apsp.payload,headr,3);
                                        p->apsp.payload[3] = p->cid;
                                        memcpy(p->apsp.payload+4,p->plen,2);
                                        p->apsp.payload[6] = p->flag1;
                                        p->apsp.payload[7] = p->flag2;
                                        p->apsp.payload[8] = p->hlength;
                                        if (p->flag2 & PTS_ONLY && p->mpeg == 2)
                                                p->apsp.flags |= PTS_FLAG;
                                        p->apsp.length = 9;
                          }
                          
                          if (p->mpeg == 1 && p->found == 7){
                                  memcpy(p->apsp.payload,headr,3);
                                  p->apsp.payload[3] = p->cid;
                                  memcpy(p->apsp.payload+4,p->plen,2);
                                  p->apsp.payload[6] = p->flag1;
                                  p->apsp.length = 7;
                                  if (c == count) return c;
                          }
                          
                          while (c < count && p->found < p->plength+6){
                                  NB_CHECK;
                                  max = MAX_AVA+32 - p->apsp.length;
                                  l = count -c;
                                  if (l+p->found > p->plength+6)
                                          l = p->plength+6-p->found;
                                  if (l < max){
                                          memcpy(p->apsp.payload+p->apsp.length,
                                                 buf+c,l);
                                          p->apsp.length += l;
                                          p->found += l;
                                          c += l;
                                          if (p->found == p->plength+6)
                                                  kwrite_pcm(t);
                                  } else {
                                          memcpy(p->apsp.payload+p->apsp.length,
                                                 buf+c,max);
                                          p->apsp.length += max;
                                          p->found += max;
                                          c += max;
                                          kwrite_pcm(t);
                                  }	
                          }				
                          break;
                        case PROG_STREAM_MAP:
                        case PRIVATE_STREAM2:
                        case PROG_STREAM_DIR:
                        case ECM_STREAM     :
                        case EMM_STREAM     :
                        case PADDING_STREAM :
                        case DSM_CC_STREAM  :
                        case ISO13522_STREAM:
                          if ( p->found + count - c < p->plength+6) {
                                  p->found += count-c;
                                  c = count;
                          } else {
                                  c += p->plength+6 - p->found;
                                  p->found = p->plength+6;
                          }
                          break;
                        }
                        if (p->plength && p->found == p->plength+6) {
                                p->found = 0;
                                p->done = 0;
                                p->plength = 0;
                        }
                }
        }
        return c;
}

static inline unsigned short get_pid(u8 *pid)
{
	u16 *pp;
	u8 ppid[2];

	ppid[0] = pid[0] & PID_MASK_HI;
	ppid[1] = pid[1];
	pp = (short *) ppid;
	return ntohs(*pp);
}


static void init_trans(trans_t *p, dvb_demux_filter_t *filter,
                       long (*fkt)(u8 const*buf, long int count, void *), 
                       void *dummy)
{
	int i;

	p->found = 0;
	
	p->pid1 = 0;
	p->pid2 = 0;
	p->cpid = 0;
        p->pes_out = fkt;

        p->pointer = dummy;
        p->filter = filter;

	for (i = 0; i < MAXFILT ; i++){
                clear_trans_filt(p,i);
	}	
}

static int set_trans_filt(trans_t *p, int filtn, u16 pid, u8 *mask, u8 *filt, int type)
{
	int i;

	if ( filtn >= MAXFILT || filtn<0 ) return -1;
	p->filter[filtn].pid = pid;
	p->filter[filtn].type = type;
	if (type == DMX_TYPE_SEC) {
          
		for (i = 0; i < MASKL ; i++){
                        p->filter[filtn].filter.filter_mask[i]  = mask[i];
			p->filter[filtn].filter.filter_value[i] = filt[i];
		}
	}		
	p->filter[filtn].state = DMX_STATE_GO;
	return 0;
}

static void clear_trans_filt(trans_t *p,int filtn)
{
	int i;

        p->filter[filtn].state = 0;
        p->filter[filtn].ts_state = 0;

	for (i = 0; i < MASKL; i++){
                p->filter[filtn].state   = DMX_STATE_FREE;
                p->filter[filtn].flags   = 0;
                p->filter[filtn].type    = -1;
                p->filter[filtn].ts_state = 0;
                p->filter[filtn].pid     = -1;

        }
	p->sec[filtn].found = 0;
	p->sec[filtn].length = 0;
        p->transcount[i] = 0;
}

static inline int filt_is_set(trans_t *p, int filtn)
{
	if (p->filter[filtn].state == DMX_STATE_GO) return 1;
	else return 0;
}

static inline int pes_is_set(trans_t *p, int filtn)
{
	if (p->filter[filtn].type == DMX_TYPE_PES) return 1;
	return 0;
}

static inline int pes_is_started(trans_t *p, int filtn)
{
	if (p->filter[filtn].ts_state & MY_STATE_PES_STARTED) return 1;
	return 0;
}

static inline int pes_is_start(trans_t *p, int filtn)
{
	if (p->filter[filtn].ts_state & MY_STATE_PES_START) return 1;
	return 0;
}

static inline int filt_is_ready(trans_t *p,int filtn)
{
	if (p->filter[filtn].ts_state & MY_STATE_FULL) return 1;
	return 0;
}



static inline void s_pes_started(trans_t *p, int filtn)
{
	p->filter[filtn].ts_state |= (u8) MY_STATE_PES_STARTED;
}

static inline void s_pes_start(trans_t *p, int filtn)
{
	p->filter[filtn].ts_state |= (u8) MY_STATE_PES_START;
}

static inline void s_filt_ready(trans_t *p,int filtn)
{
	p->filter[filtn].ts_state |= (u8) MY_STATE_FULL;
}

static inline void c_pes_started(trans_t *p, int filtn)
{
	p->filter[filtn].ts_state &= ~((u8) MY_STATE_PES_STARTED);
}

static inline void c_pes_start(trans_t *p, int filtn)
{
	p->filter[filtn].ts_state &= ~((u8) MY_STATE_PES_START);
}

static inline void c_filt_ready(trans_t *p,int filtn)
{
	p->filter[filtn].ts_state &= ~((u8) MY_STATE_FULL);
}


static void trans_filt(const u8 *buf, int count, trans_t *p)
{
	int c=0;

	while (c < count && p->found <1 ){
		if ( buf[c] == 0x47) p->found = 1;
		c++;
		p->packet[0] = 0x47;
	}
	if (c == count) return;
	
	while( c < count && p->found < TS_SIZE && p->found > 0 ){
		p->packet[p->found] = buf[c];
		c++;
		p->found++;
	}
	if (p->found == TS_SIZE){
		p->found = 0;
		ts_filter(p);
	}

	if (c < count) trans_filt(buf+c,count-c,p);
} 

static void ts_filter_out(trans_t *p, int filtn, int off)
{
	int count,c;
	u8 *buf;

	if (filtn < 0 || filtn >= MAXFILT) return; 

	count = TS_SIZE;
	c = TS_SIZE*filtn;
	buf = p->packet;
        s_filt_ready(p,filtn);
        memcpy(p->transbuf+c,buf,count);
        p->transcount[filtn] = count;
}

static void ts_filter(trans_t *p)
{
	int l,c;
	int tpid;
	u8 flag,flags;
	u8 adapt_length = 0;
	u8 cpid[2];


	cpid[0] = p->packet[1];
	cpid[1] = p->packet[2];
	tpid = get_pid(cpid);

	flag = cpid[0];
	flags = p->packet[3];
	
	if ( flags & ADAPT_FIELD ) {
		// adaption field
		adapt_length = p->packet[4];
	}

	c = 5 + adapt_length - (int)(!(flags & ADAPT_FIELD));
	if (flags & PAYLOAD){
		for ( l = 0; l < MAXFILT ; l++){
			if ( filt_is_set(p,l) ) {
				if ( p->filter[l].pid == tpid) {
                                        switch(p->filter[l].type){
                                        case DMX_TYPE_PES:
						if (cpid[0] & PAY_START){
                                                        s_pes_started(p,l);
                                                        s_pes_start(p,l);
						} else {
                                                        c_pes_start(p,l);
						}
						pes_filter(p,l,c);
                                                break;
                                                
                                        case DMX_TYPE_SEC:
						sec_filter(p,l,c);
                                                break;
                                        case DMX_TYPE_TS:
                                                ts_filter_out(p,l,c);
                                                break;
                                        }
                                }
			}
		}
	}
}	


static void pes_filter(trans_t *p, int filtn, int off)
{
	int count,c;
	u8 *buf;

	if (filtn < 0 || filtn >= MAXFILT) return; 

	count = TS_SIZE - off;
	c = TS_SIZE*filtn;
	buf = p->packet+off;
	if (pes_is_started(p,filtn)){
		s_filt_ready(p,filtn);
		memcpy(p->transbuf+c,buf,count);
		p->transcount[filtn] = count;
	}
}

static section_t *get_filt_sec(trans_t *p, int filtn)
{
	section_t *sec;
	
	sec = &p->sec[filtn];
        c_filt_ready(p,filtn);
	return sec;
}

static int get_filt_buf(trans_t *p, int filtn,u8 **buf)
{
	*buf = p->transbuf+TS_SIZE*filtn;
        c_filt_ready(p,filtn);
	return p->transcount[filtn];
}


static int match_filt(dvb_demux_filter_t *filter, u8 *buf)
{
        int i,j;
        int error = 0;

        for ( i = 0; i < MASKL; i++){
                if (i > 0 ) j=2+i;
                else j = 0;
                error +=  (buf[j] & filter->filter.filter_mask[i])^
                        (filter->filter.filter_value[i] &
                         filter->filter.filter_mask[i]);
        }
        return error;
}

static void sec_filter(trans_t *p, int filtn, int off)
{
	int error;
	int count,c;
	u8 *buf, *secbuf;
	section_t *sec;


	if (filtn < 0 || filtn >= MAXFILT) return; 

	count = TS_SIZE - off;
	c = 0;
	buf = p->packet+off;
	sec = &p->sec[filtn];
	secbuf = sec->payload;

	if(!filt_is_ready(p,filtn)){
		c_filt_ready(p,filtn);
		sec->found = 0;
		sec->length = 0;
	}
		
	if ( !sec->found ){
		c = buf[c]+1;
		if (c >= count) return;
		sec->id = buf[c];
		secbuf[0] = buf[c];
		c++;
		sec->found++;
		sec->length = 0;
	}
	
	while ( c < count && sec->found < 3){
		secbuf[sec->found] = buf[c];
		c++;
		sec->found++;
	}
	if (c == count) return;
	
	if (!sec->length && sec->found == 3){
		sec->length |= ((secbuf[1] & 0x0F) << 8); 
		sec->length |= (secbuf[2] & 0xFF);
	}
	
	while ( c < count && sec->found < sec->length+3){
		secbuf[sec->found] = buf[c];
		c++;
		sec->found++;
	}

	if ( sec->length && sec->found == sec->length+3 ){
		error = 0;
                error = match_filt(&(p->filter[filtn]),sec->payload);
		if (!error){
			s_filt_ready(p,filtn);
		}
		if (buf[0]+1 < c ) c=count;
	}
	
	if ( c < count ) sec_filter(p, filtn, off);

}




static int ts_av_pes(trans_t *p,const u8 *buf, int icount)
{
	u8 mask[MASKL];
	u8 filt[MASKL];
	int i,count,c;
        int size;
        u8 *cbuf;
	p2a_t *p1;
	p2a_t *p2;
        unsigned short ilength = 0;
	section_t *sec = NULL;

        p1 = &p->pesto[0];
        p2 = &p->pesto[1];

	for ( i = 0; i<MASKL; i++){
		mask[i] = 0x00;
		filt[i] = 0x00;
	}

        count = 0;


        if (!p->cpid){
                
                mask[0]= 0xFF;
                filt[0]= 0x00;
                
                clear_trans_filt(p,0);
                set_trans_filt(p,0,0,mask,filt,DMX_TYPE_SEC);

                while (count < icount && p->cpid == 0){
                        size = icount-count;
                        if (size > TS_SIZE) size = TS_SIZE;
                        trans_filt(buf+count, size, p);
                        count += size;
                        if (filt_is_ready(p,0)){
                                sec = get_filt_sec(p,0);
                                for (i = 8; i< sec->length-1 && 
                                             p->cpid == 0; i+=4){
                                        if (sec->payload[i] != 0 || 
                                            sec->payload[i+1] !=0){
                                                p->cpid = 
                                                        get_pid(sec->payload+i
                                                                +2);
                                                printk("TS: PMT PID: %04x\n",
                                                       p->cpid);
                                        }
                                }
                        }
                }
        }
		
        if (count == icount) return icount;

        if (!p->pid1 && !p->pid2 && p->cpid){
                mask[0]= 0xFF;
                filt[0]= 0x02;
                clear_trans_filt(p,0);
                set_trans_filt(p,0,p->cpid,mask,filt,DMX_TYPE_SEC);
		
                while (count < icount && !p->pid1 && !p->pid2){
                        size = icount-count;
                        if (size > TS_SIZE) size = TS_SIZE;
                        trans_filt(buf+count, size, p);
                        count += size;

                        if (filt_is_ready(p,0)){
                                sec = get_filt_sec(p,0);
                                ilength = (unsigned short)
                                        ((sec->payload[10]&15)<<8);
                                ilength |= (sec->payload[11]&0xFF);
                                for (i = 12+ilength; i< sec->length-1; i+=5){
                                        if (sec->payload[i] == 0x02){
                                                p->pid1 = 
                                                        get_pid(sec->payload+
                                                                i+1);
                                                printk("TS: VIDEO PID: %d\n",
                                                       p->pid1);
                                        }
                                        if (sec->payload[i] == 0x03 ||
                                            sec->payload[i] == 0x04){
                                                p->pid2 = 
                                                  get_pid(sec->payload+
                                                          i+1);
                                                printk("TS: AUDIO PID: %d\n",
                                                       p->pid2);
                                        }
                                        if (p->pid1 && p->pid2) break;
                                        i+=((sec->payload[i+3]&15)<<8)|
                                          sec->payload[i+4];
                                }
                        }
                }
        }
		
        if (count == icount) return icount;

        if (p->pid1 && p->pid2){
                set_trans_filt(p,0,p->pid1,NULL,NULL,DMX_TYPE_PES);
                set_trans_filt(p,1,p->pid2,NULL,NULL,DMX_TYPE_PES);
			
                while (count < icount){
                        size = icount-count;
                        if (size > TS_SIZE) size = TS_SIZE;
                        trans_filt(buf+count, size, p);
                        count += size;
                        
                        if (filt_is_ready(p,0)){
                                if ( pes_is_start(p,0) ){
                                        if (!p1->plength ){
                                                p1->found = 0;
                                                p1->done = 0;
                                        }
                                }
                                c = get_filt_buf(p,0,&cbuf);
                                p->which_pesto = 0;
                                p->pes_out(cbuf, c, (void *)p);
                        }

                        if (filt_is_ready(p,1)){
                                c = get_filt_buf(p,1,&cbuf);
                                if ( pes_is_start(p,1) ){
                                        if (!p2->plength){
                                                p2->found = 0;
                                                p2->done = 0;
                                        }
                                }
                                p->which_pesto = 1;
                                p->pes_out(cbuf, c, (void *)p);
                        }
                }
        }
        if (count == icount) return icount;

        return count;
} 






int w_pesh(u8 id,int length ,u8 *pts, u8 *obuf)
{
	u8 le[2];
	u8 dummy[3];
	int c;
	u8 headr[3] = {0x00, 0x00, 0x01};
	
	c = 0;
	memcpy(obuf+c,headr,3);
	c += 3;
	memcpy(obuf+c,&id,1);
	c++;

	le[0] = 0;
	le[1] = 0;
	if (length){
                if (pts) length+=8;
                else length+=3;
        }
	le[0] |= ((u8)(length >> 8) & 0xFF); 
	le[1] |= ((u8)(length) & 0xFF); 
	memcpy(obuf+c,le,2);
	c += 2;

	dummy[0] = 0x80;
	dummy[1] = 0;
	dummy[2] = 0;
	if (pts){
		dummy[1] |= PTS_ONLY;
		dummy[2] = 0x05;
	}
	memcpy(obuf+c,dummy,3);
	c += 3;

	if (pts){
		memcpy(obuf+c,pts,5);
		c += 5;
	}
	
	return c;
}


int w_tsh(u8 id,u8 *pts, u8 *obuf,a2p_t *p, int startpes)
{
	int c = 0;
	u8 tsheadv[4] = { 0x47, 0x00, 0x00, 0x10}; 
	u8 tsheada[4] = { 0x47, 0x00, 0x00, 0x10}; 

	if (id == 0xE0){
		if (startpes) tsheadv[1] = 0x40;
		tsheadv[3] |= (p->countv++ & 0x0F);
                tsheadv[2] = p->pidv & 0xFF;
                tsheadv[1] |= (p->pidv >> 8) & 0x1F;
		memcpy(obuf+c,tsheadv,4);
		c+=4;
		if (startpes) c += w_pesh(id,0,pts,obuf+c);
	} 

	if (id == 0xC0){
		if (startpes) tsheada[1] = 0x40;
		tsheada[3] |= (p->counta++ & 0x0F) ;
                tsheada[2] = p->pida & 0xff;
                tsheada[1] |= (p->pida >> 8) & 0x1F;
		memcpy(obuf+c,tsheada,4);
		c+=4;	
	}
	
	return c;
}

int write_ts_header(u16 pid, u8 *counter, int pes_start, u8 *buf)
{
	u8 tshead[4] = { 0x47, 0x00, 0x00, 0x10}; 
        
        if (pes_start) tshead[1] = 0x40;
        tshead[1] |= (u8)((pid & 0x1F00) >> 8);
        tshead[2] |= (u8)(pid & 0x00FF);
        tshead[3] |= ((*counter)++ & 0x0F) ;
        memcpy(buf,tshead,4);
        
        return 4;
}


static void init_ps(ps_packet_t *p)
{
	p->stuff_length = 0xF8;
	p->sheader_length = 0;
	p->audio_bound = 0;
	p->video_bound = 0;
	p->npes = 0;
	p->mpeg = 2;
}

static void setlength_ps(ps_packet_t *p)
{
	short *ll;
	ll = (short *) p->sheader_llength;
	if (p->mpeg == 2)
		p->sheader_length = ntohs(*ll) - 6;
	else 
		p->sheader_length = ntohs(*ll);
}	


int cwrite_ps(u8 *buf, ps_packet_t *p, long length)
{
	long count,i;
	u8 headr1[4] = {0x00, 0x00, 0x01, 0xBA };
	u8 headr2[4] = {0x00, 0x00, 0x01, 0xBB };
	u8 buffy = 0xFF;

	
	memcpy(buf,headr1,4);
	count = 4;
	if (p->mpeg == 2){
		memcpy(buf+count,p->scr,6);
		count += 6;
		memcpy(buf+count,p->mux_rate,3);
		count += 3;
		memcpy(buf+count,&p->stuff_length,1);
		count++;
		for(i=0; i< (p->stuff_length & 3); i++){
			memcpy(buf+count,&buffy,1);
			count++;
		}
	} else {
		memcpy(buf+count,p->scr,5);
		count += 5;
		memcpy(buf+count,p->mux_rate,3);
		count += 3;
	}
	if (p->sheader_length){
		memcpy(buf+count,headr2,4);
		count += 4;
		memcpy(buf+count,p->sheader_llength,2);
		count += 2;
		if ( p->mpeg == 2){
			memcpy(buf+count,p->rate_bound,3);
			count += 3;
			memcpy(buf+count,&p->audio_bound,1);
			count++;
			memcpy(buf+count,&p->video_bound,1);
			count++;
			memcpy(buf+count,&p->reserved,1);
			count++;
		}
		memcpy(buf+count,p->data,p->sheader_length);
		count += p->sheader_length;
	}

	return count;
}


void write_ps_headr(a2p_t *ap, ps_packet_t *p,u8 *pts)
{
	u8 buf[40];
	init_ps(p);
	
	p->mpeg = 2;
// SCR = 0
	p->scr[0] = 0x44;
	p->scr[1] = 0x00;
	p->scr[2] = 0x04;
	p->scr[3] = 0x00;
	p->scr[4] = 0x04;
	p->scr[5] = 0x01;

// SCR = PTS
	p->scr[0] = 0x44 | ((pts[0] >> 3)&0x18) | ((pts[0] >> 4)&0x03);
	p->scr[1] = 0x00 | ((pts[0] << 4)&0xF0) | ((pts[1] >> 4)&0x0F);
	p->scr[2] = 0x04 | ((pts[1] << 4)&0xF0) | ((pts[2] >> 4)&0x08)
		         | ((pts[2] >> 5)&0x03);
	p->scr[3] = 0x00 | ((pts[2] << 3)&0xF8) | ((pts[3] >> 5)&0x07);
	p->scr[4] = 0x04 | ((pts[3] << 3)&0xF8);
	p->scr[5] = 0x01;

// MUX Rate = 10080000 bits/sec (DVD)
	p->mux_rate[0] = 0x01;
	p->mux_rate[1] = 0x89;
	p->mux_rate[2] = 0xC3;
	p->stuff_length = 0xF8;

	p->rate_bound[0] = 0x80;
	p->rate_bound[1] = 0xC4;
	p->rate_bound[2] = 0xE1;


	p->sheader_llength[0] = 0x00;
	p->sheader_llength[1] = 0x12;

	setlength_ps(p);
	
	p->audio_bound = 0x00;
	p->video_bound = 0xe1;
	p->reserved = 0xFF;

	p->data[0] =  0xB9;
	p->data[1] =  0xE0;
	p->data[2] =  0xE8;
	p->data[3] =  0xB8;
	p->data[4] =  0xC0;
	p->data[5] =  0x20;
	p->data[6] =  0xBD;
	p->data[7] =  0xE0;
	p->data[8] =  0x44;
	p->data[9] =  0xBF;
	p->data[10] = 0xE0;
	p->data[11] = 0x02;
	
	

	
	cwrite_ps(buf, p, 38);
        ap->write_cb(buf, 38, ap->dataA);
}

void get_pespts(u8 *av_pts,u8 *pts)
{
	
	pts[0] = 0x21 | 
		((av_pts[0] & 0xC0) >>5);
	pts[1] = ((av_pts[0] & 0x3F) << 2) |
		((av_pts[1] & 0xC0) >> 6);
	pts[2] = 0x01 | ((av_pts[1] & 0x3F) << 2) |
		((av_pts[2] & 0x80) >> 6);
	pts[3] = ((av_pts[2] & 0x7F) << 1) |
		((av_pts[3] & 0x80) >> 7);
	pts[4] = 0x01 | ((av_pts[3] & 0x7F) << 1);
}


static void av_pes_cb(u8 const *buf, long int count, void *data)
{
        struct dvb_struct *dvb = (struct dvb_struct *) data;
        dvb_buf_put(&dvb->avin, (u8 *)buf, count);   
}

static void av_pes_cb_napi(u8 const *buf, long int count, void *data)
{
        dvb_demux_feed_t *dvbdmxfeed=(dvb_demux_feed_t *) data;
        
        dvbdmxfeed->cb.ts((u8 *) buf, count, 0, 0,
                          &dvbdmxfeed->feed.ts,
                          DMX_OK); 
}


static void init_a2p(a2p_t *p,void (*write_cb)(u8 const *buf, long int count,
                                     void *data), void *dataA, void *dataV, 
                     int type,u16 pidv,u16 pida )
{
	int i;

	p->type    = type;
	p->found   = 0;
	p->length  = 0;
	p->cid     = 0;
	p->flags   = 0;
	p->headr   = 0;
	p->alength = 0;
	p->vlength = 0;
	p->plength = 0;
        p->countv  = 0;
        p->countv  = 0;
        p->count0  = 0;
        p->count1  = 0;
        p->pidv    = pidv;
        p->pida    = pida;
        p->dataA   = dataA;
        p->dataV   = dataV;
	
	for ( i= 0; i < 4; i++){
		p->last_av_pts[i] = 0;
		p->av_pts[i] = 0;
		p->scr[i] = 0;
	}
        p->write_cb = write_cb;
}

static void inc_scr(a2p_t *p)
{
	u32 *s1,s2,*s3;
	
	s1 = (u32 *) p->scr;
	s3 = (u32 *) p->last_av_pts;
	s2 = ntohl(*s3);
	s2 -= 9000;
	*s1 = htonl(s2);
}

/*
static u_long crc_table[256] = {
	0x00000000, 0x04c11db7, 0x09823b6e, 0x0d4326d9, 0x130476dc, 0x17c56b6b,
	0x1a864db2, 0x1e475005, 0x2608edb8, 0x22c9f00f, 0x2f8ad6d6, 0x2b4bcb61,
	0x350c9b64, 0x31cd86d3, 0x3c8ea00a, 0x384fbdbd, 0x4c11db70, 0x48d0c6c7,
	0x4593e01e, 0x4152fda9, 0x5f15adac, 0x5bd4b01b, 0x569796c2, 0x52568b75,
	0x6a1936c8, 0x6ed82b7f, 0x639b0da6, 0x675a1011, 0x791d4014, 0x7ddc5da3,
	0x709f7b7a, 0x745e66cd, 0x9823b6e0, 0x9ce2ab57, 0x91a18d8e, 0x95609039,
	0x8b27c03c, 0x8fe6dd8b, 0x82a5fb52, 0x8664e6e5, 0xbe2b5b58, 0xbaea46ef,
	0xb7a96036, 0xb3687d81, 0xad2f2d84, 0xa9ee3033, 0xa4ad16ea, 0xa06c0b5d,
	0xd4326d90, 0xd0f37027, 0xddb056fe, 0xd9714b49, 0xc7361b4c, 0xc3f706fb,
	0xceb42022, 0xca753d95, 0xf23a8028, 0xf6fb9d9f, 0xfbb8bb46, 0xff79a6f1,
	0xe13ef6f4, 0xe5ffeb43, 0xe8bccd9a, 0xec7dd02d, 0x34867077, 0x30476dc0,
	0x3d044b19, 0x39c556ae, 0x278206ab, 0x23431b1c, 0x2e003dc5, 0x2ac12072,
	0x128e9dcf, 0x164f8078, 0x1b0ca6a1, 0x1fcdbb16, 0x018aeb13, 0x054bf6a4,
	0x0808d07d, 0x0cc9cdca, 0x7897ab07, 0x7c56b6b0, 0x71159069, 0x75d48dde,
	0x6b93dddb, 0x6f52c06c, 0x6211e6b5, 0x66d0fb02, 0x5e9f46bf, 0x5a5e5b08,
	0x571d7dd1, 0x53dc6066, 0x4d9b3063, 0x495a2dd4, 0x44190b0d, 0x40d816ba,
	0xaca5c697, 0xa864db20, 0xa527fdf9, 0xa1e6e04e, 0xbfa1b04b, 0xbb60adfc,
	0xb6238b25, 0xb2e29692, 0x8aad2b2f, 0x8e6c3698, 0x832f1041, 0x87ee0df6,
	0x99a95df3, 0x9d684044, 0x902b669d, 0x94ea7b2a, 0xe0b41de7, 0xe4750050,
	0xe9362689, 0xedf73b3e, 0xf3b06b3b, 0xf771768c, 0xfa325055, 0xfef34de2,
	0xc6bcf05f, 0xc27dede8, 0xcf3ecb31, 0xcbffd686, 0xd5b88683, 0xd1799b34,
	0xdc3abded, 0xd8fba05a, 0x690ce0ee, 0x6dcdfd59, 0x608edb80, 0x644fc637,
	0x7a089632, 0x7ec98b85, 0x738aad5c, 0x774bb0eb, 0x4f040d56, 0x4bc510e1,
	0x46863638, 0x42472b8f, 0x5c007b8a, 0x58c1663d, 0x558240e4, 0x51435d53,
	0x251d3b9e, 0x21dc2629, 0x2c9f00f0, 0x285e1d47, 0x36194d42, 0x32d850f5,
	0x3f9b762c, 0x3b5a6b9b, 0x0315d626, 0x07d4cb91, 0x0a97ed48, 0x0e56f0ff,
	0x1011a0fa, 0x14d0bd4d, 0x19939b94, 0x1d528623, 0xf12f560e, 0xf5ee4bb9,
	0xf8ad6d60, 0xfc6c70d7, 0xe22b20d2, 0xe6ea3d65, 0xeba91bbc, 0xef68060b,
	0xd727bbb6, 0xd3e6a601, 0xdea580d8, 0xda649d6f, 0xc423cd6a, 0xc0e2d0dd,
	0xcda1f604, 0xc960ebb3, 0xbd3e8d7e, 0xb9ff90c9, 0xb4bcb610, 0xb07daba7,
	0xae3afba2, 0xaafbe615, 0xa7b8c0cc, 0xa379dd7b, 0x9b3660c6, 0x9ff77d71,
	0x92b45ba8, 0x9675461f, 0x8832161a, 0x8cf30bad, 0x81b02d74, 0x857130c3,
	0x5d8a9099, 0x594b8d2e, 0x5408abf7, 0x50c9b640, 0x4e8ee645, 0x4a4ffbf2,
	0x470cdd2b, 0x43cdc09c, 0x7b827d21, 0x7f436096, 0x7200464f, 0x76c15bf8,
	0x68860bfd, 0x6c47164a, 0x61043093, 0x65c52d24, 0x119b4be9, 0x155a565e,
	0x18197087, 0x1cd86d30, 0x029f3d35, 0x065e2082, 0x0b1d065b, 0x0fdc1bec,
	0x3793a651, 0x3352bbe6, 0x3e119d3f, 0x3ad08088, 0x2497d08d, 0x2056cd3a,
	0x2d15ebe3, 0x29d4f654, 0xc5a92679, 0xc1683bce, 0xcc2b1d17, 0xc8ea00a0,
	0xd6ad50a5, 0xd26c4d12, 0xdf2f6bcb, 0xdbee767c, 0xe3a1cbc1, 0xe760d676,
	0xea23f0af, 0xeee2ed18, 0xf0a5bd1d, 0xf464a0aa, 0xf9278673, 0xfde69bc4,
	0x89b8fd09, 0x8d79e0be, 0x803ac667, 0x84fbdbd0, 0x9abc8bd5, 0x9e7d9662,
	0x933eb0bb, 0x97ffad0c, 0xafb010b1, 0xab710d06, 0xa6322bdf, 0xa2f33668,
	0xbcb4666d, 0xb8757bda, 0xb5365d03, 0xb1f740b4};

u_long crc32 (unsigned char *data, int len)
{
	register int i;
	u_long crc = 0xffffffff;

	for (i=0; i<len; i++)
		crc = (crc << 8) ^ crc_table[((crc >> 24) ^ *data++) & 0xff];

	return crc;
}

static void set_crc32(u8 *tspid, int length)
{
        u32 crc;

        crc=crc32(tspid,length);
        tspid[length]   = (crc>>24)&0xff;
        tspid[length+1] = (crc>>16)&0xff;
        tspid[length+2] = (crc>>8)&0xff;
        tspid[length+3] = (crc)&0xff;
}
*/
  

static void av_pes_to_pes(a2p_t *p, u8 *buf,int count)
{
	int c=0;
	int l,i;
	u8 pts[5];
	ps_packet_t ps;


	u8 tspid0[TS_SIZE] = { 
                0x47, 0x40, 0x00, 0x10, 0x00, 0x00, 0xb0, 0x11, 
                0x00, 0x00, 0xcb, 0x00, 0x00, 0x00, 0x00, 0xe0, 
                0x10, 0x00, 0x01, 0xe4, 0x00, 0x2a, 0xd6, 0x1a, 
                0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff
        };

	u8 tspid1[TS_SIZE] = { 
                0x47, 0x44, 0x00, 0x10, 0x00, 0x02, 0xb0, 0x1c,
                0x00, 0x01, 0xcb, 0x00, 0x00, 0xe0, 0xa0, 0xf0, 
                0x05, 0x48, 0x03, 0x01, 0x00, 0x00, 0x02, 0xe0,
                0xa0, 0xf0, 0x00, 0x03, 0xe0, 0x50, 0xf0, 0x00, 
                0xae, 0xea, 0x4e, 0x48, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
                0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
                0xff, 0xff, 0xff, 0xff
        };

	while (c < count && p->found < 2 ) {
		if (buf[c] == 0x41 ) p->found = 1;
		c++;
		if (p->found && buf[c] == 0x56) {
			p->found = 2;
			c++;
		} else p->found = 0;
	}
	
	if ( c == count ) return;
 
	if ( p->found == 2 ) {
		p->cid = buf[c];
		c++;
		p->found++;
	}
	
	if ( c == count ) return;

        if ( p->found > 2 ) {
		while ( c < count && p->found < 5){
			c++;
			p->found++;
		}
	}
	
	if ( c == count ) return;
	
	if ( p->found == 5 ) {
		p->flags = buf[c];
		c++;
		p->found++;
	}

	if ( c == count ) return;

	if ( p->found == 6 ) {
		while ( c < count && p->found < 8){
			if ( p->found == 6 ) {
				p->length =(int)( ( buf[c] & 0xFF ) << 8);
				c++;
				p->found++;
			} else {
				p->length |= ( buf[c] & 0xFF );
				c++;
				p->found++;
			}
		}
	}

	if ( p->found > 7 ){
		while ( p->type == TS_STREAM &&
			c < count && p->found < p->length+8){
			switch (p->cid){
			case 0x01:
				if ( p->found == 8){
					tspid0[3] |= (p->count0++) 
						& 0x0F ;
					tspid1[3] |= (p->count1++) 
						& 0x0F ;
                                        
                                        tspid1[24]  = p->pidv;
                                        tspid1[23] |= (p->pidv >> 8) & 0x3F;
                                        tspid1[29]  = p->pida;
                                        tspid1[28] |= (p->pida >> 8) & 0x3F;

                                        p->write_cb(tspid0,TS_SIZE,p->dataA);
                                        p->write_cb(tspid1,TS_SIZE,p->dataA);
				}
				
				if (!(p->flags & PTS_FLAG)&& !p->headr){
                                        int pesstart = 0;
                                        
                                        if (p->plength+count-c > MAX_PLENGTH){
                                                pesstart = 1;
                                                p->plength = 0;
                                        }

					p->vlength = w_tsh(0xE0, NULL,
                                                           p->vbuf,p,pesstart);
					p->headr = 1;
				}
				
				if ( (p->flags & PTS_FLAG) && !p->headr){
					if ( p->found < 12 ){
						p->av_pts[p->found-8] = buf[c];
						c++;
						p->found++;
					}
					
					if ( p->found == 12 ){
						get_pespts(p->av_pts,pts);
						
						p->vlength = w_tsh( 0xE0, pts,
                                                                    p->vbuf,p
                                                                    ,1);
                                                p->plength = 0;
	
						p->headr = 1;
						memcpy(p->last_av_pts,
						       p->av_pts,4);
					}
				}

				if ( p->headr ){
					l = count - c;
					if ( l + p->found > p->length+8 )
						l = p->length - p->found+8;
					
					if (p->vlength+l < TS_SIZE){
						memcpy(p->vbuf+p->vlength, 
						       buf+c, l);
						p->vlength += l;
						c += l;
						p->found += l;
						if (p->found == p->length+8){
							// adaptation field
							p->vbuf[3] |= 0x30;
							l = TS_SIZE-p->vlength
                                                                +4;
							memmove(p->vbuf+l,
								p->vbuf+4,
								p->vlength-4);
							p->vbuf[4] = l-5;
							if (l-5)
								p->vbuf[5] = 
									0x00;
							for ( i = 6; i < l; 
							      i++)
								p->vbuf[i] = 
									0xFF;

							p->plength += 
								p->vlength-4; 

                                                        p->write_cb(p->vbuf,
                                                                    TS_SIZE,
                                                                    p->dataV);
                                                        
                                                        
							p->vlength = 0;
							p->headr = 0;
							p->flags = 0;
						}
					} else {
						l = TS_SIZE - p->vlength;
						memcpy(p->vbuf+p->vlength, 
						       buf+c, l);
						p->vlength += l;
						c += l;
						p->found += l;
						
                                                p->plength += p->vlength-4; 

                                                p->write_cb(p->vbuf,
                                                            p->vlength,
                                                            p->dataV);

						p->vlength = 0;
						p->headr = 0;
						p->flags = 0;
					}
				}
				break;

			case 0x02:
				
				if ( !p->headr ){
					int pesstart = 0;
					
					if ( ((p->flags & PTS_FLAG) ||
					      (count -c >4 &&
					       buf[c] == 0x00 &&
					       buf[c+1] == 0x00 &&
					       buf[c+2] == 0x01 &&
					       buf[c+3] == 0xC0))
						) pesstart = 1;					
					p->alength = w_tsh(0xC0, NULL,
                                                           p->abuf,p,
							   pesstart);
					p->headr = 1;
				}

				if ( p->headr ){
					l = count - c;
					if ( l + p->found > p->length+8 )
						l = p->length - p->found+8;
					
					if (p->alength+l < TS_SIZE){
						memcpy(p->abuf+p->alength, 
						       buf+c, l);
						p->alength += l;
						c += l;
						p->found += l;
						if (p->found == p->length+8){
							// adaptation field
							p->abuf[3] |= 0x30;
							l = TS_SIZE-p->alength
                                                                +4;
							memmove(p->abuf+l,
								p->abuf+4,
								p->alength-4);
							p->abuf[4] = l-5;
							if (l-5)
								p->abuf[5] = 
									0x00;
							for ( i = 6; i < l; 
							      i++)
								p->abuf[i] = 
									0xFF;
                                                        p->write_cb(p->abuf,
                                                                    TS_SIZE,
                                                                    p->dataA);
							p->alength = 0;
							p->headr = 0;
							p->flags = 0;
						}
					} else {
						l = TS_SIZE - p->alength;
						memcpy(p->abuf+p->alength, 
						       buf+c, l);
						p->alength += l;
						c += l;
						p->found += l;
                                                p->write_cb(p->abuf,
                                                            p->alength,
                                                            p->dataA);

						p->alength = 0;
						p->headr = 0;
						p->flags = 0;
					}
				}
				break;
			}
		}
	
		while ( p->type != TS_STREAM &&
			c < count && p->found < p->length+8){
			switch (p->cid){
			case 0x01:
				if ( p->found == 8 && (p->flags & PTS_FLAG) 
				     && p->vlength ){
					l = p->vlength - 6;
					p->vbuf[4] = 0;
					p->vbuf[5] = 0;
					p->vbuf[4] |= ((u8)(l >> 8) & 0xFF); 
					p->vbuf[5] |= ((u8)(l) & 0xFF); 
					if (p->type == PS_STREAM){
						inc_scr(p);
						write_ps_headr(p,&ps,p->scr);
					}
                                        p->write_cb(p->vbuf,
                                                    p->vlength,
                                                    p->dataV);

					p->vlength=0;
					p->headr = 0;
				}
				
				if (!(p->flags & PTS_FLAG)&& !p->headr){
					p->vlength = w_pesh(0xE0,p->length,
							    NULL,p->vbuf);
					p->headr = 1;
				}
				
				if ( (p->flags & PTS_FLAG) && !p->headr){
					if ( p->found < 12 ){
						p->av_pts[p->found-8] = buf[c];
						c++;
						p->found++;
					}
					
					if ( p->found == 12 ){
						get_pespts(p->av_pts,pts);
					
						p->vlength = w_pesh(
							0xE0,p->length,
							pts, p->vbuf);
						
						p->headr = 1;
						memcpy(p->last_av_pts,
						       p->av_pts,4);
					}
				}

				if ( p->headr ){
					l = count - c;
					if ( l + p->found > p->length+8 )
						l = p->length - p->found+8;

					if (p->vlength+l < MAX_PLENGTH){
						memcpy(p->vbuf+p->vlength, 
						       buf+c, l);
						p->vlength += l;
						c += l;
						p->found += l;
					} else {
						inc_scr(p);
						l = p->vlength - 6;
						p->vbuf[4] = 0;
						p->vbuf[5] = 0;
						p->vbuf[4] |= ((u8)(l >> 8) 
							       & 0xFF); 
						p->vbuf[5] |= ((u8)(l) 
							       & 0xFF); 
                                                p->write_cb(p->vbuf,
                                                            p->vlength,
                                                            p->dataV);
						p->vlength=0;
						p->headr = 0;
					}
				}
				break;

			case 0x02:
				l = count - c;
				if ( l + p->found > p->length+8 )
					l = p->length - p->found +8;

				if ( p->found == 8 && 
				     p->alength ){
					p->abuf[4] = 0;
					p->abuf[5] = 0;
					p->abuf[4] |= 
						(u8)(((p->alength-6) >> 8) & 
						     0xFF); 
					p->abuf[5] |= 
						(u8)((p->alength-6) & 
						     0xFF); 
                                        p->write_cb(p->abuf,
                                                    p->alength,
                                                    p->dataA);
					if (   (count -c >4 &&
						 buf[c] == 0x00 &&
						 buf[c+1] == 0x00 &&
						 buf[c+2] == 0x01 &&
						 buf[c+3] == 0xC0)
                                               ||(p->flags & PTS_FLAG)
                                               ){
						p->alength=0;
					} else {
						p->alength = w_pesh(
							0xC0,0,
							NULL, p->abuf);
					}
				}
		
				memcpy(p->abuf+p->alength,buf+c,l);
				p->alength += l;
				c += l;
				p->found += l;

				break;
			}
		}
	}
	
		  
	
	
	if ( p->length+8 == p->found ) {
		p->found = 0;
		p->length = 0;
		p->cid = 0;
		p->flags = 0;
	}
	if ( c < count )
		av_pes_to_pes( p, buf+c, count-c);
	
}


void init_p2t(p2t_t *p, dvb_demux_feed_t *feed)
{
	memset(p->pes,0,TS_SIZE);
	p->counter = 0;
	p->pos = 0;
	p->frags = 0;
	if (feed) p->feed = feed;
}

void clear_p2t(p2t_t *p)
{
	memset(p->pes,0,TS_SIZE);
	p->counter = 0;
	p->pos = 0;
	p->frags = 0;
}


long int find_pes_header(u8 const *buf, long int length, int *frags)
{
	int c = 0;
	int found = 0;

	*frags = 0;

	while (c < length-3 && !found) {
		if (buf[c] == 0x00 && buf[c+1] == 0x00 && 
		    buf[c+2] == 0x01) {
			switch ( buf[c+3] ) {
				
			case PROG_STREAM_MAP:
			case PRIVATE_STREAM2:
			case PROG_STREAM_DIR:
			case ECM_STREAM     :
			case EMM_STREAM     :
			case PADDING_STREAM :
			case DSM_CC_STREAM  :
			case ISO13522_STREAM:
			case PRIVATE_STREAM1:
			case AUDIO_STREAM_S ... AUDIO_STREAM_E:
			case VIDEO_STREAM_S ... VIDEO_STREAM_E:
				found = 1;
				break;
				
			default:
				c++;
				break;
			}	
		} else c++;
	}
	if (c == length-3 && !found){
		if (buf[length-1] == 0x00) *frags = 1;
		if (buf[length-2] == 0x00 &&
		    buf[length-1] == 0x00) *frags = 2;
		if (buf[length-3] == 0x00 &&
		    buf[length-2] == 0x00 &&
		    buf[length-1] == 0x01) *frags = 3;
		return -1;
	}

	return c;
}

void pes_to_ts( u8 const *buf, long int length, u16 pid, p2t_t *p)
{
	int c,c2,l,add;
	int check,rest;

	c = 0;
	c2 = 0;
	if (p->frags){
		check = 0;
		switch(p->frags){
		case 1:
			if ( buf[c] == 0x00 && buf[c+1] == 0x01 ){
				check = 1;
				c += 2;
			}
			break;
		case 2:
			if ( buf[c] == 0x01 ){
				check = 1;
				c++;
			}
			break;
		case 3:
			check = 1;
		}
		if(check){
			switch ( buf[c] ) {
				
			case PROG_STREAM_MAP:
			case PRIVATE_STREAM2:
			case PROG_STREAM_DIR:
			case ECM_STREAM     :
			case EMM_STREAM     :
			case PADDING_STREAM :
			case DSM_CC_STREAM  :
			case ISO13522_STREAM:
			case PRIVATE_STREAM1:
			case AUDIO_STREAM_S ... AUDIO_STREAM_E:
			case VIDEO_STREAM_S ... VIDEO_STREAM_E:
				p->pes[0] = 0x00;
				p->pes[1] = 0x00;
				p->pes[2] = 0x01;
				p->pes[3] = buf[c];
				p->pos=4;
				memcpy(p->pes+p->pos,buf+c,(TS_SIZE-4)-p->pos);
				c += (TS_SIZE-4)-p->pos;
				p_to_t(p->pes,(TS_SIZE-4),pid,&p->counter,
				       p->feed);
				clear_p2t(p);
				break;
				
			default:
				c=0;
				break;
			}
		}
		p->frags = 0;
	}
		
	if (p->pos){
		c2 = find_pes_header(buf+c,length-c,&p->frags);
		if (c2 >= 0 && c2 < (TS_SIZE-4)-p->pos){
			l = c2+c;
		} else l = (TS_SIZE-4)-p->pos;
		memcpy(p->pes+p->pos,buf,l);
		c += l;
		p->pos += l;
		p_to_t(p->pes,p->pos,pid,&p->counter, p->feed);
		clear_p2t(p);
	}
			
	add = 0;
	while (c < length){
		c2 = find_pes_header(buf+c+add,length-c-add,&p->frags);
		if (c2 >= 0) {
			c2 += c+add;
			if (c2 > c){
				p_to_t(buf+c,c2-c,pid,&p->counter,
				       p->feed);
				c = c2;
				clear_p2t(p);
				add = 0;
			} else add = 1;
		} else {
			l = length-c;
			rest = l % (TS_SIZE-4);
			l -= rest;
			p_to_t(buf+c,l,pid,&p->counter,
			       p->feed);
			memcpy(p->pes,buf+c+l,rest);
			p->pos = rest;
			c = length;
		}
	}
}


int write_ts_header2(u16 pid, u8 *counter, int pes_start, u8 *buf, u8 length)
{
	int i;
	int c = 0;
	int fill;
	u8 tshead[4] = { 0x47, 0x00, 0x00, 0x10}; 
        

	fill = (TS_SIZE-4)-length;
        if (pes_start) tshead[1] = 0x40;
	if (fill) tshead[3] = 0x30;
        tshead[1] |= (u8)((pid & 0x1F00) >> 8);
        tshead[2] |= (u8)(pid & 0x00FF);
        tshead[3] |= ((*counter)++ & 0x0F) ;
        memcpy(buf,tshead,4);
	c+=4;


	if (fill){
		buf[4] = fill-1;
		c++;
		if (fill >1){
			buf[5] = 0x00;
			c++;
		}
		for ( i = 6; i < fill+4; i++){
			buf[i] = 0xFF;
			c++;
		}
	}

        return c;
}


void p_to_t(u8 const *buf, long int length, u16 pid, u8 *counter, 
            dvb_demux_feed_t *feed)
{
  
	int l, pes_start;
	u8 obuf[TS_SIZE];
	long int c = 0;

	pes_start = 0;
	if ( length > 3 && 
             buf[0] == 0x00 && buf[1] == 0x00 && buf[2] == 0x01 )
		switch (buf[3]){
			case PROG_STREAM_MAP:
			case PRIVATE_STREAM2:
			case PROG_STREAM_DIR:
			case ECM_STREAM     :
			case EMM_STREAM     :
			case PADDING_STREAM :
			case DSM_CC_STREAM  :
			case ISO13522_STREAM:
			case PRIVATE_STREAM1:
			case AUDIO_STREAM_S ... AUDIO_STREAM_E:
			case VIDEO_STREAM_S ... VIDEO_STREAM_E:
				pes_start = 1;
				break;
				
			default:
				break;
		}			

	while ( c < length ){
		memset(obuf,0,TS_SIZE);
		if (length - c >= (TS_SIZE-4)){
			l = write_ts_header2(pid, counter, pes_start
					     , obuf, (TS_SIZE-4));
			memcpy(obuf+l, buf+c, TS_SIZE-l);
			c += TS_SIZE-l;
		} else { 
			l = write_ts_header2(pid, counter, pes_start
					     , obuf, length-c);
			memcpy(obuf+l, buf+c, TS_SIZE-l);
			c = length;
		}
                feed->cb.ts(obuf, 188, 0, 0, &feed->feed.ts, DMX_OK); 
		pes_start = 0;
	}
}



/****************************************************************************
 * V4L2 SECTION
 ****************************************************************************/

#ifdef V4L2

/*
 *  Supported capture formats
 */
static struct v4l2_fmtdesc capfmt[] = 
{
	{	0, {"RGB-16 (5-5-5)"},
		V4L2_PIX_FMT_RGB555,  0, 16, {0, 0},
	},
	{	1, {"RGB-16 (5-6-5)"},
		V4L2_PIX_FMT_RGB565,  0, 16, {0, 0},
	},
	{	2, {"RGB-24 (B-G-R)"},
		V4L2_PIX_FMT_BGR24,   0, 24, {0, 0},
	},
	{	3, {"RGB-32 (B-G-R-?)"},
		V4L2_PIX_FMT_BGR32,   0, 32, {0, 0},
	},
	{	4, {"Greyscale-8"},
		V4L2_PIX_FMT_GREY,    V4L2_FMT_CS_601YUV, 8, {0, 0},
	},
	{	5, {"YUV 4:2:2 (Y-U-Y-V)"},
		V4L2_PIX_FMT_YUYV,    V4L2_FMT_CS_601YUV, 16, {0, 0},
	},
	{	6, {"YUV 4:2:2 (U-Y-V-Y)"},
		V4L2_PIX_FMT_UYVY,    V4L2_FMT_CS_601YUV, 16, {0, 0},
	},
	{	7, {"YVU 4:2:2 Planar (Y-V-U)"},
		V4L2_PIX_FMT_YVU422P,    V4L2_FMT_CS_601YUV, 16, {0, 0},
	},
	{	8, {"YUV 4:1:1 Planar (Y-V-U)"},
		V4L2_PIX_FMT_YVU411P,    V4L2_FMT_CS_601YUV, 12, {0, 0},
	},
	{	9, {"YVU 4:2:0 Planar (Y-V-U)"},
		V4L2_PIX_FMT_YVU420,    V4L2_FMT_CS_601YUV, 12, {0, 0},
	},
	{	10, {"YUV 4:2:0 Planar (Y-U-V)"},
		V4L2_PIX_FMT_YUV420,    V4L2_FMT_CS_601YUV, 12, {0, 0},
	},
	{	11, {"YVU 4:1:0 Planar (Y-V-U)"},
		V4L2_PIX_FMT_YVU410,    V4L2_FMT_CS_601YUV, 9, {0, 0},
	},
	{	12, {"YUV 4:1:0 Planar (Y-U-V)"},
		V4L2_PIX_FMT_YUV410,    V4L2_FMT_CS_601YUV, 9, {0, 0},
	},
};
#define NUM_CAPFMT (sizeof(capfmt)/sizeof(capfmt[0]))


/*  The arguments are already copied into kernel memory, so don't use
    copy_from_user() or copy_to_user() on arg.  */
static int v4l2_ioctl(void *id, unsigned int cmd, void *arg)
{
	struct device_open *o = (struct device_open *)id;
	struct capture_device *dev = o->dev;

	switch(cmd)
	{
        case VIDIOCGFRONTEND:
        {
                GetFront(dev);
		memcpy(arg, &dev->front, sizeof(struct frontend));
                return 0;
        }

        case VIDIOCSFRONTEND:
        {
                struct frontend front;

                memcpy(&front, arg, sizeof(struct frontend));
                return SetFront(dev, &front);
        }

	case VIDIOC_QUERYCAP:
	{
		struct v4l2_capability *b = arg;
		strcpy(b->name, dev->v.name);
		b->type = V4L2_TYPE_CAPTURE;
		b->flags =  V4L2_FLAG_READ |
                        V4L2_FLAG_WRITE |
                        V4L2_FLAG_STREAMING |
                        V4L2_FLAG_SELECT |
                        V4L2_FLAG_PREVIEW |
                        V4L2_FLAG_TUNER;
		b->inputs = 1;
		b->outputs = 1;
		b->audios = 2;
		b->maxwidth = 720;
		b->maxheight = 576;
		b->minwidth = 32;
		b->minheight = 32;
		b->maxframerate = 30;
		return 0;
	}

	case VIDIOC_ENUM_PIXFMT:
	{

		struct v4l2_fmtdesc *f = arg;
		if (f->index < 0 || f->index >= NUM_CAPFMT)
			return -EINVAL;
		*f = capfmt[f->index];

		return 0;
	}

	case VIDIOC_G_FMT:
	{
/*
		struct v4l2_format *fmt = arg;
		if (fmt->type != V4L2_BUF_TYPE_CAPTURE)
		{
			dprintk("G_FMT wrong buffer type %d\n",fmt->type);
			return -EINVAL;
		}
		memcpy(&fmt->fmt.pix, &dev->clientfmt, sizeof(dev->clientfmt));
*/
		return 0;
	}

	case VIDIOC_S_FMT:
	{
/*
		int ret = 0;
		struct v4l2_format *fmt = arg;
		if (o->noncapturing)
		{
			dprintk("S_FMT illegal in non-capturing open\n");
			return -EPERM;
		}
		memcpy(&dev->clientfmt, &fmt->fmt.pix, sizeof(dev->clientfmt));
		ret = capture_new_format(dev);
		memcpy(&fmt->fmt.pix, &dev->clientfmt, sizeof(dev->clientfmt));
		if(!ret)return -EINVAL;
		mmap_unrequest_buffers(dev);
*/
		return 0;
	}

	case VIDIOC_G_COMP:	return -EINVAL;
	case VIDIOC_S_COMP:	return -EINVAL;

	case VIDIOC_REQBUFS:
	{
/*
		struct v4l2_requestbuffers *req = arg;
		if (o->noncapturing)
		{
			dprintk("REQBUFS illegal in non-capturing open\n");
			return -EPERM;
		}
		if (dev->stream_buffers_mapped)
		{
			dprintk("Can't request buffers if buffers are "
				  "already mapped\n");
			return -EPERM;
		}
		capture_begin(dev);
		if (!mmap_request_buffers(dev, req))
			return -EINVAL;
*/
		return 0;
	}

	case VIDIOC_QUERYBUF:
	{
/*
		struct v4l2_buffer *buf = arg;
		int	i;
		if (o->noncapturing)
		{
			dprintk("QUERYBUF illegal in non-capturing open\n");
			return -EPERM;
		}
		i = buf->index;
		if (i < 0 || i >= MAX_CAPTURE_BUFFERS ||
		    !dev->stream_buf[i].requested ||
		    (buf->type & V4L2_BUF_TYPE_field) != 
		     (dev->stream_buf[i].vidbuf.type & V4L2_BUF_TYPE_field))
		{
			dprintk("QUERYBUF bad parameter\n");
			return -EINVAL;
		}
		*buf = dev->stream_buf[i].vidbuf;
*/
		return 0;
	}

	case VIDIOC_QBUF:
	{
/*
		struct v4l2_buffer *buf = arg;
		if (o->noncapturing)
		{
			dprintk("QBUF illegal in non-capturing open\n");
			return -EPERM;
		}
		if (!dev->stream_buffers_mapped)
		{
			dprintk("QBUF no buffers are mapped\n");
			return -EINVAL;
		}
		if (!capture_queuebuffer(dev, buf))
			return -EINVAL;
*/
		return 0;
	}

	case VIDIOC_DQBUF:
	{
		return -EINVAL;
/*
		struct v4l2_buffer *buf = arg;

		if (o->noncapturing)
		{
			dprintk("DQBUF illegal in non-capturing open\n");
			return -EPERM;
		}
		if (!capture_dequeuebuffer(dev, buf))
			return -EINVAL;
		return 0;
*/
	}

	case VIDIOC_STREAMON:
	{
/*
		__u32	type = *(__u32 *)arg;
		if (o->noncapturing)
		{
			dprintk("STREAMON illegal in non-capturing open\n");
			return -EPERM;
		}
		if (!capture_streamon(dev, type))
			return -EINVAL;
*/
		return 0;
	}

	case VIDIOC_STREAMOFF:
	{
/*
		__u32	type = *(__u32 *)arg;
		if (o->noncapturing)
		{
			dprintk("STREAMOFF illegal in non-capturing open\n");
			return -EPERM;
		}
		capture_streamoff(dev, type);
*/
		return 0;
	}

	case VIDIOC_ENUM_FBUFFMT:
	{
/*
		struct v4l2_fmtdesc *f = arg;

		dprintk("enumfmt\n");

		if (f->index < 0 || f->index >= NUM_PVFMT)
			return -EINVAL;
		*f = pvfmt[f->index];
*/
		return 0;
	}
	case VIDIOC_G_FBUF:		
	{
/*
		memcpy(arg, &dev->pvfb, sizeof(dev->pvfb));

		dprintk("gfbuf\n");
*/		
		return 0;
	}
	case VIDIOC_S_FBUF:		
	{
/*
		struct v4l2_framebuffer *fbuf = arg;
		
		if(!capable(CAP_SYS_ADMIN))
			return -EPERM;

		if(dev->pvstat&PVSTAT_RUN)
		{
			err_msg("No way - not going to change the framebuffer while PV is active!\n");
			return -EINVAL;
		}
		
		dev->pvfb.flags = fbuf->flags;
		dev->pvfb.base[0] = fbuf->base[0];
		dev->pvfb.base[1] = fbuf->base[1];
		dev->pvfb.base[2] = fbuf->base[2];
		memcpy(&dev->pvfb.fmt, &fbuf->fmt, sizeof(dev->pvfb.fmt));
		if(!(fbuf->fmt.flags&V4L2_FMT_FLAG_BYTESPERLINE))
		{
			fbuf->fmt.bytesperline = (fbuf->fmt.width*fbuf->fmt.depth)>>3;
			dprintk("setting bpl from width\n");
		}
		dprintk("sfbuf %d x %d - %d\n", fbuf->fmt.width, fbuf->fmt.height,fbuf->fmt.bytesperline);
		dev->pvstat=PVSTAT_FBUF; 
*/		
		return 0;
	}
	case VIDIOC_G_WIN:		
	{
/*
		memcpy(arg, &dev->pvwin, sizeof(dev->pvwin));
		dprintk("gwin\n");
*/
		return 0;
	}
	case VIDIOC_S_WIN:		
	{
/*
		int i = 0;
		
		dprintk("enter swin\n"); 
		
		if(!(dev->pvstat&PVSTAT_FBUF))
		{
			err_msg("cannot set window w/out framebuffer\n");
			return -EINVAL;
		}
		
		if(dev->pvstat&PVSTAT_RUN)
		{
			i = 1;
			pv_off(dev);
		}
		dprintk("swin\n");
		memcpy(&dev->pvwin, arg, sizeof(dev->pvwin));

		if(pv_new_win(dev))
			dev->pvstat|=PVSTAT_WIN;
		else
			dev->pvstat&=~PVSTAT_WIN;
		
		if(i)pv_on(dev);
		
		memcpy(arg, &dev->pvwin, sizeof(dev->pvwin));
		dprintk("exit swin\n"); 
*/
		return 0;
	}

	case VIDIOC_PREVIEW:
	{
/*
		__u32	i = *(__u32 *)arg;
		
		dprintk("base: %08X\n", (u32)dev->pvfb.base[0]);
		
		if(i)
			pv_on(dev);
		else
			pv_off(dev);
*/
		return 0;
	}

	case VIDIOC_G_PERF:
	{
/*
		memcpy(arg, &dev->perf, sizeof(dev->perf));
*/
		return 0;
	}

	case VIDIOC_ENUMINPUT:
	{
/*
		struct v4l2_input *vi = arg;
		if (vi->index < 0 || vi->index >= dev->videc.num_inputs)
			return -EINVAL;
		*vi = dev->source[vi->index].input;
*/
		return 0;
	}

	case VIDIOC_G_INPUT:
	{
/*
		memcpy(arg, &dev->input, sizeof(dev->input));
*/
		return 0;
	}

	case VIDIOC_S_INPUT:
	{
/*
		int	input = *(int *)arg;
		if (input < 0 || input >= dev->videc.num_inputs)
		{
			dprintk("Input out of range %d\n", input);
			return -EINVAL;
		}
		if (input != dev->input)
		{
			dev->input = input;
			set_video_input(dev, input);
		}
*/
		return 0;
	}

	case VIDIOC_G_PARM:
	{
/*
		struct v4l2_streamparm *sp = arg;
		if (sp->type != V4L2_BUF_TYPE_CAPTURE)
			return -EINVAL;
		sp->parm.capture = dev->capture;
*/
		return 0;
	}

	case VIDIOC_S_PARM:
	{
/*
		struct v4l2_streamparm *sp = arg;
		struct v4l2_captureparm *vp = &sp->parm.capture;

		if (vp->capturemode & ~dev->capture.capability)
		{
			dprintk("PARM unsupported capture capability\n");
			return -EINVAL;
		}
		if ((dev->capture.capability & V4L2_CAP_TIMEPERFRAME) &&
		    vp->timeperframe < 10000)
		{
			dprintk("PARM time per frame out of range %ld\n",
				  vp->timeperframe);
			return -EINVAL;
		}
		if (vp->capturemode != dev->capture.capturemode &&
		    !o->noncapturing && dev->streaming)
			return -EINVAL;

		if (o->noncapturing)
			return 0;
		if (vp->capturemode != dev->capture.capturemode)
		{
			dev->capture.capturemode = vp->capturemode;
			capture_new_format(dev);
		}
		if ((vp->capturemode & V4L2_CAP_TIMEPERFRAME) &&
		    vp->timeperframe >= dev->videc.frame_period)
			dev->capture.timeperframe = vp->timeperframe;
		else
			dev->capture.timeperframe = dev->videc.frame_period;
*/
		return 0;
	}

	case VIDIOC_G_STD:
	{
/*
		struct v4l2_standard *std = arg;
		v4l2_video_std_construct(std, dev->videc.standard, 0);
*/
		return 0;
	}

	case VIDIOC_S_STD:
	{
/*
		struct v4l2_standard	*std = arg;
		int			id;
		if ((o->noncapturing && dev->capturing_opens) ||
		    dev->stream_buffers_mapped)
			return -EPERM;
		id = v4l2_video_std_confirm(std);
		if (!((1 << id) & dev->videc.standards))
		{
			dprintk("Bad standard: %u\n", (unsigned)id);
			return -EINVAL;
		}
		dev->videc.set_standard(dev, id);
*/
		return 0;
	}

	case VIDIOC_ENUMSTD:
	{
/*
		struct v4l2_enumstd *estd = arg;
		__u32	b, i;
		if (estd->index < 0 || estd->index > 30)
			return -EINVAL;
		for (b = 1, i = 0; b < 32; ++b)
		{
			if (((1 << b) & dev->videc.standards) == 0)
				continue;
			if (i == estd->index)
			{
				v4l2_video_std_construct(&estd->std, b, 0);
				estd->inputs = (__u32)-1;
				estd->outputs = 0;
				return 0;
			}
			++i;
		}
*/
		return -EINVAL;
	}

	case VIDIOC_QUERYMENU:
	{
/*
		struct v4l2_querymenu	*qm = arg;
		dprintk("query menu\n");

		return vctrl_querymenu(qm);
*/
	}

	case VIDIOC_QUERYCTRL:
	{
/*
		struct v4l2_queryctrl	*qc = arg;
		int			i;

		i = v4l2cc_find_cid(capture_control, qc->id);
		if (i < 0)
			return v4l2cc_query_cid(&dev->host, arg);

		capture_control[i].category = qc->category;
		memcpy(capture_control[i].group, qc->group, 
		       sizeof(qc->group));
		*qc = capture_control[i];
*/
		return 0;
	}

	case VIDIOC_G_CTRL:
	{
/*
		struct v4l2_control	*vc = arg;
		int			i;
		
		dprintk("g ctrl\n");

		i = v4l2cc_find_cid(capture_control, vc->id);
		if (i < 0)  
			return v4l2cc_command(&dev->host, cmd, arg);
		
		vc->value = dev->source[dev->input].control[i];
*/
		return 0;
	}

	case VIDIOC_S_CTRL:
	{
/*
		struct v4l2_control	*vc = arg;
		int			i;
	
		dprintk("s ctrl\n");
		
		i = v4l2cc_find_cid(capture_control, vc->id);
		if (i < 0)  
			return v4l2cc_command(&dev->host, cmd, arg);

		dev->source[dev->input].control[i] = vc->value;
		dprintk("control %d = %d\n", i, vc->value);
		device_tone_controls(dev);
*/
		return 0;
	}

	case VIDIOC_G_TUNER:
	{
/*
		dprintk("gtuner\n");
		if(dev->input!=TVCARD.tuner)
		{
			err_msg("G_TUNER called w/out tuner\n");
			return -EINVAL;
		}
		v4l2_video_std_construct(&dev->host.tuner.std, dev->videc.standard, 0);

		
		dev->host.tuner.rxsubchans=0;
		v4l2cc_command(&dev->host, VIDIOC_G_TUNER, &dev->host.tuner);
		//dev->tuner.capability = dev->tunercaps | V4L2_TUNER_CAP_NORM;
		dprintk("gtuner subs: %02x caps: %02x\n",
			dev->host.tuner.rxsubchans, dev->host.tuner.capability);

		memcpy(arg, &dev->host.tuner, sizeof(struct v4l2_tuner));
		
*/
		return 0;
	}
	
	case VIDIOC_S_TUNER:
	{
/*
		struct v4l2_tuner *tuner = arg;
		dprintk("stuner\n");

		if(dev->input!=TVCARD.tuner)
		{
			err_msg("S_TUNER called w/out tuner\n");
			return -EINVAL;
		}

		memcpy(&dev->host.tuner.std, &tuner->std, sizeof(struct v4l2_standard));

		
		dev->host.tuner.audmode = tuner->audmode;
		v4l2cc_command(&dev->host, VIDIOC_S_TUNER, &dev->host.tuner);

		dprintk("stuner done\n");
*/
		return 0;
	}

#if 0
	case VIDIOC_G_FREQ:
	{
		dprintk("gfreq\n");
		memcpy(arg, &dev->host.freq, sizeof(dev->host.freq));
		return 0;
	}

	case VIDIOC_S_FREQ:
	{
		u32 i = *(u32 *)arg;
		int dummy, ret;

		dprintk("set freq %d\n", i);
		v4l2cc_command(dev, VIDIOC_CC_NOCHANNEL, &dummy);
		ret = v4l2cc_command(dev, cmd, arg);
		v4l2cc_command(dev, VIDIOC_CC_NEWCHANNEL, &dummy);
		if (ret==0)
			dev->freq = i;
		return ret;
	}
#endif

	case VIDIOC_G_FREQ:
        case VIDIOC_S_FREQ:
                //return v4l2cc_ioctl(&dev->host, cmd, arg);
		break;

	case VIDIOC_G_AUDIO:
	{
		dprintk("get audio\n");
		//return v4l2cc_command(&dev->host, cmd, arg);
	}
	case VIDIOC_S_AUDIO:
	{
		dprintk("set audio\n");
		//return v4l2cc_command(&dev->host, cmd, arg);
	}

	default:
		return -ENOIOCTLCMD;
	}
	return 0;
}


static int
v4l2_init_done(struct v4l2_device *v)
{
        return 0;
}

static int
v4l2_open(struct v4l2_device *v, int flags, void **idptr)
{
	struct capture_device *dev = (struct capture_device *)v;
	int	i, n;
	int	cap;

	for (i = 0, n = -1, cap = 0; i < MAX_OPENS; ++i)
	{
		if (!dev->open_data[i].isopen)
			n = i;/* available open_data structure */
		else if (!dev->open_data[i].noncapturing)
			cap = 1;/* another open is already capturing */
	}
	if (n == -1)/* No available open_data structures */
	{
		dprintk("No more opens on this device\n");
		return -EBUSY;
	}

	if (flags & O_NONCAP)/*  Non-capturing open */
		dev->open_data[n].noncapturing = 1;
	else if (cap)
	{
		dprintk("No more capturing opens on this device\n");
		return -EBUSY;
	}
	else
	{
		dev->open_data[n].noncapturing = 0;
		/*  Keep track of whether there is a capturing open  */
		++dev->capturing_opens;
	}

	MOD_INC_USE_COUNT;
	++dev->open_count;
	dev->open_data[n].isopen = 1;
	dev->open_data[n].dev = dev;
	*idptr = &dev->open_data[n];

	if (dev->open_count == 1)
	{
                // first open stuff
	}

	dprintk("Open succeeded\n");
	return 0;
}

static void
v4l2_close(void *id)
{
	struct device_open *o = (struct device_open *)id;
	struct capture_device *dev = o->dev;

	if (!o->noncapturing)
	{
		//pv_off(dev);
		--dev->capturing_opens;
		dprintk("Close\n");
	}
	o->isopen = 0;
        --dev->open_count;
	if (dev->open_count == 0)
	{
                // last close stuff
	}
	MOD_DEC_USE_COUNT;
}

static long
v4l2_read(void		*id,
	    char		*buf,
	    unsigned long	count,
	    int			noblock)
{
	struct device_open *o = (struct device_open *)id;
	struct capture_device *dev = o->dev;

        return av_record(dev, buf, count, noblock);
}

static long
v4l2_write(void		*id, 
	     const char		*buf, 
	     unsigned long	count, 
	     int		noblock)
{
	struct device_open *o = (struct device_open *)id;
	struct capture_device *dev = o->dev;

        return av_play(dev, buf, count, noblock, 1);
}

static int
v4l2_mmap(void			*id,
	  struct vm_area_struct *vma)
{
        return -1;
}

static int
v4l2_poll(void	*id,
	    struct file	*file,
	    poll_table	*table)
{
        return -1;
}

int vid_register(struct dvb_struct *dev)
{
	sprintf(dev->v.name, "DVB capture driver (%d)", dev->num);

	dev->v.type = V4L2_TYPE_CAPTURE;
	dev->v.minor = dev->num;

	dev->v.open = v4l2_open;
	dev->v.close = v4l2_close;
	dev->v.read = v4l2_read;
	dev->v.write = v4l2_write;
	dev->v.ioctl = v4l2_ioctl;
	dev->v.mmap = v4l2_mmap;
	dev->v.poll = v4l2_poll;
	dev->v.initialize = v4l2_init_done;
	dev->v.priv = NULL;

        if (v4l2_register_device((struct v4l2_device *)dev) != 0)
                return -1;
        return 0;
}

int vid_unregister(struct dvb_struct *dvb)
{
        v4l2_unregister_device((struct v4l2_device *)dvb);
        return 0;
}


#else

/****************************************************************************
 * V4L SECTION
 ****************************************************************************/

static long dvb_read(struct video_device *dev, char *buf,
                     unsigned long count, int nonblock)
{
	struct dvb_struct* dvb = (struct dvb_struct*)dev;
        
        return av_record(dvb, buf, count, nonblock);
}


static long dvb_write(struct video_device *dev, const char *buf,
                      unsigned long count, int nonblock)
{
	struct dvb_struct* dvb = (struct dvb_struct*)dev;

        AV_StartPlay(dvb, RP_AV);
        return dvb_play(dvb, buf, count, nonblock);
}


static struct video_channel dvb_channels[1] = {
        { 0, "DVB", 1, 3, 1, 1},
};


static int dvb_ioctl(struct video_device *dev, unsigned int cmd, void *arg)
{
	struct dvb_struct* dvb = (struct dvb_struct*)dev;
        
	switch (cmd)
	{
		/* Get capabilities */
        case VIDIOCGCAP:
        {
                struct video_capability b;
                
                dprintk(KERN_ERR "dvb: VIDIOCGCAP called\n");

                strcpy(b.name, &dev->name[0]);
			
                b.type 		= dvb->video.type;
						
                b.channels	= 1;	
                b.audios	= 2;
                b.maxwidth	= 768;
                b.maxheight	= 576;
                b.minwidth	= 32;
                b.minheight	= 32;
                
                if(copy_to_user(arg,&b,sizeof(b)))
                        return -EFAULT;
                return 0;
        }
		/* Get channel info (sources) */
        case VIDIOCGCHAN:
        {
                struct video_channel v;
                
                dprintk(KERN_ERR "dvb: VIDIOCGCHAN called\n");
                
                if(copy_from_user(&v, arg,sizeof(v)))
                        return -EFAULT;
                
                /* copy the desired structure to v */
                memcpy(&v, &dvb_channels[v.channel], sizeof(v));
                
                if(copy_to_user(arg,&v,sizeof(v)))
                        return -EFAULT;
                
                return 0;
                
        }
		/* Set channel 	*/
        case VIDIOCSCHAN:
        {
                struct video_channel v;
		if(copy_from_user(&v, arg,sizeof(v)))
			return -EFAULT;
		
                
		if (v.channel>0)
			return -EINVAL;
		if (v.norm > 1)
			return -EOPNOTSUPP;

                dvb->vidmode=v.norm;
                SetMode(dvb, v.norm);

		return 0;
        }
                
		/* Get tuner abilities */
        case VIDIOCGTUNER:
        {
                struct video_tuner v;
                
                dprintk(KERN_ERR "dvb: VIDIOCGTUNER called\n");
                
                if(copy_from_user(&v,arg,sizeof(v)))
                        return -EFAULT;
                
                /* only channel 0 has a tuner */	
                if(v.tuner != 0 )
                        return -EINVAL;
                
                /* fill the structure */
                strcpy(v.name, "DVB");
                v.rangelow  = 0x00000000;
                v.rangehigh = 0xffffffff;
                
                /* fixme: currently we only support PAL */
                v.flags= VIDEO_TUNER_PAL | VIDEO_TUNER_NTSC;// | VIDEO_TUNER_SECAM; 
                v.mode = dvb->vidmode;
		
                /* fixme: fill in signal strength here */
                v.signal = 0xffff;
                
                if(copy_to_user(arg,&v,sizeof(v)))
                        return -EFAULT;
                
                return 0;
        }
		/* Tune the tuner for the current channel */
        case VIDIOCSTUNER:
        {
                struct video_tuner v;
                
                dprintk(KERN_ERR "dvb: VIDIOCSTUNER called\n");
                
                if(copy_from_user(&v, arg, sizeof(v)))
                        return -EFAULT;
                
                /* only channel 0 has a tuner */
                if( v.tuner != 0)
                        return -EINVAL;
                
                /* check if format is supported */
                if(    v.mode != VIDEO_MODE_PAL 
                       && v.mode != VIDEO_MODE_NTSC
                    /* && v.mode != VIDEO_MODE_SECAM */ )
                       return -EOPNOTSUPP;
                
                dvb->vidmode=v.mode;
                SetMode(dvb, v.mode);
                
                return 0;
        }
		/* Get picture properties */
        case VIDIOCGPICT:
        {
                struct video_picture p;
                
                saacomm(SAA7146_V4L_GPICT, &p);
                
                if(copy_to_user(arg, &p, sizeof(struct video_picture)))
                        return -EFAULT;
                
                dprintk(KERN_ERR "dvb: VIDIOCGPICT called: b:%d c:%d s:%d d:%d p:%d\n",p.brightness, p.contrast, p.colour, p.depth, p.palette);
                
                return 0;
        }
		/* Set picture properties */
        case VIDIOCSPICT:
        {
                struct video_picture p;
                
                if(copy_from_user(&p, arg,sizeof(struct video_picture)))
                        return -EFAULT;
                
                dprintk(KERN_ERR "dvb: VIDIOCGPICT called: b:%d c:%d s:%d d:%d p:%d\n",p.brightness, p.contrast, p.colour, p.depth, p.palette);
                
                saacomm(SAA7146_V4L_SPICT, &p);
                
                return 0;
                
        }
		/* Set the video overlay window */
        case VIDIOCSWIN:
        {
                struct video_window w;
                
                if(copy_from_user(&w,arg,sizeof(struct video_window)))
                        return -EFAULT;
                
                dprintk(KERN_ERR "dvb: VIDIOCSWIN called: clips: %d, x:%d, y:%d, h:%d, w:%d\n", w.clipcount,w.x, w.y, w.height,w.width);
                
                saacomm(SAA7146_V4L_SWIN, &w);
                
                return 0;
        }
		/* Set the video overlay window - passes clip list for hardware smarts , chromakey etc */
        case VIDIOCGWIN:
        {
                dprintk(KERN_ERR "dvb: VIDIOCGWIN called\n");
                return 0;
        }
		/* Start, end capture */
        case VIDIOCCAPTURE:
        {
                int v;
                
                if(copy_from_user(&v, arg,sizeof(v)))
                        return -EFAULT;
                
                dprintk(KERN_ERR "dvb: VIDIOCCAPTURE called, mode:%d (0=disable)\n",v);
                
                saacomm(SAA7146_V4L_CCAPTURE, &v);
                
                return 0;
        }
		/* Get frame buffer */
        case VIDIOCGFBUF:
        {
                struct video_buffer b;
                
                dprintk(KERN_ERR "dvb: VIDIOCGFBUF called\n");
                
                saacomm(SAA7146_V4L_GFBUF, &b);
                
                if(copy_to_user(arg, &b,sizeof(struct video_buffer)))
                        return -EFAULT;
                
                return 0;
                
        }
		/* Set frame buffer - root only */
        case VIDIOCSFBUF:
        {
                struct video_buffer b;
                u32 vid = (vidmem << 16) | vidlow;
                
                if(copy_from_user(&b, arg,sizeof(struct video_buffer)))
                        return -EFAULT;
                
                /* see if vidmem-override is requested */
                if( 0 != vidmem ) {
                        printk(KERN_ERR "dvb: video-memory-override. (0x%08x)\n",vid);
                        b.base = (void*)vid;
                }
                
                saacomm(SAA7146_V4L_SFBUF, &b);
                
                dprintk(KERN_ERR "dvb: VIDIOCSFBUF called\n");
                
                return 0;		
        }
		/* Video key event - to dev 255 is to all - cuts capture on all DMA windows with this key (0xFFFFFFFF == all) */
        case VIDIOCKEY:
        {
                dprintk(KERN_ERR "dvb: VIDIOCKEY called\n");
                return 0;
        }
		/* Get tuner */
        case VIDIOCGFREQ:
        {
                unsigned long v = dvb->front.freq;
                
                dprintk(KERN_ERR "dvb: VIDIOCGFREQ called\n");
                
                if(copy_to_user(arg,&v,sizeof(v)))
                        return -EFAULT;
        }
		/* Set tuner */
        case VIDIOCSFREQ:
        {
                int v;
                
                if(copy_from_user(&v, arg, sizeof(v)))
                        return -EFAULT;
                
                //dprintk(KERN_ERR "dvb: VIDIOCSFREQ called with freq %u; audio:%d\n", v, dvb->current_audio_state);
                
                /* mute audio to avoid strange tuning noises */	
                
                //mdelay( TUNER_NOISE_DELAY );
                
                /* tune in desired frequency */			
                //tuner_command(dvb, TUNER_SET_TVFREQ,&v); 
                //mdelay( TUNER_NOISE_DELAY );
                
                /* restore audio if it should not be muted */
                
                dvb->front.freq = v;
                return 0;
        }
		/* Get audio info */	
        case VIDIOCGAUDIO:
        {
                struct video_audio v;
                
                v.flags = VIDEO_AUDIO_MUTABLE;
                /* let's auto-detect */
                return 0;
        }
		/* Audio source, mute etc */
        case VIDIOCSAUDIO:
        {
          //struct video_audio v;
                
                
                return 0;
        }
                /* Sync with mmap grabbing */
        case VIDIOCSYNC:
        {
                int i = 0, frame = 0;
                
                if(copy_from_user((void *)&frame,arg,sizeof(int)))
                        return -EFAULT;
                
                dprintk(KERN_ERR "dvb: VIDIOCSYNC called - frame: %d\n",frame);
                
                /* simply pass the requested frame-number to the corresponding
                   saa7146-function ... */
                i = saacomm(SAA7146_V4L_CSYNC, &frame);
                
                return i;
        }
		/* Grab frames */
        case VIDIOCMCAPTURE:
        {
                struct video_mmap vm;
                int i = 0;
                
                if(copy_from_user((void *)&vm, (void *) arg, sizeof(vm)))
                        return -EFAULT;
                
                dprintk(KERN_ERR "dvb: VIDIOCMCAPTURE called: fr:%d, fmt:%d, w:%d, h:%d\n",vm.frame, vm.format,vm.width, vm.height);
                
                /* simply pass the structure for the requested frame-number
                   to the corresponding saa7146-function ... */
                i = saacomm(SAA7146_V4L_CMCAPTURE, &vm);
                
                return i;
        }
                
		/* Memory map buffer info */
        case VIDIOCGMBUF:
        {
                struct video_mbuf mbuf;
                
                dprintk(KERN_ERR "dvb: VIDIOCGMBUF called\n");
                
                saacomm(SAA7146_V4L_GMBUF, &mbuf);
                
                if (copy_to_user((void *)arg, &mbuf, sizeof(struct video_mbuf)))
                        return -EFAULT;
                
                return 0;
                
        }
		/* Get attached units */
        case VIDIOCGUNIT:
        {
                struct video_unit vu;
                dprintk(KERN_ERR "dvb: VIDIOCGUNIT called\n");
                
                if(copy_to_user((void *)arg, (void *)&vu, sizeof(vu)))
                        return -EFAULT;
                return 0;
        }
		
		/* Get frame buffer */
        case VIDIOCGCAPTURE:
        {
                dprintk(KERN_ERR "dvb: VIDIOCGCAPTURE called\n");
                return 0;
        }
		/* Set frame buffer - root only */
        case VIDIOCSCAPTURE:
        {
                dprintk(KERN_ERR "dvb: VIDIOCSCAPTURE called\n");
                return 0;
        }


        /* DVB ioctls */
                
        case VIDIOCSBITFILTER:
        {
                struct bitfilter filter;

                if(copy_from_user(&filter, arg, sizeof(struct bitfilter)))
                        return -EFAULT;

                if (SetBitFilter(dvb, &filter)<0)
                        return -EFAULT;

                if(copy_to_user(arg, &filter, sizeof(struct bitfilter)))
                        return -EFAULT;
                return 0;
        }
        
        case VIDIOCSSHUTDOWNFILTER:
        {
                u16 handle;

                if(copy_from_user(&handle, arg, sizeof(u16)))
                        return -EFAULT;

                if (ShutDownFilter(dvb, handle)<0) 
                        return -EFAULT;
                return 0;
        }
        
        case VIDIOCSDISEQCMSG:
        {
                struct diseqc ds;

                if(copy_from_user(&ds, arg, sizeof(struct diseqc)))
                        return -EFAULT;

                return SendDiSEqCMsg(dvb, ds.msglen, ds.msg, ds.burst);
        }
        
        case VIDIOCSCIMSG:
        {
                struct cimsg cm;

                if(copy_from_user(&cm, arg, sizeof(struct cimsg)))
                        return -EFAULT;

                return SendCICommand(dvb, cm.command, cm.message, cm.length);
        }
        
        case VIDIOCSOSDCOMMAND:
        {
                struct drawcmd dc;

                if(copy_from_user(&dc, arg, sizeof(struct drawcmd)))
                        return -EFAULT;

                return OSD_DrawCommand(dvb, &dc);
        }
        
        case VIDIOCGFRONTEND:
        {
                GetFront(dvb);
                if(copy_to_user(arg, &dvb->front, sizeof(struct frontend)))
                        return -EFAULT;
                
                return 0;
        }

        case VIDIOCGVIDEOSTATE:
        {
                VIDEOSTATE vstate;

                GetDecoderStatus(dvb, &vstate);
                if(copy_to_user(arg, &vstate, sizeof(VIDEOSTATE)))
                        return -EFAULT;
                
                return 0;
        }

        case VIDIOCSFRONTEND:
        {
                struct frontend front;

                if(copy_from_user(&front, arg, sizeof(struct frontend)))
                        return -EFAULT;
                
                return SetFront(dvb, &front);
        }

        case VIDIOCSASPECTRATIO:
        {
                int iarg=(int) arg;

                return outcom(dvb, COMTYPE_ENCODER, SetMonitorType, 1, (u16) iarg);
        }

        case VIDIOCSPANSCAN:
        {
                int iarg=(int) arg;
                                
                return outcom(dvb, COMTYPE_ENCODER, SetPanScanType, 1, (u16) iarg);
        }

        case VIDIOCSFREEZEMODE:
        {
                int iarg=(int) arg;
                                
                return outcom(dvb, COMTYPE_ENCODER, SetFreezeMode, 1, (u16) iarg);
        }

	case VIDIOCSPLAYMODE:
        {
                struct video_play_mode pmode;
                if (copy_from_user((void *) &pmode, arg,
                                   sizeof(struct video_play_mode)))
                        return -EFAULT;
                switch (pmode.mode) {
                case VID_PLAY_VID_OUT_MODE:
                        if (pmode.p1 != VIDEO_MODE_NTSC &&
                            pmode.p1 != VIDEO_MODE_PAL)
                          return -EINVAL;
                        SetMode(dvb, pmode.p1);
                        return 0;
                case VID_PLAY_GENLOCK:
                        return -EOPNOTSUPP;
                case VID_PLAY_NORMAL:
                        dvb->playmode = pmode.mode;
                        outcom(dvb, COMTYPE_REC_PLAY, __Play, 2, AV_PES, 0);
                        return 0;
                case VID_PLAY_PAUSE:
                case VID_PLAY_SINGLE_FRAME:
                        dvb->playmode = pmode.mode;
                        outcom(dvb, COMTYPE_REC_PLAY, __Pause, 2, AV_PES, 0);
                        return 0;
                case VID_PLAY_FAST_FORWARD:
                        dvb->playmode = pmode.mode;
                        outcom(dvb, COMTYPE_REC_PLAY, __FF_IP, 2, AV_PES, 0);
                        return 0;
                case VID_PLAY_SLOW_MOTION:
                        dvb->playmode = pmode.mode;
                        outcom(dvb, COMTYPE_REC_PLAY, __Slow, 2, AV_PES, 0);
                        return 0;
                case VID_PLAY_IMMEDIATE_NORMAL:
                        dvb->playmode = VID_PLAY_NORMAL;
                        return 0;
                case VID_PLAY_SWITCH_CHANNELS:
                        dvb->playmode = VID_PLAY_NORMAL;
                        return 0;
                case VID_PLAY_FREEZE_FRAME:
                        dvb->playmode = pmode.mode;
                        return 0;
                case VID_PLAY_STILL_MODE:
                        dvb->playmode = pmode.mode;
                        return 0;
                case VID_PLAY_MASTER_MODE:
                        
                        return 0;
                case VID_PLAY_ACTIVE_SCANLINES:
                        return 0;
                case VID_PLAY_RESET:
                        AV_Stop(dvb,RP_AV);
                        init_p2a(&dvb->transi.pesto[0]);
                        init_p2a(&dvb->transi.pesto[1]);
                        init_a2p(&dvb->ps_ts_stream,av_pes_cb,(void *)dvb,
                                 (void *)dvb, outstream, 0xa0, 0x50);
                        dvb->transi.which_pesto = 0;
                        init_trans(&dvb->transi,dvb->p2a_filter, 
                                   kpes_to_av_pes, (void *)dvb);
                        return 0;
                case VID_PLAY_END_MARK:
                        return 0;
                case VID_PLAY_CLEAR_BUFFER:
                        //AV_Stop(dvb,RP_AV);
                        spin_lock_irq(&dvb->avout.lock);
                        dvb->avout.tail = dvb->avout.head;
                        spin_unlock_irq(&dvb->avout.lock);

                        init_trans(&dvb->transi, dvb->p2a_filter, 
                                   kpes_to_av_pes, (void *) dvb);
                        init_p2a(&dvb->transi.pesto[0]);
                        init_p2a(&dvb->transi.pesto[1]);
                        dvb->transi.which_pesto = 0;
                        if (dvb->playing==RP_AV)
                                outcom(dvb, COMTYPE_REC_PLAY, 
                                       __Play, 2, AV_PES, 0);
                        return 0;
                 }
        }
	case VIDIOCSWRITEMODE:
        {
                int mode;
                if (copy_from_user((void *) &mode, arg, sizeof(int)))
                        return -EFAULT;
                
                switch (mode) {
                case VID_WRITE_MPEG_AUD:
                case VID_WRITE_MPEG_VID:
                case VID_WRITE_CC:
                case VID_WRITE_TTX:
                case VID_WRITE_OSD:
                        //dvb->writemode = mode;
                        return 0;
                default:
                        return -EINVAL;
                }
        }
        case 0xf0:
        {
                u32 val;
                static u32 
                  dumpmem(struct dvb_struct *dvb, u32 adr)
                  {
                    u16 Buffer[2];
                    
                    u16 adrbuf[] = { ((COMTYPE_REQUEST << 8) + ReqRegister), 2, 
                                     (adr>>16), adr&0xffff};
                    CommandRequest(dvb, adrbuf, 4, Buffer, 2);
                    
                    return (Buffer[0] << 16) + Buffer[1];
                  }

                if (copy_from_user((void *) &val, arg, sizeof(u32)))
                        return -EFAULT;
                val=dumpmem(dvb, val);
                if(copy_to_user(arg, (void *)&val, sizeof(u32)))
                        return -EFAULT;
                return 0;
        }
        case 0xf1:
        {
                u16 kk[5];
                u8  *ll=(u8 *)kk;

                if (copy_from_user((void *)kk, arg, 10))
                        return -EFAULT;
                return outcom(dvb, ll[0], ll[1], 4, 
                              kk[1], kk[2], kk[3], kk[4]);
        }
        default:
                return -ENOIOCTLCMD;
	}
        return 0;
}


static int dvb_init_done(struct video_device *dev)
{
	fprintk(KERN_ERR "dvb: dvb_init_done called\n");
	return 0;
}


static int dvb_mmap(struct video_device *dev, const char *adr, unsigned long size)
{
	struct dvb_struct* dvb = (struct dvb_struct*)dev;
	struct saa7146_mmap_struct m;
	
	fprintk(KERN_ERR "dvb: dvb_mmap called, adr:%p, size:0x%08x\n",adr,(u32)size);
        
	m.adr = adr;
	m.size = size;
        
	if ( 0 != saacomm(SAA7146_DO_MMAP, &m) ) {
		printk(KERN_ERR "dvb: dvb_mmap failed!\n");
		return -1;
	}
        
	return 0;
}

static int dvb_open(struct video_device *dev, int flags)
{
	struct dvb_struct* dvb = (struct dvb_struct*)dev;

        if (dvb->cardtype==DVB_CARD_TT_SIEMENS) {
                ARM_ResetMailBox(dvb); //FIXME: why is this needed with CI installed????
                if (!dvb->recording){
                        dvb->avin.head = dvb->avin.tail;
                }        
                if (!dvb->playing){
                        dvb->avout.tail = dvb->avout.head;
                        dvb->aout.tail = dvb->aout.head;
                        dvb->vout.tail = dvb->vout.head;
                }
        } else {
                dvb->avin.tail = dvb->avin.head;
        }
	MOD_INC_USE_COUNT;
	return 0;   
}

static void dvb_close(struct video_device *dev)
{
        struct dvb_struct* dvb = (struct dvb_struct*)dev;
	
	fprintk(KERN_ERR "dvb: dvb_close called\n");

        if (dvb->cardtype==DVB_CARD_TT_SIEMENS) {
                AV_Stop(dvb,RP_AV);
                //ShutDownFilters(dvb);
                DestroyOSDWindows(dvb);
        } else {
                dvb->recording=0;
                TTBStop(dvb);
        }
        MOD_DEC_USE_COUNT;  
}

static unsigned int dvb_poll(struct video_device *dev, struct file *file,
	poll_table *wait)
{
        struct dvb_struct *dvb=(struct dvb_struct *) dev;
        unsigned int mask=0;

        if (dvb->recording) {
                if (dvb->avin.head != dvb->avin.tail)
                        return (POLLIN | POLLRDNORM);
          
                poll_wait(file, &dvb->avin.queue, wait);
          
                if (dvb->avin.head != dvb->avin.tail)
                        mask |= (POLLIN | POLLRDNORM);
        }
        
        if (dvb->playing) {
                if (dvb_buf_free(&dvb->avout)>8192)
                        return (POLLOUT | POLLWRNORM);

                poll_wait(file, &dvb->avout.queue, wait);
          
                if (dvb_buf_free(&dvb->avout)>8192)
                        mask |= (POLLOUT | POLLWRNORM);
        }
        
        return mask;
}


/* template for video_device-structure */
static struct video_device dvb_template = {
	"Digital Video Broadcasting Board",
	VID_TYPE_TUNER		|
        VID_TYPE_CAPTURE	|
        VID_TYPE_TELETEXT	|
        VID_TYPE_OVERLAY	|
        VID_TYPE_CLIPPING	|
        VID_TYPE_FRAMERAM	|
        VID_TYPE_SCALES         |
        VID_TYPE_MPEG_DECODER   |
        VID_TYPE_MPEG_ENCODER   |  /* Well not really, but it can output the received MPEG stream */
        VID_TYPE_DVB,
        VID_HARDWARE_SAA7146,
        dvb_open,
        dvb_close,
        dvb_read,
	dvb_write,		
	dvb_poll,
	dvb_ioctl,
	dvb_mmap,
	dvb_init_done,
	NULL,
	0,
	0
};


static long vbi_read(struct video_device *dev, char *buf, unsigned long count,
		     int nonblock)
{
	struct dvb_struct *dvb=(struct dvb_struct *) (dev-1);

        return section_read(dvb, buf, count, nonblock);
}

static unsigned int vbi_poll(struct video_device *dev, struct file *file,
	poll_table *wait)
{
        struct dvb_struct *dvb=(struct dvb_struct *) (dev-1);
        
        return section_poll(dvb, file, wait);
}

static int vbi_open(struct video_device *dev, int flags)
{
        //struct dvb_struct *dvb=(struct dvb_struct *) (dev-1);

	MOD_INC_USE_COUNT;
	return 0;   
}

static void vbi_close(struct video_device *dev)
{
        struct dvb_struct *dvb=(struct dvb_struct *) (dev-1);
 
        ShutDownFilters(dvb);
        section_reset(dvb);
	MOD_DEC_USE_COUNT;  
}

static int vbi_ioctl(struct video_device *dev, unsigned int cmd, void *arg)
{
        struct dvb_struct *dvb=(struct dvb_struct *) (dev-1);
        
	switch (cmd)
	{
        case VIDIOCSBITFILTER:
        {
                struct bitfilter filter;

                if(copy_from_user(&filter, arg, sizeof(struct bitfilter)))
                        return -EFAULT;

                if (SetBitFilter(dvb, &filter)<0)
                        return -EFAULT;

                if(copy_to_user(arg, &filter, sizeof(struct bitfilter)))
                        return -EFAULT;
                return 0;
        }
        
        case VIDIOCSSHUTDOWNFILTER:
        {
                u16 handle;

                if(copy_from_user(&handle, arg, sizeof(u16)))
                        return -EFAULT;

                if (ShutDownFilter(dvb, handle)<0) 
                        return -EFAULT;

                return 0;
        }

        case 0xf1:
        {
                u16 kk[5];
                u8  *ll=(u8 *)kk;

                if (copy_from_user((void *)kk, arg, 10))
                        return -EFAULT;
                outcom(dvb, ll[0], ll[1], 4, 
                       kk[1], kk[2], kk[3], kk[4]);
        }

        default:
                return -ENOIOCTLCMD;
        }
}

static struct video_device vbi_template=
{
	"dvb vbi",
	VID_TYPE_CAPTURE|VID_TYPE_TELETEXT,
        VID_HARDWARE_SAA7146,
	vbi_open,
	vbi_close,
	vbi_read,
	NULL,
	vbi_poll,
	vbi_ioctl,
	NULL,	/* no mmap yet */
	dvb_init_done,
	NULL,
	0,
	-1
};


static int vid_register(struct dvb_struct *dvb)
{
        /* Copy structure */
        memcpy( &dvb->video, &dvb_template, sizeof(struct video_device));
        
        if( 0 > (video_register_device(&dvb->video, VFL_TYPE_GRABBER))) {
                printk(KERN_ERR "dvb: can't register videodevice\n");
                return -1;
        }

        memcpy( &dvb->vbi, &vbi_template, sizeof(struct video_device));
	if(video_register_device(&dvb->vbi, VFL_TYPE_VBI)<0) 
        {
	        video_unregister_device(&dvb->video);
		return -1;
	}
        return 0;
}

static inline int vid_unregister(struct dvb_struct *dvb)
{
        if(dvb->video.minor!=-1)
                video_unregister_device(&dvb->video);
        if(dvb->vbi.minor!=-1)
                video_unregister_device(&dvb->vbi);
        return 0;
}

/****************************************************************************
 * END OF V4L SECTION
 ****************************************************************************/
#endif


/****************************************************************************
 * Nokia API SECTION
 ****************************************************************************/


/******************************************************************************
 * hardware filter functions
 ******************************************************************************/

static int 
StartHWFilter(dvb_demux_filter_t *dvbdmxfilter)
{
        dvb_demux_feed_t *dvbdmxfeed=dvbdmxfilter->feed;
        struct dvb_struct *dvb=
                (struct dvb_struct *) dvbdmxfeed->demux->priv;

        u16 buf[20];
        int ret, i;
        u16 handle;
        u16 mode=0x5020;
        
        if (dvbdmxfilter->type==DMX_TYPE_SEC) {
                for (i=0; i<16; i++)
                        buf[i+4]=(dvbdmxfilter->filter.filter_value[i]<<8)|
                                  dvbdmxfilter->filter.filter_mask[i];
                mode=4;
        } else
        if ((dvbdmxfeed->ts_type & TS_PACKET) &&
            !(dvbdmxfeed->ts_type & TS_PAYLOAD_ONLY)) 
                init_p2t(&dvb->p2t_filter[dvbdmxfilter->index], dvbdmxfeed);

        buf[0] = (COMTYPE_PID_FILTER << 8) + AddPIDFilter; 
        buf[1] = 16;	
        buf[2] = dvbdmxfeed->pid;
        buf[3] = mode;

        ret=CommandRequest(dvb, buf, 20, &handle, 1);          
        if (ret<0)
                return ret;

        dvb->handle2filter[handle]=dvbdmxfilter;
        dvbdmxfilter->hw_handle=handle;

        return ret;
}

static int 
StopHWFilter(dvb_demux_filter_t *dvbdmxfilter)
{
        struct dvb_struct *dvb=
          (struct dvb_struct *) dvbdmxfilter->feed->demux->priv;

        u16 buf[3];
        u16 answ[2];
        int ret;
        u16 handle=dvbdmxfilter->hw_handle;
        
        dvb->handle2filter[handle]=NULL;

        buf[0] = (COMTYPE_PID_FILTER << 8) + DelPIDFilter; 
        buf[1] = 1;	
        buf[2] = handle;
        ret=CommandRequest(dvb, buf, 3, answ, 2);          

        if (answ[1] != handle) {
                printk("dvb: filter shutdown error :%d\n", answ[1]);
                ret=-1;
        }
        return ret;
}

static int 
dvb_write_to_decoder(dvb_demux_feed_t *dvbdmxfeed, u8 *buf, size_t count)
{
        dvb_demux_t *dvbdmx=dvbdmxfeed->demux;
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdmx->priv;
        trans_t *p=&dvb->transi;
	p2a_t *pesto;

        if (dvbdmxfeed->pes_type>1)
                return -1;
        if (!(buf[3]&0x10)) // no payload?
                return -1;

        pesto=&p->pesto[dvbdmxfeed->pes_type];
        p->which_pesto=dvbdmxfeed->pes_type;

        dprintk ("function : %s %02x %02x %02x %02x %02x\n", __FUNCTION__, 
                buf[0], buf[1], buf[2], buf[3], buf[4]);

        if (buf[1]&0x40) 
                if (pesto->plength==MAX_PLENGTH)
                        pesto->found=0;

        if (buf[3]&0x20) {  // adaptation field?
                count-=buf[4]+1;
                buf+=buf[4]+1;
        }
        
        kpes_to_av_pes(buf+4, count-4, (void *)p);
        return 0;
}

static int 
dvb_start_feed(dvb_demux_feed_t *dvbdmxfeed)
{
        dvb_demux_t *dvbdmx=dvbdmxfeed->demux;
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdmx->priv;

        if (!dvbdmx->dmx.frontend)
                return -EINVAL;

        if (dvb->cardtype==DVB_CARD_TT_BUDGET)
                return TTBStart(dvb); 

        if (dvbdmxfeed->type == DMX_TYPE_TS) {
	        if ((dvbdmxfeed->ts_type & TS_DECODER) 
		    && (dvbdmxfeed->pes_type<=1)) {
		        switch (dvbdmx->dmx.frontend->source) {
			case DMX_MEMORY_FE: 
			        if (dvbdmxfeed->ts_type & TS_DECODER)
				       if (dvbdmx->pids[0]!=0xffff &&
					    dvbdmx->pids[1]!=0xffff) {
                                               dvb->transi.pesto[0].nonblock=0;
                                               dvb->avout.tail = dvb->avout.head;
                                               AV_StartPlay(dvb,RP_AV);
                                               dvbdmx->playing=1;
					}
				break;
			default:
			        SetPIDArray(dvb, dvbdmx->pids);
				if (dvbdmxfeed->ts_type & TS_PACKET) {
				  if (dvbdmx->pids[0]!=0xffff && 
				      dvbdmx->pids[1]!=0xffff) {
                                          AV_StartRecordNAPI(dvb,RP_AV, dvbdmxfeed);
				  }
				}
				break;
			}
		} else
		        if ((dvbdmxfeed->ts_type & TS_PACKET) &&
                            (dvbdmx->dmx.frontend->source!=DMX_MEMORY_FE))
                                StartHWFilter(dvbdmxfeed->filter); 
        }

        
        if (dvbdmxfeed->type == DMX_TYPE_SEC) {
                int i;

	        for (i=0; i<DVB_DEMUX_FILTER_MAX; i++) {
		        if (dvbdmx->filter[i].state!=DMX_STATE_READY)
			        continue;
			if (dvbdmx->filter[i].type!=DMX_TYPE_SEC)
			        continue;
			if (dvbdmx->filter[i].filter.parent!=&dvbdmxfeed->feed.sec)
			        continue;

			dvbdmxfeed->feed.sec.is_filtering=1;
			dvbdmx->filter[i].state=DMX_STATE_GO;
                        if (dvbdmx->dmx.frontend->source!=DMX_MEMORY_FE)
                                StartHWFilter(&dvbdmx->filter[i]); 
                }
	}
        return 0;
}


static int 
dvb_stop_feed(dvb_demux_feed_t *dvbdmxfeed)
{
        dvb_demux_t *dvbdmx=dvbdmxfeed->demux;
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdmx->priv;

        if (dvb->cardtype==DVB_CARD_TT_BUDGET)
                return TTBStop(dvb); 

        if (dvbdmxfeed->type == DMX_TYPE_TS) {
        if ((dvbdmxfeed->ts_type & TS_DECODER) 
            && (dvbdmxfeed->pes_type<=1)) {
                if (dvbdmx->recording) {
                        AV_Stop(dvb,RP_AV);
                        dvbdmx->recording=0;
                }
                if (dvbdmx->playing) {
                        AV_Stop(dvb,RP_AV);
                        dvbdmx->playing=0;
                }
                switch (dvbdmx->dmx.frontend->source) {
                case DMX_MEMORY_FE: 
                        if (dvbdmx->playing) {
                                AV_Stop(dvb,RP_AV);
                                dvbdmx->playing=0;
                        }
                        break;
                default:
                        if (!dvbdmx->pesfilter[dvbdmxfeed->pes_type])
                                return -EINVAL;
                        dvbdmx->pids[dvbdmxfeed->pes_type]=0xffff;
                        dvbdmx->pesfilter[dvbdmxfeed->pes_type]=0;
                        if (dvbdmx->recording) {
                                dvbdmx->recording=0;
                                AV_Stop(dvb,RP_AV);
                        }
#if 0 // should we switch off decoding when closing the device?
                        SetPIDArray(dvb, dvbdmx->pids);
#endif
                }
        } else
          if ((dvbdmxfeed->ts_type & TS_PACKET) &&
              (dvbdmx->dmx.frontend->source!=DMX_MEMORY_FE))
            return StopHWFilter(dvbdmxfeed->filter); 
	}



        if (dvbdmxfeed->type == DMX_TYPE_SEC) {
                int i;

	        for (i=0; i<DVB_DEMUX_FILTER_MAX; i++)
		        if (dvbdmx->filter[i].state==DMX_STATE_GO && 
			    dvbdmx->filter[i].filter.parent==&dvbdmxfeed->feed.sec) {
			        dvbdmx->filter[i].state=DMX_STATE_READY;
                                if (dvbdmx->dmx.frontend->source!=DMX_MEMORY_FE)
                                        StopHWFilter(&dvbdmx->filter[i]); 
                }

	}

        return 0;
}

/******************************************************************************
 * SEC device file operations
 ******************************************************************************/

#define QPSK_MAX_EVENT 8

static void add_qpsk_event(qpsk_t *qpsk, struct qpskEvent *ev)
{
        int wp;
        struct timeval tv;

        //printk("add_qpsk_event\n");
        get_fast_time(&tv);
        ev->timestamp=tv.tv_sec;
        
        spin_lock(&qpsk->eventlock);
        wp=qpsk->eventw;
        wp=(wp+1)%QPSK_MAX_EVENT;
        if (wp==qpsk->eventr) {
                qpsk->overflow=1;
                qpsk->eventr=(qpsk->eventr+1)%QPSK_MAX_EVENT;
        }
        memcpy(&qpsk->events[qpsk->eventw], ev, sizeof(struct qpskEvent));
        qpsk->eventw=wp;
        spin_unlock(&qpsk->eventlock);
        wake_up(&qpsk->eventq);
}



/* The following are similar to the SECCDI but not the same
   since SECCDI would not make much sense (e.g. no driver data pointer) 
*/

int secSetTone(struct dvb_struct *dvb, secToneMode mode)
{
        int val;

        switch (mode) {
        case SEC_TONE_ON:
                val=1;
                break;
        case SEC_TONE_OFF:
                val=0;
                break;
        default:
          return -EINVAL;
        }
        dvb->front.ttk=val;
        Set22K(dvb, val);
        return 0;
}

int secSetVoltage(struct dvb_struct *dvb, secVoltage voltage)
{
        int power=1, volt=0;
                       
        switch (voltage) {
        case SEC_VOLTAGE_LT: //FIXME: Test which one is true
        case SEC_VOLTAGE_OFF:
                power=0;
                break;
        case SEC_VOLTAGE_13:
                volt=0;
                break;
        case SEC_VOLTAGE_18:
                volt=1;
                break;
        case SEC_VOLTAGE_13_5:
        case SEC_VOLTAGE_18_5:
        default:
                return -EINVAL;
        }
        dvb->front.volt=volt;
        // this guarantees that running SEC commands are done atomically
        Set22K(dvb, dvb->front.ttk);
        SetVoltage(dvb, power, volt);
        return 0;
}

int secSendSequence(struct dvb_struct *dvb, struct secCmdSequence *seq)
{
        int i, ret, burst, len;
        u8 msg[16];

        switch (seq->miniCommand) {
        case SEC_MINI_NONE:
                burst=-1;
                break;
        case SEC_MINI_A:
                burst=0;
                break;
        case SEC_MINI_B:
                burst=1;
                break;
        default:
                return -EINVAL;
        }

        for (i=0; i<seq->numCommands; i++) {
                switch (seq->commands[i].type) {
                case SEC_CMDTYPE_DISEQC:
                        len=seq->commands[i].u.diseqc.numParams;
                        if (len>SEC_MAX_DISEQC_PARAMS)
                                return -EINVAL;

                        msg[0]=0xe0;
                        msg[1]=seq->commands[i].u.diseqc.addr;
                        msg[2]=seq->commands[i].u.diseqc.cmd;
                        memcpy(msg+3, &seq->commands[i].u.diseqc.params, len);
                        SendDiSEqCMsg(dvb, len+3, msg, -1);
                        break;
                case SEC_CMDTYPE_PAUSE:
                        //FIXME: fix firmware to allow programmed pause length
                        SendDiSEqCMsg(dvb, 0, msg, -1);
                        break;
                case SEC_CMDTYPE_VSEC:
                default:
                        return -EINVAL;
                }
        }
        
        if (burst!=-1)
                SendDiSEqCMsg(dvb, 0, msg, burst);
        
        ret=secSetVoltage(dvb, seq->voltage);
        if (ret<0)
                return ret;
        return secSetTone(dvb, seq->continuousTone);
}




/******************************************************************************
 * DVB device file operations
 ******************************************************************************/

static int 
dvbdev_open(struct dvb_device *dvbdev, int type, 
            struct inode *inode, struct file *file)
{
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdev->priv;

	switch (type) {
	case DVB_DEVICE_VIDEO:
                        dvb->avout.tail = dvb->avout.head;
                dvb->avout.tail = dvb->avout.head;
                dvb->video_blank=true;
                dvb->audiostate.AVSyncState=true;
                dvb->videostate.streamSource=VIDEO_SOURCE_DEMUX;
                break;
	case DVB_DEVICE_AUDIO:
                dvb->avout.tail = dvb->avout.head;
                dvb->audiostate.streamSource=AUDIO_SOURCE_DEMUX;
                break;
	case DVB_DEVICE_SEC:
                if (file->f_flags&O_NONBLOCK) 
                        return -EWOULDBLOCK;
                break;
	case DVB_DEVICE_FRONTEND:
                break;
	case DVB_DEVICE_DEMUX:
                if ((file->f_flags&O_ACCMODE)!=O_RDWR)
                        return -EINVAL;
                return DmxDevFilterAlloc(&dvb->dmxdev, file);
	case DVB_DEVICE_DVR:
                return DmxDevDVROpen(&dvb->dmxdev, file);
        default:
                return -EINVAL;
	}
        return 0;
}

static int 
dvbdev_close(struct dvb_device *dvbdev, int type, 
             struct inode *inode, struct file *file)
{
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdev->priv;

	switch (type) {
	case DVB_DEVICE_VIDEO:
	        AV_Stop(dvb, RP_VIDEO);
		break;
	case DVB_DEVICE_AUDIO:
	        AV_Stop(dvb, RP_AUDIO);
                break;
	case DVB_DEVICE_SEC:
                break;
	case DVB_DEVICE_FRONTEND:
                break;
	case DVB_DEVICE_DEMUX:
                return DmxDevFilterFree(&dvb->dmxdev, file);
	case DVB_DEVICE_DVR:
                return DmxDevDVRClose(&dvb->dmxdev, file);
        default:
                return -EINVAL;
	}
        return 0;
}

static ssize_t 
dvbdev_read(struct dvb_device *dvbdev, int type, 
            struct file *file, char *buf, size_t count, loff_t *ppos)
{
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdev->priv;

	switch (type) {
	case DVB_DEVICE_VIDEO:
		break;
	case DVB_DEVICE_AUDIO:
                break;
	case DVB_DEVICE_DEMUX:
                return DmxDevRead(&dvb->dmxdev, file, buf, count, ppos);
	case DVB_DEVICE_DVR:
                return DmxDevDVRRead(&dvb->dmxdev, file, buf, count, ppos);
	default:
	        return -EOPNOTSUPP;
	}
        return 0;
}

static ssize_t 
dvbdev_write(struct dvb_device *dvbdev, int type,
             struct file *file, 
             const char *buf, size_t count, loff_t *ppos)
{
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdev->priv;

	switch (type) {
	case DVB_DEVICE_VIDEO:
                if (dvb->videostate.streamSource!=VIDEO_SOURCE_MEMORY)
                        return -EPERM;
                return dvb_play(dvb, buf, count, file->f_flags&O_NONBLOCK);
	case DVB_DEVICE_AUDIO:
                if (dvb->audiostate.streamSource!=AUDIO_SOURCE_MEMORY)
                        return -EPERM;
	        return dvb_play(dvb, buf, count, file->f_flags&O_NONBLOCK);
	case DVB_DEVICE_DVR:
                return DmxDevDVRWrite(&dvb->dmxdev, file, buf, count, ppos);
	default:
	        return -EOPNOTSUPP;
	}
}

// badly transferred from graph in datasheet
uint32_t nest2snr[256] = {
  
  

};

static int 
dvbdev_ioctl(struct dvb_device *dvbdev, int type, 
             struct file *file, unsigned int cmd, unsigned long arg)
{
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdev->priv;
        void *parg=(void *)arg;

	switch (type) {
	case DVB_DEVICE_VIDEO:
                if (((file->f_flags&O_ACCMODE)==O_RDONLY) &&
                    (cmd!=VIDEO_GET_STATUS))
                        return -EPERM;

                switch (cmd) {

                case VIDEO_STOP:
		        dvb->videostate.playState=VIDEO_STOPPED;
                        if (dvb->videostate.streamSource==VIDEO_SOURCE_MEMORY)
                                AV_Stop(dvb, RP_VIDEO);
                        else
                                vidcom(dvb, 0x000e, 
                                       dvb->videostate.videoBlank ? 0 : 1);
                        return 0; 

                case VIDEO_PLAY:
                     
                        if (dvb->videostate.streamSource==VIDEO_SOURCE_MEMORY) {
                                if (dvb->playing==RP_AV) {
                                        outcom(dvb, COMTYPE_REC_PLAY, __Stop, 0);
                                        dvb->playing&=~RP_VIDEO;
                                }
                                AV_StartPlay(dvb,RP_VIDEO);
                                vidcom(dvb, 0x000d, 0);
                        } else {
                                AV_Stop(dvb, RP_VIDEO);
                                vidcom(dvb, 0x000d, 0);
                        }
		        dvb->videostate.playState=VIDEO_PLAYING;
                        break;

                case VIDEO_FREEZE:
		        dvb->videostate.playState=VIDEO_FREEZED;
                        vidcom(dvb, 0x000e, 1);
                        break;

                case VIDEO_CONTINUE:
		        if (dvb->videostate.playState==VIDEO_FREEZED) {
			        dvb->videostate.playState=VIDEO_PLAYING;
				vidcom(dvb, 0x000d, 0);
			} 
                        break;

                case VIDEO_SELECT_SOURCE:
                        dvb->videostate.streamSource=(videoStreamSource_t) arg;
                        break;

                case VIDEO_SET_BLANK:
                        dvb->videostate.videoBlank=(boolean) arg;
                        break;

                case VIDEO_GET_STATUS:
                        if(copy_to_user(parg, &dvb->videostate, 
					sizeof(struct videoStatus)))
                                return -EFAULT;
                        break;

                case VIDEO_GET_EVENT:
                        //FIXME: write firmware support for this
                        return -EOPNOTSUPP;

                case VIDEO_SET_DIPLAY_FORMAT:
                {
                        videoDisplayFormat_t format=(videoDisplayFormat_t) arg;
                        u16 val=0;
                        
                        switch(format) {
                        case VIDEO_PAN_SCAN:
                                val=VID_PAN_SCAN_PREF;
                                break;

                        case VIDEO_LETTER_BOX:
                                val=VID_VC_AND_PS_PREF;
                                break;

                        case VIDEO_CENTER_CUT_OUT:
                                val=VID_CENTRE_CUT_PREF;
                                break;

                        default:
                                return -EINVAL;
                        }

                        dvb->videostate.videoFormat=format;
                        return outcom(dvb, COMTYPE_ENCODER, SetPanScanType, 
                                      1, (u16) val);
                }

                case VIDEO_STILLPICTURE:
		{ 
		        struct videoDisplayStillPicture pic;

                        if(copy_from_user(&pic, parg, 
                                          sizeof(struct videoDisplayStillPicture)))
                                return -EFAULT;

                        AV_StartPlay(dvb, RP_VIDEO);
                        dvb_play(dvb, pic.iFrame, pic.size, file->f_flags&O_NONBLOCK);
                        dvb_play(dvb, pic.iFrame, pic.size, file->f_flags&O_NONBLOCK);
                        dvb_play(dvb, pic.iFrame, pic.size, file->f_flags&O_NONBLOCK);
                        dvb_play(dvb, pic.iFrame, pic.size, file->f_flags&O_NONBLOCK);
                        dvb_play(dvb, pic.iFrame, pic.size, file->f_flags&O_NONBLOCK);
                        break;
		}

                case VIDEO_FAST_FORWARD:
                        if (dvb->videostate.streamSource!=VIDEO_SOURCE_MEMORY)
                                return -EPERM;
                        //note: arg is ignored by firmware
                        if (dvb->playing==RP_AV) 
                                outcom(dvb, COMTYPE_REC_PLAY, __FF_IP, 2, AV_PES, 0);
                        else 
                                vidcom(dvb, 0x16, arg); 
                        break;

                case VIDEO_SLOWMOTION:
                        if (dvb->videostate.streamSource!=VIDEO_SOURCE_MEMORY)
                                return -EPERM;
                        if (dvb->playing==RP_AV) {
                                outcom(dvb, COMTYPE_REC_PLAY, __Slow, 2, 0, 0);
                                vidcom(dvb, 0x22, arg);
                        } else {
                                vidcom(dvb, 0x0d, 0);
                                vidcom(dvb, 0x0e, 0);
                                vidcom(dvb, 0x22, arg);
                        }
                        break;

                default:
                        return -ENOIOCTLCMD;
		}
                return 0;

	case DVB_DEVICE_AUDIO:
                if (((file->f_flags&O_ACCMODE)==O_RDONLY) &&
                    (cmd!=AUDIO_GET_STATUS))
                        return -EPERM;

                switch (cmd) {

                case AUDIO_STOP:
                        if (dvb->audiostate.streamSource==AUDIO_SOURCE_MEMORY)
                                AV_Stop(dvb, RP_AUDIO);
                        else
                                audcom(dvb, 1);
                        dvb->audiostate.playState=AUDIO_STOPPED;
                        break;

                case AUDIO_PLAY:
                        if (dvb->audiostate.streamSource==AUDIO_SOURCE_MEMORY)
                                AV_StartPlay(dvb, RP_AUDIO);
                        audcom(dvb, 2);
                        dvb->audiostate.playState=AUDIO_PLAYING;
		        break;

                case AUDIO_PAUSE:
			audcom(dvb, 1);
		        dvb->audiostate.playState=AUDIO_PAUSED;
                        break;

                case AUDIO_CONTINUE:
		        if (dvb->audiostate.playState==AUDIO_PAUSED) {
			        dvb->audiostate.playState=AUDIO_PLAYING;
				audcom(dvb, 2);
			} 
		        break;

                case AUDIO_SELECT_SOURCE:
                        dvb->audiostate.streamSource=(audioStreamSource_t) arg;
		        break;

                case AUDIO_SET_MUTE:
                {
		        u16 val=((boolean) arg) ? 0x80 : 0xc5; 

                        dvb->audiostate.muteState=(boolean) arg;
		        SendDAC(dvb, 3, val);
		        SendDAC(dvb, 4, val);
			audcom(dvb, arg ? 1 : 2);
                        break;
		}
                case AUDIO_SET_AV_SYNC:
                        dvb->audiostate.AVSyncState=(boolean) arg;
			audcom(dvb, arg ? 0x0f : 0x0e);
                        break;

                case AUDIO_SET_BYPASS_MODE:
		        return -EINVAL;

                case AUDIO_CHANNEL_SELECT:
		        dvb->audiostate.channelSelect=(audioChannelSelect_t) arg;
                        
                        switch(dvb->audiostate.channelSelect) {
                        case AUDIO_STEREO:
  			        audcom(dvb, 0x80);
                                break;

                        case AUDIO_MONO_LEFT:
  			        audcom(dvb, 0x100);
                                break;

                        case AUDIO_MONO_RIGHT:
  			        audcom(dvb, 0x200);
                                break;

                        default:
                                return -EINVAL;
			}
			return 0;

                case AUDIO_GET_STATUS:
                        if(copy_to_user(parg, &dvb->audiostate, 
					sizeof(struct audioStatus)))
                                return -EFAULT;
                        break;

                default:
                        return -ENOIOCTLCMD;
                }
		break;

	case DVB_DEVICE_FRONTEND:
                switch (cmd) {
                case OST_SELFTEST:
                        break;

                case OST_SET_POWER_STATE:
                        switch (arg) {
                        case OST_POWER_ON:
                                dvb->powerstate=arg;
                                SetVoltage(dvb,1,0);
                                break;
                        case OST_POWER_SUSPEND:
                        case OST_POWER_STANDBY:
                        case OST_POWER_OFF:
                                dvb->powerstate=OST_POWER_OFF;
                                SetVoltage(dvb,0,0);
                                break;
                        default:
                                return -EINVAL;
                        }
                        break;

                case OST_GET_POWER_STATE:
                        if(copy_to_user(parg, &dvb->powerstate, sizeof(u32)))
                                 return -EFAULT;
                        break;

                case FE_READ_STATUS:
                {
                        feStatus stat;
                        
                        GetFront(dvb);
                        stat=0;
                        if (dvb->front.power)
                                stat|=FE_HAS_POWER;
                        if ((dvb->front.sync&0x1f)==0x1f)
                                stat|=FE_HAS_SIGNAL;
                        if (dvb->front.inv)
                                stat|=QPSK_SPECTRUM_INV;

                        if(copy_to_user(parg, &stat, sizeof(stat)))
                                return -EFAULT;
                        break;
                }

                case FE_READ_BER:
                {
                        uint32_t ber;
                        
                        GetFront(dvb);
                        if (!dvb->front.power)
                                return -ENOSIGNAL;
                        ber=dvb->front.vber*10;
                        if(copy_to_user(parg, &ber, sizeof(ber)))
                                return -EFAULT;
                        break;
                }

                case FE_READ_SIGNAL_STRENGTH:
                {
                        int32_t signal;
                        
                        GetFront(dvb);
                        if (!dvb->front.power)
                                return -ENOSIGNAL;
                        signal=dvb->front.agc; //FIXME: scale value
                        if(copy_to_user(parg, &signal, sizeof(signal)))
                                return -EFAULT;
                        break;
                }

                case FE_READ_SNR:
                {
                        int32_t snr;
                        
                        GetFront(dvb);
                        if (!dvb->front.power)
                                return -ENOSIGNAL;
                        //linear approximation, FIXME: make nest2snr table
                        snr=20000000+(10-dvb->front.nest)*20000000/160;
                        //snr=nest2snr[dvb->front.nest];
                        if(copy_to_user(parg, &snr, sizeof(snr)))
                                return -EFAULT;
                        break;
                }

                case FE_READ_UNCORRECTED_BLOCKS: 
                  //FIXME: VES1893 only supports counter up to 0x7f
                  //       how should we handle this??
                        return -ENOSYS; 

                case FE_GET_NEXT_FREQUENCY:
                {
                        uint32_t freq;
                    
                        if(copy_from_user(&freq, parg, sizeof(freq)))
                                return -EFAULT;

                        // FIXME: how does one calculate this?
                        freq+=1000; //FIXME: KHz like in QPSK_TUNE??
                        
                        if(copy_to_user(parg, &freq, sizeof(freq)))
                                return -EFAULT;
                        break;
                }

                case FE_GET_NEXT_SYMBOL_RATE:
                {
                        uint32_t rate;
                        
                        if(copy_from_user(&rate, parg, sizeof(rate)))
                                return -EFAULT;
                        // FIXME: how does one really calculate this?
                        if (rate<5000000)
                                rate+=500000;
                        else if(rate<10000000)
                                rate+=1000000;
                        else if(rate<30000000)
                                rate+=2000000;
                        else 
                                return -EINVAL;
                        if(copy_to_user(parg, &rate, sizeof(rate)))
                                return -EFAULT;
                        break;
                }

                case QPSK_TUNE:
                {
                        struct qpskParameters para;
                        static const uint8_t fectab[8]={8,0,1,2,4,6,0,8};
                        u32 val;
                        
                        if(copy_from_user(&para, parg, sizeof(para)))
                                return -EFAULT;
                        if ((file->f_flags&O_ACCMODE)==O_RDONLY) 
                                return -EPERM;
                        if (para.FEC_inner>7)
                                return -EINVAL;
                        val=para.iFrequency*1000;
                        if (dvb->front.freq!=val) {
                                tuner_command(dvb, TUNER_SET_TVFREQ, &val); 
                                dvb->front.freq=val;
                        }

                        val=para.SymbolRate;
                        if (dvb->front.srate!=val || 
                            dvb->front.fec!=fectab[para.FEC_inner]) {
                                dvb->front.srate=val;
                                dvb->front.fec=fectab[para.FEC_inner];
                                decoder_command(dvb, DVB_SET_FRONTEND, &dvb->front); 
                        } 
                        {
                                struct qpskEvent ev;
                                
                                ev.type=QPSK_COMPLETION_EV;
                                ev.u.completionEvent.iFrequency=para.iFrequency;
                                ev.u.completionEvent.SymbolRate=para.SymbolRate;
                                ev.u.completionEvent.FEC_inner=para.FEC_inner;
                                
                                add_qpsk_event(&dvb->qpsk, &ev);

                        }
                        return 0;
                }

                case QPSK_GET_EVENT:
                { 
                        int ret;

                        //printk("get_qpsk_event\n");
                        if (dvb->qpsk.overflow) {
                                dvb->qpsk.overflow=0;
                                return -EBUFFEROVERFLOW;
                        }
                        if (dvb->qpsk.eventw==dvb->qpsk.eventr) {
                                if (file->f_flags&O_NONBLOCK) 
                                        return -EWOULDBLOCK;
                                else {
                                  ret=wait_event_interruptible(dvb->qpsk.eventq,
                                                               dvb->qpsk.eventw!=
                                                               dvb->qpsk.eventr);
                                  if (ret<0)
                                          return ret;
                                }
                        }
                        spin_lock(&dvb->qpsk.eventlock);

                        if(copy_to_user(parg, &dvb->qpsk.events[dvb->qpsk.eventr], 
                                        sizeof(struct qpskEvent))) {
                                spin_unlock(&dvb->qpsk.eventlock);
                                return -EFAULT;
                        }
                        dvb->qpsk.eventr=(dvb->qpsk.eventr+1)%QPSK_MAX_EVENT;
                        spin_unlock(&dvb->qpsk.eventlock);
                        return 0;
                }

                case QPSK_FE_INFO:
                {        
                        struct qpskFrontendInfo feinfo;
                        
                        feinfo.minFrequency=500;  //KHz?
                        feinfo.maxFrequency=2100000;
                        feinfo.minSymbolRate=500000;
                        feinfo.maxSymbolRate=30000000;
                        feinfo.hwType=0;    //??
                        feinfo.hwVersion=0; //??
                        if(copy_to_user(parg, &feinfo, sizeof(feinfo)))
                                return -EFAULT;
                        break;
                }

                case QPSK_WRITE_REGISTER:
                        return -EINVAL;

                case QPSK_READ_REGISTER:
                        return -EINVAL;
                }
		return 0;
        
        // NOTE: right now the DiSEqC support in the firmware is set up
        //       as busy waiting so that atomicity is guaranteed automatically
	case DVB_DEVICE_SEC:
                if (file->f_flags&O_NONBLOCK) 
                        return -EWOULDBLOCK;
                switch (cmd) {
                case SEC_GET_STATUS: 
                {        
                        struct secStatus status;
                        
                        status.busMode=SEC_BUS_IDLE;
                        
                        if (dvb->secbusy) 
                                status.busMode=SEC_BUS_BUSY;
                        if (!dvb->front.power) 
                                status.busMode=SEC_BUS_OFF;
                        
                        status.selVolt=(dvb->front.volt ?
                                        SEC_VOLTAGE_18 :
                                        SEC_VOLTAGE_13);

                        status.contTone=(dvb->front.ttk ?
                                         SEC_TONE_ON :
                                         SEC_TONE_OFF);
                        if(copy_to_user(parg, &status, sizeof(status)))
                                return -EFAULT;
                }
                        break;
                case SEC_RESET_OVERLOAD:
                        if ((file->f_flags&O_ACCMODE)==O_RDONLY) 
                                return -EPERM;
                        // overload control not supported by Siemens DVB card
                        break;

                case SEC_SEND_SEQUENCE:
                {
                        struct secCmdSequence seq;

                        if(copy_from_user(&seq, parg, sizeof(seq)))
                                return -EFAULT;

                        if ((file->f_flags&O_ACCMODE)==O_RDONLY) 
                                return -EPERM;

                        return secSendSequence(dvb, &seq);
                }
                case SEC_SET_TONE:
                {
                        secToneMode mode = (secToneMode) arg;

                        if ((file->f_flags&O_ACCMODE)==O_RDONLY) 
                                return -EPERM;
                        
                        return secSetTone(dvb, mode);
                }
                case SEC_SET_VOLTAGE:
                {
                        secVoltage val = (secVoltage) arg;

                        if ((file->f_flags&O_ACCMODE)==O_RDONLY) 
                                return -EPERM;
                        
                        return secSetVoltage(dvb, val);

                }
                default:
                        return -EINVAL;
                }
		return 0;

        case DVB_DEVICE_DEMUX:
                return DmxDevIoctl(&dvb->dmxdev, file, cmd, arg);

        case DVB_DEVICE_CA:
                switch (cmd) {
                case CA_RESET:
                  
                      break;

                case CA_GET_CAP:
                {
                        ca_cap_t cap;

                        cap.slot_num=2;
                        cap.slot_type=CA_CI;
                        cap.dscr_num=0;
                        cap.dscr_type=0;
                        if(copy_to_user(parg, &cap, sizeof(cap)))
                                return -EFAULT;
                }
                        break;
                case CA_GET_SLOT_INFO:
                {
                        ca_slot_info_t info;

                        if(copy_from_user(&info, parg, sizeof(info)))
                                return -EFAULT;
                        if (info.num>1)
                                return -EINVAL;
                        info.type=CA_CI;
                        info.flags=0;
                        if (dvb->ci_slot[info.type]&CI_MODULE_PRESENT)
                                info.flags|=CA_CI_MODULE_PRESENT;
                        if (dvb->ci_slot[info.type]&CI_MODULE_OK)
                                info.flags|=CA_CI_MODULE_READY;
                                                
                        if(copy_to_user(parg, &info, sizeof(info)))
                                return -EFAULT;

                }
                        break;
                case CA_GET_MSG:

                        break;
                case CA_SEND_MSG:

                        break;

                default:
                        return -EINVAL;
                }
                break;
        default:
                return -EOPNOTSUPP;
        }

        return 0;
}

static unsigned int 
dvbdev_poll(struct dvb_device *dvbdev, int type, 
            struct file *file, poll_table * wait)
{
        struct dvb_struct *dvb=(struct dvb_struct *) dvbdev->priv;

	switch (type) {
	case DVB_DEVICE_FRONTEND:
                if (dvb->qpsk.eventw!=dvb->qpsk.eventr)
                        return (POLLIN | POLLRDNORM | POLLPRI);

                poll_wait(file, &dvb->qpsk.eventq, wait);
                
                if (dvb->qpsk.eventw!=dvb->qpsk.eventr)
                        return (POLLIN | POLLRDNORM | POLLPRI);
                return 0;
        case DVB_DEVICE_DEMUX:
                return DmxDevPoll(&dvb->dmxdev, file, wait);

        case DVB_DEVICE_VIDEO:
                return dvb_poll(&dvb->video, file, wait);
                
        case DVB_DEVICE_AUDIO:
                return dvb_poll(&dvb->video, file, wait);

	default:
	        return -EOPNOTSUPP;
        }

        return 0;
}

/******************************************************************************
 * driver registration 
 ******************************************************************************/

static int dvb_register(struct dvb_struct *dvb)
{
        int result, i;
        struct dvb_device *dvbd=&dvb->dvb_dev;

        dvb->dmxdev.filternum=32;
        dvb->dmxdev.demux=&dvb->hw_demux.dmx;
        //dvb->dmxdev.set_pids=(int (*)(void *, u16 *)) SetPIDArray;
        //dvb->dmxdev.priv=(void *) dvb;
        dvb->dmxdev.capabilities=0;
        DmxDevInit(&dvb->dmxdev);
        
        for (i=0; i<32; i++)
                dvb->handle2filter[i]=NULL;

        init_waitqueue_head(&dvb->qpsk.eventq);
        spin_lock_init (&dvb->qpsk.eventlock);
        dvb->qpsk.eventw=dvb->qpsk.eventr=0;
        dvb->qpsk.overflow=0;
        dvb->secbusy=0;

	dvb->audiostate.AVSyncState=0;
	dvb->audiostate.muteState=0;
	dvb->audiostate.playState=AUDIO_STOPPED;
	dvb->audiostate.streamSource=AUDIO_SOURCE_DEMUX;
	dvb->audiostate.channelSelect=AUDIO_STEREO;
	dvb->audiostate.bypassMode=0;

	dvb->videostate.videoBlank=0;
	dvb->videostate.playState=VIDEO_STOPPED;
	dvb->videostate.streamSource=VIDEO_SOURCE_DEMUX;
	dvb->videostate.videoFormat=VIDEO_FORMAT_4_3;
	dvb->videostate.displayFormat=VIDEO_CENTER_CUT_OUT;

        // init and register dvb device structure
        dvbd->priv=(void *) dvb;
        dvbd->open=dvbdev_open;
        dvbd->close=dvbdev_close;
        dvbd->read=dvbdev_read;
        dvbd->write=dvbdev_write;
        dvbd->ioctl=dvbdev_ioctl;
        dvbd->poll=dvbdev_poll;
        
        result=dvb_register_device(&dvb->dvb_dev);
        if (result<0)
                return result;


        // init and register demuxes
        
        memcpy(dvb->hw_demux_id, "demux0", 7);
        dvb->hw_demux_id[5]=dvb->num+0x30;
        DvbDmxInit(&dvb->hw_demux, (void *) dvb, dvb->hw_demux_id,
                   "TI", "AV7110");
        dvb->hw_demux.start_feed=dvb_start_feed;
        dvb->hw_demux.stop_feed=dvb_stop_feed;
        if (dvb->cardtype==DVB_CARD_TT_SIEMENS)
                dvb->hw_demux.write_to_decoder=dvb_write_to_decoder;

        dvb->mem_frontend.id="mem_frontend";
        dvb->mem_frontend.vendor="memory";
        dvb->mem_frontend.model="sw";
        dvb->mem_frontend.source=DMX_MEMORY_FE;

        /*
        memcpy(dvb->sw_demux_id, "demuxs0", 8);
        dvb->sw_demux_id[6]=dvb->num+0x30;
        DvbDmxInit(&dvb->sw_demux, (void *) dvb, dvb->sw_demux_id,
                     "TI", "AV7110");
        
        dvb->sw_demux.dmx.add_frontend(&dvb->sw_demux.dmx, &dvb->mem_frontend);
        */

        return 0;
}


/* Register frontend
   This is an extra routine because the frontend type is not known until the
   frontend module registered itself on the I2C bus (which can happen after the 
   dvb module is installed)
*/

static int dvb_register_frontend(struct dvb_struct *dvb)
{
        int ret;
        dmx_frontend_t *dvbfront=&dvb->hw_frontend;

        dvbfront->id="hw_frontend";
        dvbfront->vendor="VLSI";
        switch (dvb->dvbtype) {
        default:
        case DVB_S:
                dvbfront->model="1893";
                dvbfront->source=DMX_SATELLITE_FE;
                break;
        case DVB_C:
                dvbfront->model="1820";
                dvbfront->source=DMX_CABLE_FE;
                break;
        }

        ret=dvb->hw_demux.dmx.add_frontend(&dvb->hw_demux.dmx, &dvb->hw_frontend);
        if (ret<0)
                return ret;
        
        ret=dvb->hw_demux.dmx.add_frontend(&dvb->hw_demux.dmx, &dvb->mem_frontend);
        if (ret<0)
                return ret;

        ret=dvb->hw_demux.dmx.connect_frontend(&dvb->hw_demux.dmx, &dvb->hw_frontend);
        if (ret<0)
                return ret;

        return 0;
}


static void dvb_unregister(struct dvb_struct *dvb)
{
        dvb->hw_demux.dmx.close(&dvb->hw_demux.dmx);
        dvb->hw_demux.dmx.remove_frontend(&dvb->hw_demux.dmx, &dvb->hw_frontend);
        dvb->hw_demux.dmx.remove_frontend(&dvb->hw_demux.dmx, &dvb->mem_frontend);
        dmx_unregister_demux(&dvb->hw_demux.dmx);

        /*
        dvb->sw_demux.dmx.close(&dvb->sw_demux.dmx);
        dvb->sw_demux.dmx.remove_frontend(&dvb->sw_demux.dmx, &dvb->mem_frontend);
        dmx_unregister_demux(&dvb->sw_demux.dmx);
        */

        dvb_unregister_device(&dvb->dvb_dev);
}

/****************************************************************************
 * NETWORK DRIVER
 ****************************************************************************/

struct dvb_net_priv {
        struct net_device_stats stats;
        struct dvb_struct *dvb;
};


/*
 *	Determine the packet's protocol ID. The rule here is that we 
 *	assume 802.3 if the type field is short enough to be a length.
 *	This is normal practice and works for any 'now in use' protocol.
 *
 *  stolen from eth.c out of the linux kernel, hacked for dvb-device
 *  by Michael Holzt <kju@debian.org>
 */
 
unsigned short my_eth_type_trans(struct sk_buff *skb, struct net_device *dev)
{
	struct ethhdr *eth;
	unsigned char *rawp;
	
	skb->mac.raw=skb->data;
	skb_pull(skb,dev->hard_header_len);
	eth= skb->mac.ethernet;
	
	if(*eth->h_dest&1)
	{
		if(memcmp(eth->h_dest,dev->broadcast, ETH_ALEN)==0)
			skb->pkt_type=PACKET_BROADCAST;
		else
			skb->pkt_type=PACKET_MULTICAST;
	}
	
	if (ntohs(eth->h_proto) >= 1536)
		return eth->h_proto;
		
	rawp = skb->data;
	
	/*
	 *	This is a magic hack to spot IPX packets. Older Novell breaks
	 *	the protocol design and runs IPX over 802.3 without an 802.2 LLC
	 *	layer. We look for FFFF which isn't a used 802.2 SSAP/DSAP. This
	 *	won't work for fault tolerant netware but does for the rest.
	 */
	if (*(unsigned short *)rawp == 0xFFFF)
		return htons(ETH_P_802_3);
		
	/*
	 *	Real 802.2 LLC
	 */
	return htons(ETH_P_802_2);
}

 
static void net_fsection(struct net_device *dev, u8 *pkt, u16 pkt_len, u16 port)
{
        u8 *eth, *ip, *udp;
        u16 udplen, iplen;
        struct sk_buff *skb;
        struct in_device *in_dev;
        struct in_ifaddr *ifa; 
       
        if (pkt_len == 0)
                return;

        skb = dev_alloc_skb(pkt_len+42);
        if (skb == NULL) {
                printk(KERN_NOTICE "%s: Memory squeeze, dropping packet.\n",
                       dev->name);
                return;
        }

        eth=(u8 *) skb_put(skb,pkt_len+42);
        memcpy(eth+42, (void*)pkt, pkt_len);

        udplen=pkt_len+8;
        iplen=udplen+20;

        ip=eth+14; udp=ip+20; 

        memcpy(eth, dev->dev_addr, 6);
        eth[6]=eth[7]=eth[8]=eth[9]=eth[10]=eth[11]=0;
        eth[12]=0x08; eth[13]=0x00;

        // IP
        ip[0]=0x45; ip[1]=0x00;
	ip[2] = (iplen>>8); ip[3] = (iplen&0xff); 
	ip[4] = 0; ip[5] = 0; 
	ip[6] = 0; ip[7] = 0; 
	ip[8] = 0x20; ip[9] = 0x11; // TTL, Protocol
	ip[10]= 0x00; ip[11]= 0x00; // header CRC
	ip[12]= 0x00; ip[13]= 0x00; ip[14]= 0x00; ip[15]= 0x00;
        ip[16]= 0; ip[17]= 0; ip[18]= 0; ip[19]= 0; 

        /*
        for (i=0, cs=0; i<10; i++)
                cs+=~htons(*(i+(u16 *)ip));
        cs=~cs;
	ip[10]= cs>>8; ip[11]= cs&0xff;
        */

	if ((in_dev=dev->ip_ptr) != NULL) 
                if ((ifa=in_dev->ifa_list) != NULL)
                        memcpy(ip+16, &ifa->ifa_address, 4),
                          memcpy(ip+12, &ifa->ifa_address, 4);

        // UDP
	udp[0] = (port>>8); udp[1] = (port&0xff); 
	udp[2] = (port>>8); udp[3] = (port&0xff); 
        udp[4] = (udplen>>8); udp[5] = (udplen&0xff);
	udp[6] = 0; udp[7]=0;

	skb->protocol=my_eth_type_trans(skb,dev);
        skb->dev = dev;
	skb->ip_summed = CHECKSUM_UNNECESSARY;

        ((struct dvb_net_priv *)dev->priv)->stats.rx_bytes += skb->len;
        ((struct dvb_net_priv *)dev->priv)->stats.rx_packets++;
        //sti();
        netif_rx(skb);

}
 

static void net_ipmpe(struct net_device *dev, u8 *pkt, int pkt_len)
{
        u8 *eth;
        struct sk_buff *skb;
       
        if (pkt_len == 0)
                return;

        skb = dev_alloc_skb(pkt_len+2);
        if (skb == NULL) {
                printk(KERN_NOTICE "%s: Memory squeeze, dropping packet.\n",
                       dev->name);
                ((struct dvb_net_priv *)dev->priv)->stats.rx_dropped++;
                return;
        }

        eth=(u8 *) skb_put(skb,pkt_len+2);
        memcpy(eth+14, (void*)pkt+12, pkt_len-12);

        eth[0]=pkt[0x0b];
        eth[1]=pkt[0x0a];
        eth[2]=pkt[0x09];
        eth[3]=pkt[0x08];
        eth[4]=pkt[0x04];
        eth[5]=pkt[0x03];
        eth[6]=eth[7]=eth[8]=eth[9]=eth[10]=eth[11]=0;
        eth[12]=0x08; eth[13]=0x00;

	skb->protocol=my_eth_type_trans(skb,dev);
        skb->dev = dev;
        
        ((struct dvb_net_priv *)dev->priv)->stats.rx_packets++;
        ((struct dvb_net_priv *)dev->priv)->stats.rx_bytes += skb->len;
        sti();
        netif_rx(skb);

}
 

static int
net_tx(struct sk_buff *skb, struct net_device *dev)
{
	return 0;
}

static void
set_multicast_list(struct net_device *dev)
{
	if (dev->flags&IFF_PROMISC)
	{
		/* Enable promiscuous mode */
	}
	else if((dev->flags&IFF_ALLMULTI))
	{
		/* Disable promiscuous mode, use normal mode. */
          //hardware_set_filter(NULL);

	}
	else if(dev->mc_count)
	{
                int mci=0;
                struct dev_mc_list *mc;

                for (mc=dev->mc_list; (mc!=NULL) && (mci<MCMAX); mc=mc->next, mci++) {
                  //set_mc_filter(dev, );
                } 

	}
	else 
          ;
}

static int
net_open(struct net_device *dev)
{
#if LINUX_VERSION_CODE < 0x02032b
  	dev->tbusy = 0;
	dev->interrupt = 0;
	dev->start = 1;
#endif
	MOD_INC_USE_COUNT;

	return 0;
}

static int
net_close(struct net_device *dev)
{
#if LINUX_VERSION_CODE < 0x02032b
	dev->tbusy = 1;
	dev->start = 0;
#endif
	MOD_DEC_USE_COUNT;

	return 0;
}

static struct net_device_stats *net_get_stats(struct net_device *dev)
{
        return &((struct dvb_net_priv *)dev->priv)->stats;
}


static int readMAC(struct dvb_struct *dvb, u8 *buf)
{
        struct i2c_adapter *adap=dvb->i2cbus;
        unsigned char mm1=0xd4;
        struct i2c_msg msgs[2];
        
        msgs[0].flags=0;
        msgs[1].flags=I2C_M_RD;
        msgs[0].addr=msgs[1].addr=0x50;
        msgs[0].len=1; msgs[1].len=6;
        msgs[0].buf=&mm1; msgs[1].buf=buf;
        i2c_transfer(adap, msgs, 2);
        return 0;
}

static const u8  k0[3] = { 0x54, 0x7B, 0x9E };
static const u8  k1[3] = { 0xD3, 0xF1, 0x23 };

static void getMAC(struct dvb_struct *dvb)
{
        u8 data[6];
        u8 *imac=data;
        u8 tmp1, tmp0;
        int i;

        dvb->net.dev_addr[0]=0x00;
        dvb->net.dev_addr[1]=0xd0;
        dvb->net.dev_addr[2]=0x5c;
        dvb->net.dev_addr[3]=0x01;
        dvb->net.dev_addr[4]=0x02;
        dvb->net.dev_addr[5]=0x03;

        if (!readMAC(dvb, data))
                for(i=0; i<3; i++) {
                        tmp0 = *(imac++) ^ k0[i];
                        tmp1 = *(imac++) ^ k1[i];
                        dvb->net.dev_addr[5-i]=
                          (u8)(( ((tmp1<<8)|tmp0) >> ((tmp1 >> 6) & 0x3)));
                }
}

static int netcard_probe(struct net_device *dev)
{
	int i;

	if (dev == NULL) {
		dev = init_etherdev(0, 0);
		if (dev == NULL)
			return -ENOMEM;
	}
        printk("dvb: MAC = ");
	for (i = 0; i < 6; i++)
                printk("%2.2x ", dev->dev_addr[i]);
        printk("\n");

	/* Initialize the device structure. */
	dev->mtu		= 4096;
	dev->open		= net_open;
	dev->stop		= net_close;
	dev->hard_start_xmit	= net_tx;
	dev->get_stats		= net_get_stats;
	dev->set_multicast_list = &set_multicast_list;
	ether_setup(dev);

	return 0;
}

static int 
init_net(struct dvb_struct *dvb)
{
	int result;
        struct net_device *net=&dvb->net;
        
	net->base_addr = 0;
	net->irq       = 0;
	net->dma       = 0;
	net->mem_start = 0;

        memcpy(dvb->netname, "dvb0", 5);
        dvb->netname[3]=dvb->num+0x30;

/* Does not work pre one of the pre-patches of 2.3.99pre7 */
#if LINUX_VERSION_CODE < 0x020363    
        net->name      = dvb->netname;
#else
        memcpy(net->name, dvb->netname, 5);
#endif
        net->next      = NULL;
        net->init      = netcard_probe;
        net->priv      = kmalloc(sizeof(struct dvb_net_priv), GFP_KERNEL);
	if (net->priv == NULL)
			return -ENOMEM;
	memset(&((struct dvb_net_priv *)net->priv)->stats,
               0, sizeof(struct net_device_stats));

        ((struct dvb_net_priv *)net->priv)->dvb=dvb;
        getMAC(dvb);
                
	if ((result = register_netdev(net)) != 0)
		return result;
        return 0;
}

static void 
release_net(struct dvb_struct *dvb)
{
        kfree(dvb->net.priv);
        unregister_netdev(&dvb->net);
}


/*************************************************************************
  END NETWORK DRIVER
*************************************************************************/



/****************************************************************************
 * INITIALIZATION
 ****************************************************************************/

static int init_arm(struct dvb_struct *dvb)
{
        int ret;

        /* enable DEBI */
        saa7146_write(dvb->saa->mem, MC1, 0x08800880);
        saa7146_write(dvb->saa->mem, DD1_STREAM_B, 0x00000000);
        saa7146_write(dvb->saa->mem, MC2, (MASK_09 | MASK_25 | MASK_10 | MASK_26));
        
        ret=bootarm(dvb);
        if (ret<0)
                return ret;
        firmversion(dvb);

        return 0;
}

static int 
get_card_type(struct pci_dev *dev)
{
        //FIXME: change back to -1 when we are sure to know all subvendor device ids
        int type=DVB_CARD_TT_SIEMENS; 
        u16 subvendor, subdevice;

        pci_read_config_word(dev, PCI_SUBSYSTEM_VENDOR_ID, &subvendor);
        pci_read_config_word(dev, PCI_SUBSYSTEM_ID, &subdevice);
        
        if (subvendor==0x13c2) {
                switch (subdevice) {
                case 0x1003:
                        type=DVB_CARD_TT_BUDGET;
                        break;                  
                case 0x0000:
                case 0x0001:
                        type=DVB_CARD_TT_SIEMENS;
                        break;                  
                }
        }
        return type;
}


static int init_dvb(struct dvb_struct *dvb, int num, struct i2c_adapter *adap)
{
        int i;
        spin_lock_init (&dvb->debilock);
        sema_init(&dvb->dcomlock, 1);

        dvb->num=num;
        dvb->dvbtype=0;
        dvb->i2cbus=adap;
        dvb->saa=(struct saa7146 *) (adap->data);

        /* DVB decoder, tuner and sound decoder */
        dvb->tuner=0;
        dvb->decoder=0;
        dvb->sound=0;

        for (i=0; i<MAXFILT; i++) {
                dvb->filter[i].flags=0x00;
                dvb->filter[i].state=0x00;
                dvb->hfilter[i]=NULL;
        }

        dvb->osdwin=1;
        dvb->ci_slot[0]=dvb->ci_slot[1]=0;
        dvb->ci_stat=0;


        dvb->debitype=-1;

        if(vid_register(dvb) < 0)
                return -1;

        dvb->playing=0;
        dvb->recording=0;
        dvb->playmode=VID_PLAY_NORMAL;
        dvb->vidmode=VIDEO_MODE_PAL;

        /* allocate and init buffers */
        dvb->iobuf=vmalloc(AVINLEN+AVOUTLEN+SECBUFLEN+AOUTLEN+VOUTLEN);
        if (!dvb->iobuf)
                return -1;

        dvb_buf_init(&dvb->avin, dvb->iobuf, AVINLEN);
        dvb_buf_init(&dvb->avout, dvb->iobuf+AVINLEN, AVOUTLEN);
        dvb_buf_init(&dvb->fsec, dvb->iobuf+AVINLEN+AVOUTLEN, SECBUFLEN);

        dvb_buf_init(&dvb->aout, 
                     dvb->iobuf+AVINLEN+AVOUTLEN+SECBUFLEN, 
                     AOUTLEN);
        dvb_buf_init(&dvb->vout, 
                     dvb->iobuf+AVINLEN+AVOUTLEN+SECBUFLEN+AOUTLEN,
                     VOUTLEN);


        /* init BMP buffer */
        init_waitqueue_head(&dvb->bmpq);
        spin_lock_init (&(dvb->bmplock));
        dvb->bmpp=dvb->bmplen=0;
        dvb->bmpbuf=vmalloc(8*32768+1024); 
        init_waitqueue_head(&dvb->debiq);
        dvb->debilock=SPIN_LOCK_UNLOCKED;

        if (dvb->cardtype==DVB_CARD_TT_SIEMENS) 
                init_arm(dvb);

        if (dvb->cardtype==DVB_CARD_TT_BUDGET) {
                saa7146_write(dvb->saa->mem, DD1_INIT, 0x02000600);
                saa7146_write(dvb->saa->mem, MC2, 
                              (MASK_09 | MASK_25 | MASK_10 | MASK_26));
                //setgpio(dvb, 2, GPIO_OUTLO);
                  setgpio(dvb, 2, GPIO_OUTHI);
                  //setgpio(dvb, 1, GPIO_OUTLO);
                  //setgpio(dvb, 0, GPIO_OUTLO);
        }

        saa7146_write(dvb->saa->mem, ISR, (MASK_19 | MASK_03));
        saa7146_write(dvb->saa->mem, IER, 
                      saa7146_read(dvb->saa->mem, IER) | MASK_03 );
       
        if (init_net(dvb)<0)
                printk("dvb: init_net() failed\n");

        if(dvb_register(dvb) < 0)
                return -1;
        return 0;
}

/* This callback is called for every I2C adapter.
   We look for the SAA7146s, count and configure them
*/

static int attach_adapter(struct i2c_adapter *adap)
{
        struct dvb_struct *dvb=&dvbs[num_dvb];
        int cardtype;

        if (adap->id!=I2C_ALGO_SAA7146)
                return 0;
        
        cardtype=get_card_type(((struct saa7146 *) (adap->data))->device);
        if (cardtype<0)
                return 0;
          
        dvb->cardtype=cardtype;
        init_dvb(dvb, num_dvb, adap);
        num_dvb++;
        
        dprintk("dvb: attached adapter #%d\n", num_dvb);
        return 0;
}

static int 
detach_client(struct i2c_client *client)
{
        struct dvb_struct *dvb=0;
        int i;

	for( i = 0; i < num_dvb; i++) 
                if (dvbs[i].i2cbus==client->adapter) {
                        dvb=&dvbs[i];
                        break;
                }
        if (!dvb)
                return -1;

        dprintk("dvb: attached/detached client %s in dvb(%d)\n", client->name, i);
	
	if (client->driver->id==I2C_DRIVERID_MSP3400) {
	        dvb->sound = dvb->sound ? 0 : client;
                return 0;
        }
        
	if (client->driver->id==I2C_DRIVERID_TUNER)
                dvb->tuner = dvb->tuner ? 0 : client;
	
	/* Hmmm, there should not be more than 1 DVB decoder chip on 
           one board at the same time! */
	
	if (client->driver->id==I2C_DRIVERID_VES1893) {
                dvb->decoder = dvb->decoder ? 0 : client;
                dvb->dvbtype = DVB_S;
        }

	if (client->driver->id==I2C_DRIVERID_VES1820) {
	        dvb->decoder = dvb->decoder ? 0 : client;
                dvb->dvbtype = DVB_C;
        }
        

	if (dvb->tuner && dvb->decoder) {
                InitFront(dvb);
        }

	dprintk("dvb: decoder=%08x\n", (u32) dvb->decoder);
	dprintk("dvb: tuner=%08x\n", (u32) dvb->tuner);
	
	return 0;
}


static void 
inc_use (struct i2c_client *client)
{
#ifdef MODULE
        MOD_INC_USE_COUNT;
#endif
}

static void 
dec_use (struct i2c_client *client)
{
#ifdef MODULE
        MOD_DEC_USE_COUNT;
#endif
}

static struct i2c_driver dvb_driver = {
  /* name */            "dvbi2c",
  /* id */              0x1234,
  /* flags */           I2C_DF_DUMMY,
  /* attach_adapter */  attach_adapter,
  /* detach_client */   detach_client,
  /* command */         0,
  /* inc_use */         inc_use,
  /* dec_use */         dec_use,
};

static void dvb_irq(struct saa7146 *saa, u32 isr, void *data) 
{
        dprintk("dvb_irq\n");
        if (isr&MASK_19)
                debiirq(saa, data);
        if (isr&MASK_03)
                gpioirq(saa, data);
        if (isr&MASK_10)
                vpeirq(saa, data);
        if (isr&MASK_07)
                fidbirq(saa, data);
}

static int dvb_command(struct saa7146* saa, void *p, unsigned int cmd, void *arg)
{
        switch(cmd) {
        case SAA7146_SUSPEND:
                printk("dvb_suspend()\n");
                break;
        case SAA7146_RESUME:
                printk("dvb_resume()\n");
                break;
        default:
                return -ENOIOCTLCMD;
        }
        return 0;
}

static int dvb_extnum=0;

static int dvb_attach(struct saa7146* adap, void** p)
{
        if (dvb_extnum==MAX_NUM_DVB)
                return -1;
	*(struct dvb_struct**)p = &dvbs[dvb_extnum++];
        return 0;
}

static int dvb_detach(struct saa7146* adap, void** p)
{
        if (dvb_extnum==0)
                return -1;
        
        saa7146_write(adap->mem, IER, 
                      saa7146_read(adap->mem, IER) & 
                      ~(MASK_19|MASK_03));

        dvb_extnum--;
	*(struct dvb_struct**)p = 0;
        return 0;
}

static void dvb_inc_use(struct saa7146* adap)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
}

static void dvb_dec_use(struct saa7146* adap)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
}


static struct saa7146_extension dvb_extension = {
	"dvb extension\0",
	MASK_07|MASK_10|MASK_19|MASK_03|MASK_27,  /* handles debi and gpio0 irqs */
	dvb_irq,
	dvb_command,
	dvb_attach,
	dvb_detach,
	dvb_inc_use,
	dvb_dec_use
};	


#ifdef MODULE
int init_module(void) {
#else
int init_dvb(struct video_init *unused) 
{
#endif
	int result = 0;
	
	num_dvb = 0;

	dprintk("dvb: init_dvb\n");

	/* We only register a dummy driver to get all the adapters listed and 
           initialized 
	   */
 
	if ((result = i2c_add_driver(&dvb_driver))) 
	{
	        printk(KERN_ERR "dvb: I2C driver registration failed!\n");
		return result;
	}
        dprintk("dvb: i2c driver added\n");

        if ( 0 == num_dvb ) 
        {
	  printk(KERN_ERR "dvb: no dvb(s) found!\n");
	  return -ENODEV;
	}
        dprintk("dvb: dvbdevice found\n");

	if( 0 != (result = saa7146_add_extension(&dvb_extension))) {
		printk("dvb: extension registration failed, module not inserted.\n");
		return result;
	}
        printk(KERN_ERR "dvb: %d dvb(s) found!\n", num_dvb);
        return 0;
}

#ifdef MODULE
void cleanup_module(void)
{
	int i;

	fprintk("dvb: ==> cleanup_module\n");

	/* release the saa7146s properly */
	for( i = 0; i < num_dvb; i++) {
                SetVoltage(&dvbs[i],0,0);
                saa7146_write(dvbs[i].saa->mem, IER, 
                              saa7146_read(dvbs[i].saa->mem, IER) & 
                              ~(MASK_19 | MASK_03));
                saa7146_write(dvbs[i].saa->mem, ISR,(MASK_19 | MASK_03));
                
                vfree(dvbs[i].iobuf);
                vfree(dvbs[i].bmpbuf);
                dvb_unregister(&dvbs[i]);
                vid_unregister(&dvbs[i]);
                release_net(&dvbs[i]);
        }
        
        if ( saa7146_del_extension(&dvb_extension)) 
                printk(KERN_ERR "dvb: extension deregistration failed.\n");

	if (i2c_del_driver(&dvb_driver))
	        printk(KERN_ERR "dvb: i2c driver deregistration failed.\n");

	printk(KERN_ERR "dvb: %d dvb(s) released properly.\n", num_dvb);
}
#endif

/*
 * Local variables:
 * c-indent-level: 8
 * c-brace-imaginary-offset: 0
 * c-brace-offset: -8
 * c-argdecl-indent: 8
 * c-label-offset: -8
 * c-continued-statement-offset: 8
 * c-continued-brace-offset: 0
 * indent-tabs-mode: nil
 * tab-width: 8
 * End:
 */
